package br.com.eselo.component;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.commons.mail.DefaultAuthenticator;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.caelum.vraptor.ioc.Component;
import br.com.caelum.vraptor.ioc.SessionScoped;

@Component
@SessionScoped
public class Mailer {

	private static final Logger LOGGER = LoggerFactory.getLogger(Mailer.class);
	private static final String EMAIL_LOG_TEMPLATE = "Sending message \"%s\" from %s to %s using server %s:%s (using TLS: %b)";

	private static final String DEFAULT_TO_PROPERTIES = "eselo.mail.default_to";
	private static final String FROM = "eselo.mail.from";
	private static final String FROM_NAME = "eselo.mail.from.name";
	private static final String SERVER = "eselo.mail.server";
	private static final String PORT = "eselo.mail.port";
	private static final String TLS = "eselo.mail.tls";
	private static final String SSL = "eselo.mail.ssl";
	private static final String USERNAME = "eselo.mail.username";
	private static final String PASSWORD = "eselo.mail.password";
	private static final String REPLY_TO = "eselo.mail.replyTo";
	private static final String DEBUG = "eselo.mail.debug";

	private final Properties properties = new Properties();

	public Mailer() throws IOException {
		loadAndPut();
	}

	@SuppressWarnings("deprecation")
	public void send(Email email) throws EmailException {
		if (email.getFromAddress() == null) {
			email.setFrom(properties.getProperty(FROM),
					properties.getProperty(FROM_NAME));
		}

		email.setHostName(properties.getProperty(SERVER));
		email.setSmtpPort(Integer.parseInt(properties.getProperty(PORT)));

		boolean tls = Boolean.parseBoolean(properties.getProperty(TLS));
		boolean ssl = Boolean.parseBoolean(properties.getProperty(SSL));
		
		email.setTLS(tls);
		email.setSSL(ssl);

		if (tls) {
			email.setAuthenticator(new DefaultAuthenticator(properties
					.getProperty(USERNAME), properties.getProperty(PASSWORD)));
		}

		if (properties.containsKey(REPLY_TO)) {
			String replyTo = properties.getProperty(REPLY_TO);
			email.addReplyTo(replyTo);
		}
		
		if (properties.containsKey(DEFAULT_TO_PROPERTIES)) {
			email.getToAddresses().clear();
			email.addTo(properties.getProperty(DEFAULT_TO_PROPERTIES));
			email.getBccAddresses().clear();
			email.getCcAddresses().clear();
		}
		
		boolean debug = Boolean.parseBoolean(properties.getProperty(DEBUG));
		email.setDebug(debug);
		
		wrapUpAndSend(email);
	}

	@SuppressWarnings("deprecation")
	private void wrapUpAndSend(Email email) throws EmailException {
		LOGGER.debug(String.format(emailLogTemplate(),
				email.getSubject(),
				email.getFromAddress(),
				email.getToAddresses(),
				email.getHostName(),
				email.getSmtpPort(),
				email.isTLS()));
		email.send();
	}
	
	private String emailLogTemplate() {
		return EMAIL_LOG_TEMPLATE;
	}

	private void loadAndPut() throws IOException {
		String fileName = "mail.properties";
		InputStream stream = Mailer.class.getResourceAsStream("/" + fileName);
		Properties propertiesFile = new Properties();

		if (stream != null) {
			propertiesFile.load(stream);
			this.properties.putAll(propertiesFile);
		} else {
			LOGGER.warn("Could not find the file '{}' to load.", fileName);
		}
	}

}
