package br.com.eselo.conexaoAsesWeb;

import java.util.ArrayList;
import java.util.List;

import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.Resultado;

import com.google.gson.Gson;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;

public class LeitorXML {
	
	public AvaliationReport lerAvaliacaoXML(String fonte){
		
XStream xstream = new XStream(new DomDriver("UTF-8", new XmlFriendlyReplacer("__", "_")));  
		
		xstream.alias("relatorio_de_avaliacao", AvaliationReport.class);
		xstream.alias("recomendacao", CheckPoint.class);
		xstream.alias("criterio", Criterio.class);

		xstream.aliasField("resultados", AvaliationReport.class, "result");

		xstream.aliasField("data", AvaliationReport.class, "date");
		xstream.addImplicitCollection(AvaliationReport.class, "checkPoints");

		xstream.aliasField("criterio", CheckPoint.class, "criterio");
		xstream.aliasField("descricao", Criterio.class, "description");
		xstream.aliasField("qtde_erros", Criterio.class, "qtdeErrors");		
				
		xstream.aliasField("identificador", CheckPoint.class, "identificador");
		xstream.aliasField("total_erros", CheckPoint.class, "totalErrors");
		xstream.aliasField("total_avisos", CheckPoint.class, "totalWarnings");
		
		return (AvaliationReport) xstream.fromXML(fonte);
	}
	
	public List<Resultado> lerResultadosXML(String xml, Avaliacao avaliacao){
		List<CheckPoint> cps = ((AvaliationReport)lerAvaliacaoXML(xml)).getCheckPoints();
		return lerResultadosXML(cps, avaliacao);
	}
	
	public List<Resultado> lerResultadosXML(List<CheckPoint> cps, Avaliacao avaliacao){

		Resultado r;
		List<Resultado> resultados = new ArrayList<Resultado>();
		
		for(CheckPoint cp:cps){
			r = new Resultado();
			r.setAvaliacao(avaliacao);
			r.setNumDiretriz(cp.getIdentificador());
			r.setNumAvisos(cp.getTotalWarnings());
			r.setNumErros(cp.getTotalErrors());
			
			resultados.add(r);
		}
		
		return resultados;
	}
	
	public String toJson(AvaliationReport ar){
		return new Gson().toJson(ar);
	}
}
