package br.com.eselo.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.view.Results;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.conexaoAsesWeb.AvaliationReport;
import br.com.eselo.conexaoAsesWeb.CheckPoint;
import br.com.eselo.conexaoAsesWeb.LeitorXML;
import br.com.eselo.conexaoAsesWeb.ResultadoAvaliacao;
import br.com.eselo.dao.AvaliacaoDao;
import br.com.eselo.dao.ConformidadeDao;
import br.com.eselo.dao.DiretrizDao;
import br.com.eselo.dao.ParametroAvaliacaoDao;
import br.com.eselo.dao.ResultadoDao;
import br.com.eselo.dao.UrlDao;
import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.Conformidade;
import br.com.eselo.model.Diretriz;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.Resultado;
import br.com.eselo.model.Url;

@Resource
public class AvaliacaoController {

	private final AvaliacaoDao dao;
	private final Avaliacao avaliacao;
	private final Result result;
	private final UrlDao daoUrl;
	private final ResultadoDao resultDao;
	private final ParametroAvaliacaoDao paramDao;
	private final UsuarioWeb usuarioWeb;


	public AvaliacaoController(AvaliacaoDao _dao, Avaliacao _avaliacao,
			Result _result, UrlDao _daoUrl, ResultadoDao _resultDao, ParametroAvaliacaoDao _paramDao,
			UsuarioWeb _usuarioWeb,Validator _validator) {
		dao = _dao;
		avaliacao = _avaliacao;
		daoUrl = _daoUrl;
		result = _result;
		resultDao = _resultDao;
		paramDao = _paramDao;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/avaliacoes")
	@Restrito(descricao = "AVALIACOES_LISTA")
	public List<Avaliacao> index() {
		return this.index(1);
	}

	@Get("/avaliacoes/{pagina}")
	@Restrito(descricao = "AVALIACOES_LISTA")
	public List<Avaliacao> index(int pagina) {
		List<Avaliacao> resultado = dao.listAll();

		// Paginação
		// quantidade de registros por pagina
		int exibi = 7;
		// Informa a quantidade de registros
		int registros = resultado.size();
		// Quantidade de paginas na paginação
		int totalPag = registros / exibi;
		// se houver resto na divisao acima acrescenta uma pagina na paginação
		if ((registros - (totalPag * exibi)) > 0)
			totalPag++;

		// Calculo do registro inicial e final
		int regIni = ((exibi * pagina) - exibi);
		int regFim = (regIni + exibi);

		if (regFim > registros)
			regFim = registros;

		// fim paginação

		List<Avaliacao> paginacao = new ArrayList<>();

		for (int i = regIni; i < regFim; i++) {
			paginacao.add(resultado.get(i));
		}

		result.include("paginas", totalPag);
		result.include("registros", registros);
		result.include("regInic", ++regIni);
		result.include("regFim", regFim);

		return paginacao;
	}

	@Post("/avaliacoes/pesquisa")
	@Restrito(descricao = "AVALIACOES_LISTA")
	public List<Avaliacao> lista(String nUrl, int paginacao) {
		result.include("paginacao", paginacao);
		result.include("nUrl", nUrl);
		return dao.listFiltro(nUrl, usuarioWeb.getUsuario().getId());
	}

	public void form() {

	}

	@Get("/avaliacoes/avalia/{idUrl}")
	@Restrito(descricao = "AVALIACOES_AVALIACAO")
	public void avaliacao(Long idUrl){

		java.sql.Timestamp data = new java.sql.Timestamp(new java.util.Date().getTime());

		Url dadosUrl = (Url) this.daoUrl.load(idUrl);

		Long idParametro = this.paramDao.getMaxId();

		ParametroAvaliacao dadosParam = this.paramDao.load(idParametro);
		dadosParam.setId(idParametro);

		dadosUrl.setId(idUrl);

		this.avaliacao.setIdUrl(dadosUrl);
		this.avaliacao.setParametroAvaliacao(dadosParam);
		this.avaliacao.setDataAvaliacao(data);
		dao.save(this.avaliacao);
		Long idAvaliacao = this.avaliacao.getId();

		result.redirectTo(AvaliacaoController.class).resultado(idAvaliacao);
	}

	@Get("/avaliacoes/resultado/{idAval}")
	public List<Resultado> resultado(Long idAval) {

		Avaliacao aval = dao.getAvaliacao(idAval);

		//Obtendo um xml de teste
		String xml =  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"+
				"<relatorio_de_avaliacao>"+
				"		<url>www.bol.com.br</url>"+
				"		<data>2014-04-24 10:07:02.447 GMT-03:00</data>"+
				"		<recomendacao>"+
				"			<criterio>"+
				"				<descricao>51</descricao>"+
				"				<qtde_erros>1</qtde_erros>"+
				"			</criterio>"+
				"			<identificador>1</identificador>"+
				"			<total_erros>11</total_erros>"+
				"			<total_avisos>50</total_avisos>"+
				"		</recomendacao>" +
				"		<recomendacao>"+
				"			<criterio>"+
				"				<descricao>31</descricao>"+
				"				<qtde_erros>2</qtde_erros>"+
				"			</criterio>"+
				"			<identificador>2</identificador>"+
				"			<total_erros>13</total_erros>"+
				"			<total_avisos>30</total_avisos>"+
				"		</recomendacao>" +
				"</relatorio_de_avaliacao>";

		//Receber o xml
		List<Resultado> resultadosXML = new LeitorXML().lerResultadosXML(xml, aval);

		//obter os resultados
		List<Resultado> resultadosBD;
		ResultadoDao rdao = new ResultadoDao(dao.getSession());
		resultadosBD = rdao.listResultado(idAval);

		for(Resultado rxml:resultadosXML){
			int check=0;
			for(Resultado rbd:resultadosBD){
				if(rxml.getNumDiretriz().equals(rbd.getNumDiretriz()) && rxml.getAvaliacao().getId().equals(idAval)){
					check = check+1;
				}
			}
			if(check ==0 ){
				rdao.save(rxml);
				System.out.println("Salvei resultado da Diretriz "+rxml.getNumDiretriz());
			}else{
				System.err.println("Nada salvo para Diretriz "+rxml.getNumDiretriz());
			}
		}

		//Avaliar
		String mensagem ="Avaliação realizada com sucesso. [Nota "+avaliar(idAval)+"]";
		java.util.Date dataAval = aval.getDataAvaliacao();
		result.include("mensagem", mensagem);
		result.include("idAval", aval.getId());
		result.include("nomeUrl", aval.getIdUrl().getUrl());
		result.include("dataAval", dataAval);
		return resultDao.listResultado(idAval);
	}

	private Double avaliar(long idAvaliacao){
		Double nota=0.0;
		Avaliacao avaliacao = new AvaliacaoDao(dao.getSession()).getAvaliacao(idAvaliacao);
		List<Diretriz> diretrizesValidas;
		List<Resultado> resultadosObtidos;
		int somaDosPesos=0;


		//obter os resultados
		resultadosObtidos = new ResultadoDao(dao.getSession()).listResultado(idAvaliacao);

		//obter as diretrizes e níveis de conformidade
		diretrizesValidas = new DiretrizDao(dao.getSession()).getByParametroAvaliacao(avaliacao.getParametroAvaliacao());

		//Comparando
		for (Diretriz diretriz: diretrizesValidas){
			for (Resultado resultado : resultadosObtidos) {
				if(resultado.getNumDiretriz().equals(diretriz.getNumeroDiretriz())){
					List<Conformidade> conformidades = new ConformidadeDao(dao.getSession()).getByDiretriz(diretriz);
					for (Conformidade conformidade: conformidades){
						if (resultado.getNumErros() >= conformidade.getFaixaInicio() && resultado.getNumErros() <= conformidade.getFaixaFim()){
							nota = nota+(conformidade.getPercentualConformidade()*diretriz.getPeso());
							somaDosPesos = somaDosPesos+diretriz.getPeso();
						}
					}
				}
			}
		}


		nota = 100*(nota/somaDosPesos);

		ResultadoAvaliacao ra = new ResultadoAvaliacao();
		ra.setData(new Date());
		ra.setUrl(avaliacao.getIdUrl().getUrl());
		ra.setNota(nota);

		return nota;
	}

	 @Get("/avaliar")
	 public void avaliarURL(String url){
		
		 Url site = new UrlDao(dao.getSession()).getStringUrl(url);
		 Long idAval = dao.getAvaliacaoByURL(site);
		 
		 result.redirectTo(AvaliacaoController.class).resultado(idAval);
	 }
	
	@Post("/calcular-nota")
	public void calcularNota(AvaliationReport avaliationReport) {

		ResultadoAvaliacao ra = new ResultadoAvaliacao();
		
		Long idAval=null;
		Url url = new Url();
		List<CheckPoint> recomendacoes;
	
			if(avaliationReport.getCheckPoints()!=null && avaliationReport.getDate()!=null && avaliationReport.getUrl()!=null){
				url.setUrl(avaliationReport.getUrl());
				daoUrl.save(url);

				recomendacoes = avaliationReport.getCheckPoints();

				ParametroAvaliacao parametroAvaliacao =  new ParametroAvaliacaoDao(dao.getSession()).getLastParametrosAvaliacao();

				Avaliacao avaliacao = new Avaliacao();
				avaliacao.setDataAvaliacao(avaliationReport.getDate());
				avaliacao.setIdUrl(url);
				avaliacao.setParametroAvaliacao(parametroAvaliacao);

				dao.save(avaliacao);
				idAval =avaliacao.getId(); 

				LeitorXML leitor = new LeitorXML();
				List<Resultado> resultadosNovaAvaliacao = leitor.lerResultadosXML(recomendacoes, avaliacao);

				ResultadoDao rdao = new ResultadoDao(dao.getSession());
				List<Resultado> resultadosCadastrados = rdao.listResultado(idAval);

				for(Resultado rNovaAvaliacao:resultadosNovaAvaliacao){
					int check=0;
					for(Resultado rbd:resultadosCadastrados){
						if(rNovaAvaliacao.getNumDiretriz().equals(rbd.getNumDiretriz()) && rNovaAvaliacao.getAvaliacao().getId().equals(idAval)){
							check = check+1;
						}
					}
					if(check ==0 ){
						rdao.save(rNovaAvaliacao);
						System.out.println("Salvei resultado da Diretriz "+rNovaAvaliacao.getNumDiretriz());
					}else{
						System.err.println("Nada salvo para Diretriz "+rNovaAvaliacao.getNumDiretriz());
					}
				}

				ra.setNota(avaliar(idAval));
				ra.setUrl(url.getUrl());
				ra.setData(new Date());
				ra.setObservacoes("Avaliação realizada com sucesso!");

			}else{
				ra.setNota(null);
				ra.setUrl(url.getUrl());
				ra.setData(avaliationReport.getDate());
				ra.setObservacoes("Foi impossivel realizar a avaliação.");
			}

			result.use(Results.json()).withoutRoot().from(ra).serialize();
	}
}
