package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.dao.GrupoDao;
import br.com.eselo.model.Grupo;

@Resource
public class GruposController {

	private final GrupoDao dao;
	private final Result result;
	private final Validator validator;

	public GruposController(GrupoDao _dao, Result _result, Validator _validator) {
		dao = _dao;
		result = _result;
		validator = _validator;
	}

	@Get("/grupos")
	@Restrito(descricao = "GRUPOS_LISTA")
	public List<Grupo> lista() {
		return dao.listAll();
	}

	@Restrito(descricao = "GRUPOS_ADICIONAR")
	public void form() {
		result.include("permissaoList", dao.getAllPermissoes());
	}

	@Get("/grupos/{id}")
	@Restrito(descricao = "GRUPOS_ATUALIZAR")
	public Grupo form(Long id) {
		Grupo grupo = this.dao.load(id);
		result.include("permissaoList", dao.getAllPermissoes(grupo.getPermissoes()));
		return grupo;
	}

	@Post("/grupos")
	@Restrito(descricao = "GRUPOS_ADICIONAR")
	public void adicionar(Grupo grupo) {
		Long idGrupoMesmoNome = dao.getIdByNome(grupo.getNome());

		if (grupo.getNome() == null) {
			validator.add(new ValidationMessage("O nome do grupo é obrigatório.", "grupo.nome"));
		} else if (idGrupoMesmoNome != null) {
			validator.add(new ValidationMessage("O grupo não pode ser igual a um já existente.",
					"grupo.nome"));
		}
		validator.onErrorUsePageOf(GruposController.class).form();

		dao.save(grupo);

		result.include("mensagem", "Grupo salvo com sucesso.");
		result.redirectTo(GruposController.class).lista();
	}

	@Put("/grupos")
	@Restrito(descricao = "GRUPOS_ATUALIZAR")
	public void atualizar(Grupo grupo) {
		Grupo grupoCadastrado = dao.load(grupo.getId());
		grupoCadastrado.setNome(grupo.getNome());
		grupoCadastrado.setPermissoes(grupo.getPermissoes());

		Long idGrupoMesmoNome = dao.getIdByNome(grupo.getNome(), grupo.getId());

		if (grupo.getNome() == null) {
			validator.add(new ValidationMessage("O nome do grupo é obrigatório.", "grupo.nome"));
		} else if (idGrupoMesmoNome != null && !idGrupoMesmoNome.equals(grupo.getId())) {
			validator.add(new ValidationMessage("O grupo não pode ser igual a um já existente.",
					"grupo.nome"));
		}
		validator.onErrorUsePageOf(GruposController.class).form();

		dao.saveOrUpdate(grupoCadastrado);

		result.include("mensagem", "Grupo alterado com sucesso.");
		result.redirectTo(GruposController.class).lista();
	}

	@Path("/grupos/delete/{id}")
	@Restrito(descricao = "GRUPOS_EXCLUIR")
	public void excluir(Long id) {
		Grupo grupo = dao.load(id);

		if (!grupo.getUsuarios().isEmpty()) {
			validator.add(new ValidationMessage(
					"O grupo está sendo referenciado e não pode ser excluído.", "grupo.nome"));
		}
		validator.onErrorForwardTo(GruposController.class).lista();

		dao.remove(id);

		result.include("mensagem", "Grupo excluído com sucesso.");
		result.redirectTo(GruposController.class).lista();
	}

}
