package br.com.eselo.controller;

import java.util.ArrayList;
import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.AvaliacaoDao;
import br.com.eselo.dao.DiretrizDao;
import br.com.eselo.dao.ParametroAvaliacaoDao;
import br.com.eselo.dao.UsuarioDao;
import br.com.eselo.model.Conformidade;
import br.com.eselo.model.Diretriz;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.Usuario;

/**
 * 
 * @author Magno
 * @version 1.0
 */
@Resource
public class ParametroAvaliacaoController {

	private final ParametroAvaliacaoDao dao;
	private final DiretrizDao daoDiretriz;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public ParametroAvaliacaoController(ParametroAvaliacaoDao _dao,
			DiretrizDao _daoDiretriz, Result _result, Validator _validator,
			UsuarioWeb _usuarioWeb) {
		dao = _dao;
		daoDiretriz = _daoDiretriz;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/parametrosAvaliacao/novo")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_ADICIONAR")
	public void novo() {
		ParametroAvaliacao parametroAvaliacao = new ParametroAvaliacao();
		parametroAvaliacao.setUsuario(dao.getUsuario(usuarioWeb.getId()));
		parametroAvaliacao.setTsInicio(new java.util.Date(System
				.currentTimeMillis()));
		inserirParametroAvaliacao(parametroAvaliacao);

		result.redirectTo(ParametroAvaliacaoController.class).form(
				dao.getMaxId());
	}

	@Get("/parametrosAvaliacao/importar")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_ADICIONAR")
	public void importar() {
		criarNovoParametroAvaliacao();

		result.redirectTo(ParametroAvaliacaoController.class).form(
				dao.getMaxId());
	}

	@Post("/parametrosAvaliacao")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_ADICIONAR")
	public void adicionar(ParametroAvaliacao parametroAvaliacao) {
		if (parametroAvaliacao.getTsInicio() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o início.",
					"parametroAvaliacao.tsInicio"));
		} else if (parametroAvaliacao.getTsFim() != null) {
			if (parametroAvaliacao.getTsFim().before(
					parametroAvaliacao.getTsInicio())) {
				validator
						.add(new ValidationMessage(
								"O campo \"Fim\" deve ser maior que o campo \"Início\".",
								"parametroAvaliacao.tsFim"));
			}
		} else if (parametroAvaliacao.getUsuario() == null) {
			validator.add(new ValidationMessage(
					"É obrigatório informar o usuário.",
					"parametroAvaliacao.usuario"));
		}
		validator.onErrorUsePageOf(ParametroAvaliacaoController.class).form();

		inserirParametroAvaliacao(parametroAvaliacao);

		result.include("mensagem", "Parâmetro de Avaliação salvo com sucesso.");
		result.redirectTo(ParametroAvaliacaoController.class).index(1);
	}

	protected void inserirParametroAvaliacao(
			ParametroAvaliacao parametroAvaliacao) {
		ParametroAvaliacao ultimo = dao.load(dao.getMaxId());

		if (ultimo != null) {
			ultimo.setTsFim(parametroAvaliacao.getTsInicio());
			dao.update(ultimo);
		}

		dao.save(parametroAvaliacao);
	}

	@Put("/parametrosAvaliacao")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_ATUALIZAR")
	public void atualizar(ParametroAvaliacao parametroAvaliacao) {

		ParametroAvaliacao parametroAvaliacaoCadastrado = dao.load(parametroAvaliacao.getId());
		parametroAvaliacaoCadastrado.setTsFim(parametroAvaliacao.getTsFim());
		parametroAvaliacaoCadastrado.setUsuario(parametroAvaliacao.getUsuario());

		if (parametroAvaliacao.getTsFim() != null) {
			if (parametroAvaliacao.getTsFim().before(
					parametroAvaliacao.getTsInicio())) {
				validator
						.add(new ValidationMessage(
								"O campo \"Fim\" deve ser maior que o campo \"Início\"!",
								"parametroAvaliacao.tsFim"));
			}
		}

		validator.onErrorUsePageOf(ParametroAvaliacaoController.class).form();

		dao.saveOrUpdate(parametroAvaliacaoCadastrado);

		result.include("mensagem",
				"Parâmetro de Avaliação alterado com sucesso.");
		result.redirectTo(ParametroAvaliacaoController.class).index(1);
	}

	@Path("/parametrosAvaliacao/delete/{id}")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_EXCLUIR")
	public void excluir(Long id) {
		
		ParametroAvaliacao pa = dao.load(id);
		if((pa.getTsFim()==null)){
			if(new AvaliacaoDao(dao.getSession()).existeAvaliacao(pa)){
				result.include("mensagem", "Parametro de Avaliao no pode ser excludo pois j foi utilizado em uma avaliao.");
			}else{
				dao.remove(id);
				
				pa = dao.getLastParametrosAvaliacao();
				pa.setTsFim(null);
				dao.update(pa);
				result.include("mensagem", "Parametro de Avaliao excludo com sucesso.");
			}
		}else{
			result.include("mensagem", "Parametro de Avaliao no pode ser excludo pois no est ativo.");
		}
		
		result.redirectTo(ParametroAvaliacaoController.class).lista();
	}
	
	@Get("/parametrosAvaliacao")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_LISTA")
	public List<ParametroAvaliacao> lista() {
		return this.index(1);
	}

	@Get("/parametrosAvaliacao/{pagina}")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_LISTA")
	public List<ParametroAvaliacao> index(int pagina) {
		ParametroAvaliacao ultimoParametro = dao.load(dao.getMaxId());

		// Paginação
		// quantidade de registros por pagina
		int exibi = 7;
		// Informa a quantidade de registros
		int registros = dao.numLinhas();
		// Quantidade de paginas na paginação
		int totalPag = registros / exibi;
		// se houver resto na divisao acima acrescenta uma pagina na paginação
		if ((registros - (totalPag * exibi)) > 0)
			totalPag++;

		// Calculo do registro inicial e final
		int regIni = ((exibi * pagina) - exibi);
		int regFim = (regIni + exibi);

		if (regFim > registros)
			regFim = registros;

		// fim paginação

		List<ParametroAvaliacao> resultado = dao.listAll(regIni, regFim);
		List<ParametroAvaliacao> paginacao = new ArrayList<>();

		for (int i = regIni; i < regFim; i++) {
			paginacao.add(resultado.get(i));
		}

		result.include("paginas", totalPag);
		result.include("registros", registros);
		result.include("regInic", ++regIni);
		result.include("regFim", regFim);
		result.include("controle", "parametrosAvaliacao");
		result.include("ultimoParametro", ultimoParametro);

		return paginacao;
	}

	@Get("/parametrosAvaliacao/atualizar/{id}")
	@Restrito(descricao = "PARAMETRO_AVALIACAO_ATUALIZAR")
	public ParametroAvaliacao form(Long id) {
		ParametroAvaliacao parametroAvaliacao = this.dao.load(id);
		result.include("usuarioList", dao.getAllUsuarios());
		return parametroAvaliacao;
	}

	@Restrito(descricao = "PARAMETRO_AVALIACAO_ADICIONAR")
	public void form() {
		ParametroAvaliacao parametroAvaliacao = new ParametroAvaliacao();
		parametroAvaliacao.setUsuario(dao.getUsuario(usuarioWeb.getId()));
		parametroAvaliacao.setTsInicio(new java.util.Date(System
				.currentTimeMillis()));
		result.include("parametroAvaliacao", parametroAvaliacao);
	}

	private void criarNovoParametroAvaliacao() {
		ParametroAvaliacao ultimoParametroAvaliacao = dao
				.getLastParametrosAvaliacao();

		UsuarioDao uDao = new UsuarioDao(dao.getSession());
		Usuario usuario = uDao.load(usuarioWeb.getId());

		// Cria o parametro
		ParametroAvaliacao parametroAvaliacao = new ParametroAvaliacao();
		parametroAvaliacao.setUsuario(usuario);
		parametroAvaliacao.setTsInicio(new java.util.Date(System
				.currentTimeMillis()));
		parametroAvaliacao.setTsFim(null);

		ultimoParametroAvaliacao.setTsFim(parametroAvaliacao.getTsInicio());

		ParametroAvaliacaoDao paDao = new ParametroAvaliacaoDao(
				dao.getSession());

		// Cria a lista das diretrizes
		List<Diretriz> listaCadastrada = daoDiretriz
				.diretrizesDoUltimoParametro(ultimoParametroAvaliacao);

		// Salva novo e atualiza o ultimo
		paDao.save(parametroAvaliacao);
		paDao.update(ultimoParametroAvaliacao);

		// Salva as diretrizes para nova avaliação
		Diretriz novaDiretriz;
		Conformidade conformidade;
		for (Diretriz diretrizCadastrada : listaCadastrada) {
			novaDiretriz = new Diretriz();
			novaDiretriz.setParametroAvaliacao(parametroAvaliacao);
			novaDiretriz.setDescricao(diretrizCadastrada.getDescricao());
			novaDiretriz.setPeso(diretrizCadastrada.getPeso());
			novaDiretriz.setNumeroDiretriz(diretrizCadastrada
					.getNumeroDiretriz());
			novaDiretriz.setTipoDiretriz(diretrizCadastrada.getTipoDiretriz());
			novaDiretriz.setConformidades(new ArrayList<Conformidade>());

			if (diretrizCadastrada.getConformidades().size() > 0) {
				for (Conformidade conformidadeCadastrada : diretrizCadastrada
						.getConformidades()) {
					conformidade = new Conformidade();
					conformidade.setFaixaInicio(conformidadeCadastrada
							.getFaixaInicio());
					conformidade.setFaixaFim(conformidadeCadastrada
							.getFaixaFim());
					conformidade.setDiretriz(novaDiretriz);

					novaDiretriz.getConformidades().add(conformidade);
				}
			}

			daoDiretriz.save(novaDiretriz);
		}
	}

}
