package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.Messages;
import br.com.eselo.dao.TipoDiretrizDao;
import br.com.eselo.model.TipoDiretriz;

@Resource
public class TiposDiretrizController {

	private final TipoDiretrizDao dao;
	private final Result result;
	private final Validator validator;

	public TiposDiretrizController(TipoDiretrizDao _dao, Result _result,
			Validator _validator) {
		dao = _dao;
		result = _result;
		validator = _validator;
	}

	@Get("/tiposDiretriz")
	@Restrito(descricao = "TIPOS_CONFORMIDADE_LISTA")
	public List<TipoDiretriz> lista() {
		return dao.listAll();
	}

	@Restrito(descricao = "TIPOS_CONFORMIDADE_ADICIONAR")
	public void form() {
	}

	@Get("/tiposDiretriz/{id}")
	@Restrito(descricao = "TIPOS_CONFORMIDADE_ATUALIZAR")
	public TipoDiretriz form(Long id) {
		TipoDiretriz tipoDiretriz = dao.load(id);

		if (!tipoDiretriz.getDiretrizes().isEmpty()) {
			result.include("mensagem",
					Messages.getString("tipo_conformidade.validacao.edit"));
			result.redirectTo(TiposDiretrizController.class).lista();
		}

		return tipoDiretriz;
	}

	@Post("/tiposDiretriz")
	@Restrito(descricao = "TIPOS_CONFORMIDADE_ADICIONAR")
	public void adicionar(TipoDiretriz tipoDiretriz) {
		validarTipoConformidade(tipoDiretriz);

		validator.onErrorUsePageOf(TiposDiretrizController.class).form();

		dao.save(tipoDiretriz);

		result.include("mensagem",
				Messages.getString("tipo_conformidade.mensagem.salva"));
		result.redirectTo(TiposDiretrizController.class).lista();
	}

	@Put("/tiposDiretriz")
	@Restrito(descricao = "TIPOS_CONFORMIDADE_ATUALIZAR")
	public void atualizar(TipoDiretriz tipoDiretriz) {
		TipoDiretriz tipoDiretrizCadastrado = dao.load(tipoDiretriz.getId());

		validarTipoConformidade(tipoDiretriz);

		validator.onErrorUsePageOf(TiposDiretrizController.class).form(
				tipoDiretriz.getId());

		tipoDiretrizCadastrado.setDescricao(tipoDiretriz.getDescricao());
		tipoDiretrizCadastrado.setMaximoConformidade(tipoDiretriz
				.getMaximoConformidade());

		dao.saveOrUpdate(tipoDiretrizCadastrado);

		result.include("mensagem",
				Messages.getString("tipo_conformidade.mensagem.alterada"));
		result.redirectTo(TiposDiretrizController.class).lista();
	}

	@Path("/tiposDiretriz/delete/{id}")
	@Restrito(descricao = "TIPOS_CONFORMIDADE_EXCLUIR")
	public void excluir(Long id) {
		TipoDiretriz tipoDiretriz = dao.load(id);

		if (!tipoDiretriz.getDiretrizes().isEmpty()) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_conformidade.validacao.remove"),
					"tipoConformidade.tipoConformidade"));
		}
		validator.onErrorForwardTo(TiposDiretrizController.class).lista();

		dao.remove(id);

		result.include("mensagem", "Tipo de Diretriz excluído com sucesso.");
		result.redirectTo(TiposDiretrizController.class).lista();
	}

	private void validarTipoConformidade(TipoDiretriz tipoDiretriz) {
		if (tipoDiretriz.getDescricao() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_conformidade.validacao.descricao"),
					"tipoDiretriz.descricao"));
		} else if (dao.existeTipo(tipoDiretriz)) {
			Long id = dao.getIdByDescricao(tipoDiretriz.getDescricao());

			if ((tipoDiretriz.getId() == null && id != null)
					|| (tipoDiretriz.getId() != null && tipoDiretriz.getId() == id)) {
				validator
						.add(new ValidationMessage(
								Messages.getString("tipo_conformidade.validacao.descricao.unique"),
								"tipoDiretriz.descricao"));
			}
		} else if (tipoDiretriz.getMaximoConformidade() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_conformidade.validacao.maximo"),
					"tipoDiretriz.maximoConformidade"));
		} else if (tipoDiretriz.getMaximoConformidade() <= 0) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_conformidade.validacao.maximo.min"),
					"tipoDiretriz.maximoConformidade"));
		}
	}

}
