package br.com.eselo.dao;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.Url;

@Component
public class AvaliacaoDao extends GenericDao<Avaliacao> {

	public AvaliacaoDao(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Avaliacao> listAll(int regIni, int regFim) {
		return getSession().createCriteria(getPersistentClass())
				//.setFirstResult(regIni)
				//.setMaxResults(regFim)
				.addOrder(Order.desc("dataAvaliacao")).list();
	}
	
	public List<Avaliacao> listUsuario(Long idUsuario) {
		List<Avaliacao> avaliacaoList = this.listAll();

		List<Avaliacao> resultado = new ArrayList<Avaliacao>();
		for (Avaliacao aval : avaliacaoList) {
			if (aval.getIdUrl().getUsuario().getId().equals(idUsuario))
				resultado.add(aval);
		}
		
		return resultado;
	}

	public List<Avaliacao> listFiltro(String nUrl, Long idUsuario) {
		List<Avaliacao> avaliacaoList = this.listAll();

		List<Avaliacao> resultado = new ArrayList<Avaliacao>(0);
		for (Avaliacao aval : avaliacaoList) {
			if ((aval.getIdUrl().getUrl().contains(nUrl)) && aval.getIdUrl().getUsuario().getId().equals(idUsuario))
				resultado.add(aval);
		}
		return resultado;
	}

	public Avaliacao getAvaliacao(Long idAval) {
		return (Avaliacao) getSession().createCriteria(getPersistentClass())
				.add(Restrictions.eq("id", idAval))
				.uniqueResult();

	}
	
	public Long getAvaliacaoByURL(Url url) {
		return (Long) getSession().createCriteria(getPersistentClass())
				.add(Restrictions.eq("idUrl", url))
				.setProjection(Projections.max("id"))
				.uniqueResult();
	}
	
	public int numLinhas(){
		return (int) session.createCriteria(Avaliacao.class)
				.setProjection(Projections.rowCount())
				.uniqueResult();
	}

	public boolean existeAvaliacao(ParametroAvaliacao parametroAvaliacao) {
		@SuppressWarnings("unchecked")
		List<Avaliacao> encontrado = getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("parametroAvaliacao", parametroAvaliacao))
				.list();
		return encontrado.size() >0;
	}
	
}
