package br.com.eselo.dao;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.Diretriz;
import br.com.eselo.model.ParametroAvaliacao;
import br.com.eselo.model.TipoDiretriz;

@Component
public class DiretrizDao extends GenericDao<Diretriz> {

	public DiretrizDao(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Diretriz> listAll() {
		return getSession().createCriteria(getPersistentClass())
				.addOrder(Order.asc("id")).list();
	}

	public List<ParametroAvaliacao> getAllParametrosAvaliacao() {
		ParametroAvaliacaoDao parametroAvaliacao = (ParametroAvaliacaoDao) new ParametroAvaliacaoDao(
				getSession());

		return parametroAvaliacao.listAll();
	}

	@SuppressWarnings("unchecked")
	public List<Diretriz> getByParametroAvaliacao(
			ParametroAvaliacao parametroAvaliacao) {
		return getSession().createCriteria(getPersistentClass())
		// .add(Restrictions.eq("id_parametro_avaliacao",
		// parametroAvaliacao.getId()))
				.list();
	}

	public List<Diretriz> diretrizesDoUltimoParametro(
			ParametroAvaliacao ultimoParametroAvaliacao) {
		List<Diretriz> diretrizes = this.listAll();
		List<Diretriz> diretrizesDoUltimoParametro = new ArrayList<Diretriz>();

		for (Diretriz diretriz : diretrizes) {
			if (diretriz.getParametroAvaliacao().getId() == ultimoParametroAvaliacao
					.getId()) {
				diretrizesDoUltimoParametro.add(diretriz);
			}
		}

		return diretrizesDoUltimoParametro;
	}

	public ParametroAvaliacao getLastParametrosAvaliacao() {
		return new ParametroAvaliacaoDao(getSession())
				.load(new ParametroAvaliacaoDao(getSession()).getMaxId());
	}

	public boolean hasAvaliacao(ParametroAvaliacao ultimoParametroAvaliacao) {
		AvaliacaoDao dao = new AvaliacaoDao(this.getSession());
		List<Avaliacao> avaliacoes = dao.listAll();
		boolean result = false;

		for (Avaliacao avaliacao : avaliacoes) {
			if (avaliacao.getParametroAvaliacao().getId().longValue() == ultimoParametroAvaliacao
					.getId().longValue()) {
				result = true;
				break;
			}
		}

		return result;
	}

	@SuppressWarnings("unchecked")
	public List<Diretriz> listAllAtivas(ParametroAvaliacao parametroAvaliacao) {
		return getSession().createCriteria(getPersistentClass())
				.add(Restrictions.eq("parametroAvaliacao", parametroAvaliacao))
				.addOrder(Order.asc("numeroDiretriz")).list();
	}

	public int numLinhas(ParametroAvaliacao parametroAvaliacao) {
		return (int) session.createCriteria(getPersistentClass())
				.add(Restrictions.eq("parametroAvaliacao", parametroAvaliacao))
				.setProjection(Projections.rowCount()).uniqueResult();
	}

	public List<TipoDiretriz> getAllTipoDiretriz() {
		TipoDiretrizDao tipoDiretrizDao = (TipoDiretrizDao) new TipoDiretrizDao(
				getSession());

		return tipoDiretrizDao.listAll();
	}
}
