package br.com.eselo.dao;

import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.eselo.model.ParametroAvaliacao;

public abstract class GenericDao<T> {

	protected final Session session;

	private final Class<T> persistentClass;

	private final Logger logger;

	@SuppressWarnings("unchecked")
	public GenericDao(Session _session) {
		this.session = _session;
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
		this.logger = LoggerFactory.getLogger(this.persistentClass);
	}

	public Session getSession() {
		return this.session;
	}

	public Class<T> getPersistentClass() {
		return this.persistentClass;
	}

	public T load(Long id) throws HibernateException {
		T obj = this.persistentClass.cast(this.session
				.createCriteria(this.persistentClass)
				.add(Restrictions.idEq(id)).uniqueResult());
		Hibernate.initialize(obj);
		return obj;
	}

	@SuppressWarnings("unchecked")
	public List<T> listAll() throws HibernateException {
		return this.session.createCriteria(persistentClass).list();
	}
	
	@SuppressWarnings("unchecked")
	public List<T> listAll(int regIni, int regFim) {
		return getSession().createCriteria(getPersistentClass())
				//.setFirstResult(regIni)
				//.setMaxResults(regFim)
				.addOrder(Order.desc("id")).list();
	}
	
	public void saveAll(List<T> entities) throws HibernateException {
		for (T entity : entities) {
			this.save(entity);
		}
	}

	public void save(T entity) throws HibernateException {
		try {
			this.session.save(entity);
			this.session.flush();
		} catch (HibernateException e) {
			logger.debug(e.getMessage());
			throw e;
		}
	}

	public void update(T entity) throws HibernateException {
		try {
			this.session.update(entity);
			this.session.flush();
		} catch (HibernateException e) {
			logger.debug(e.getMessage());
			throw e;
		}
	}

	@SuppressWarnings("unchecked")
	public void remove(Long id) throws HibernateException {
		try {
			session.delete((T) session.load(persistentClass, id));
			session.flush();
		} catch (HibernateException e) {
			logger.debug(e.getMessage());
			throw e;
		}
	}

	public void saveOrUpdate(T obj) throws HibernateException {
		try {
			this.session.saveOrUpdate(obj);
		} catch (HibernateException e) {
			logger.debug(e.getMessage());
			throw e;
		}
	}
	
	public int numLinhas(){
		return (int) session.createCriteria(ParametroAvaliacao.class)
				.setProjection(Projections.rowCount()).uniqueResult();
	}

}
