package br.com.eselo.dao;

import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.Grupo;
import br.com.eselo.model.Permissao;
import br.com.eselo.model.Usuario;

@Component
public class GrupoDao extends GenericDao<Grupo> {

	public GrupoDao(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Grupo> listAll() throws HibernateException {
		return this.getSession().createCriteria(this.getPersistentClass())
				.addOrder(Order.asc("nome")).list();
	}

	public Long getIdByNome(String nome) {
		Grupo grupo = (Grupo) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("nome", nome)).uniqueResult();
		if (grupo != null) {
			return grupo.getId();
		}
		return null;
	}
	
	public Long getIdByNome(String nome, Long id) {
		Grupo grupo = (Grupo) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("nome", nome))
				.add(Restrictions.ne("id", id)).uniqueResult();
		if (grupo != null) {
			return grupo.getId();
		}
		return null;
	}
	
	public List<Permissao> getAllPermissoes() {
		PermissaoDao permissaoDao = (PermissaoDao) new PermissaoDao(
				getSession());

		return permissaoDao.listAll();
	}

	public List<Permissao> getAllPermissoes(List<Permissao> list) {
		PermissaoDao permissaoDao = (PermissaoDao) new PermissaoDao(
				getSession());

		List<Permissao> permissoesList = permissaoDao.listAll();

		for (Permissao perm : permissoesList) {
			perm.setMarcado(list.contains(perm));
		}

		return permissoesList;
	}

	public boolean hasPermission(String descricaoPermissao, Usuario usuario) {
		Grupo grupo = this.load(usuario.getGrupo().getId());
		List<Permissao> permissoes = grupo.getPermissoes();
		boolean result = false;
		
		for (Permissao permissao : permissoes) {
			if (permissao.getPermissao().equalsIgnoreCase(descricaoPermissao)) {
				result = true;
				break;
			}
		}
		
		return result;
	}
	
}
