package br.com.eselo.model;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import br.com.eselo.component.Util;

import com.google.common.hash.HashCode;
import com.google.common.hash.HashFunction;
import com.google.common.hash.Hashing;

@Entity
@Table(name = "sel_usuario")
public class Usuario {

	@Id
	@SequenceGenerator(name = "sel_usuario_id_usuario_seq", sequenceName = "sel_usuario_id_usuario_seq")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "sel_usuario_id_usuario_seq")
	@Column(name = "id_usuario", columnDefinition = "serial", unique = true, nullable = false)
	private Long id;

	@Column(name = "no_usuario")
	private String nome;

	@Column(name = "nu_cpf")
	private String cpf;

	private String senha;

	@Column(name = "ds_email")
	private String email;

	@Column(name = "ds_chave_alteracao_senha", nullable = true)
	private String chaveAlteracaoSenha;

	@Transient
	private String confirmacaoSenha;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "id_grupo", columnDefinition = "integer", nullable = false, insertable = true, updatable = true)
	private Grupo grupo;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public String getCpf() {
		return cpf;
	}

	public String getCpfFormatado() {
		return Util.format("###.###.###-##", cpf);
	}

	public void setCpf(String cpf) {
		this.cpf = cpf.replaceAll("[.-]", "");
	}

	public String getSenha() {
		return senha;
	}

	public void setSenha(String senha) {
		if (senha != null) {
			HashFunction hf = Hashing.sha512();
			HashCode hc = hf.newHasher().putString(senha).hash();

			senha = hc.toString();
		}

		this.senha = senha;
	}

	public void setHashedSenha(String senha) {
		this.senha = senha;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getChaveAlteracaoSenha() {
		return chaveAlteracaoSenha;
	}

	public void setChaveAlteracaoSenha(String chaveAlteracaoSenha) {
		this.chaveAlteracaoSenha = chaveAlteracaoSenha;
	}

	public String getConfirmacaoSenha() {
		return confirmacaoSenha;
	}

	public void setConfirmacaoSenha(String confirmacaoSenha) {
		if (confirmacaoSenha != null) {
			HashFunction hf = Hashing.sha512();
			HashCode hc = hf.newHasher().putString(confirmacaoSenha).hash();

			confirmacaoSenha = hc.toString();
		}

		this.confirmacaoSenha = confirmacaoSenha;
	}

	public Grupo getGrupo() {
		return grupo;
	}

	public void setGrupo(Grupo grupo) {
		this.grupo = grupo;
	}

}
