package br.com.eselo.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.caelum.vraptor.ioc.Component;
import br.com.caelum.vraptor.ioc.SessionScoped;

@Component
@SessionScoped
public class AsyncMailer {

	private static final Logger LOGGER = LoggerFactory
			.getLogger(AsyncMailer.class);

	private final ExecutorService executor;
	private final Mailer mailer;
	private final Queue<Email> mailQueue = new LinkedList<Email>();

	public AsyncMailer(ExecutorService _executor, Mailer _mailer) {
		this.executor = _executor;
		this.mailer = _mailer;
	}

	public Future<Void> asyncSend(final Email email) {
		LOGGER.debug("New email to be sent asynchronously: {} to {}",
				email.getSubject(), email.getToAddresses());
		Callable<Void> task = new Callable<Void>() {
			public Void call() {
				LOGGER.debug("Asynchronously sending email {} to {}",
						email.getSubject(), email.getToAddresses());
				try {
					AsyncMailer.this.mailer.send(email);
				} catch (EmailException e) {
					LOGGER.error(
							"Error while sending async email "
									+ email.getSubject() + " to "
									+ email.getToAddresses(), e);
				}
				return null;
			}
		};
		return executor.submit(task);
	}

	public void sendLater(Email email) {
		mailQueue.add(email);
	}

	public List<Email> clearPostponedMails() {
		List<Email> undeliveredMails = new ArrayList<Email>(mailQueue);
		mailQueue.clear();
		return undeliveredMails;
	}

	public Map<Email, Future<Void>> deliverPostponedMails() {
		Map<Email, Future<Void>> deliveries = new HashMap<Email, Future<Void>>();
		LOGGER.debug("Delivering all {} postponed emails", mailQueue.size());
		while (!mailQueue.isEmpty()) {
			Email nextMail = this.mailQueue.poll();
			Future<Void> sendingResult = this.asyncSend(nextMail);
			deliveries.put(nextMail, sendingResult);
		}
		return deliveries;
	}

	public boolean hasMailToDeliver() {
		return !mailQueue.isEmpty();
	}

}
