
package br.com.eselo.component;

import br.com.eselo.integracao.CriterioAvaliado;
import br.com.eselo.integracao.RecomendacaoAvaliada;
import br.com.eselo.integracao.RelatorioAvaliacao;
import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.Criterio;
import br.com.eselo.model.ResultadoCriterio;
import br.com.eselo.model.ResultadoRecomendacao;

import com.google.gson.Gson;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.basic.DateConverter;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;

public class LeitorXML {
	
	public RelatorioAvaliacao lerAvaliacaoXML(String fonte){

		XStream xstream = new XStream(new DomDriver("UTF-8", new XmlFriendlyReplacer("__", "_")));  
		
		xstream.registerConverter(new DateConverter( "EEE MMM dd HH:mm:ss zzz yyyy", new String[0] ));  

		
		xstream.alias("relatorio_de_avaliacao", RelatorioAvaliacao.class);
		xstream.alias("recomendacao", RecomendacaoAvaliada.class);

		
		xstream.aliasField("data", RelatorioAvaliacao.class, "date");
		xstream.addImplicitCollection(RelatorioAvaliacao.class, "recomendacoes");


		xstream.aliasField("id_recomendacao", RecomendacaoAvaliada.class, "idRecomendacao");
		xstream.aliasField("total_erros", RecomendacaoAvaliada.class, "totalErros");
		xstream.aliasField("total_avisos", RecomendacaoAvaliada.class, "totalAvisos");
		xstream.aliasField("qtde_linhas", RelatorioAvaliacao.class, "qtdeLinhas");

		
		xstream.alias("criterio", CriterioAvaliado.class);
		xstream.addImplicitCollection(RecomendacaoAvaliada.class, "criterios");
		xstream.aliasField("id_criterio", CriterioAvaliado.class, "idCriterio");
		xstream.aliasField("itens_criterio", CriterioAvaliado.class, "qtdeItens");
		xstream.aliasField("erros_criterio", CriterioAvaliado.class, "qtdeErros");		

		
		return (RelatorioAvaliacao) xstream.fromXML(fonte);
	}
	
	public String toJson(RelatorioAvaliacao ar){
		return new Gson().toJson(ar);
	}

	public Avaliacao lerResultadosXML(RelatorioAvaliacao relatorioAvaliacao, Avaliacao avaliacao){
		
		for(RecomendacaoAvaliada ra:relatorioAvaliacao.getRecomendacoes()){
			
			ResultadoRecomendacao recomendacao = new ResultadoRecomendacao();
			recomendacao.setTotalErros(ra.getTotalErros());
			recomendacao.setTotalAvisos(ra.getTotalAvisos());
			recomendacao.setNumRecomendacao(ra.getIdRecomendacao());
			recomendacao.setNotaRecomendacao(ra.getNuNota());
			recomendacao.setAvaliacao(avaliacao);
			
			
			for(CriterioAvaliado ca:ra.getCriterios()){
				
				ResultadoCriterio criterio = new ResultadoCriterio();
				Criterio criterio2 = new Criterio();
				
				criterio2.setId(ca.getCriterio().getId());
				criterio2.setNumeroCriterio(ca.getCriterio().getNumeroCriterio());
				
				//criterio.setNumCriterio(ca.getIdCriterio());
				criterio.setIdCriterio(criterio2.getId());
				criterio.setNumErros(ca.getQtdeErros());
				criterio.setNumItens(ca.getQtdeItens());
				criterio.setNotaCriterio(ca.getNu_nota());
				criterio.setResultadoRecomendacao(recomendacao);
								
				recomendacao.addCriterio(criterio);
			}
			
			avaliacao.addResultadoRecomendacao(recomendacao);
		}
		
		return avaliacao;
	}
}
