package br.com.eselo.component;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

public class URLNormalizer {

	public static String normalize(String taintedURL)
			throws MalformedURLException {
		final URL url;

		if ((!taintedURL.startsWith("http://"))
				&& (!taintedURL.startsWith("https://"))) {
			taintedURL = "http://" + taintedURL;
		}

		if (taintedURL.endsWith("/")) {
			taintedURL = taintedURL.substring(0, taintedURL.length() - 1);
		}

		try {
			url = new URI(taintedURL).normalize().toURL();
		} catch (URISyntaxException e) {
			throw new MalformedURLException();
		}

		final String path = url.getPath().replace("/$", "");
		final SortedMap<String, String> params = createParameterMap(url
				.getQuery());
		final int port = url.getPort();
		final String querString;

		if (params != null) {
			for (Iterator<String> i = params.keySet().iterator(); i.hasNext();) {
				final String key = i.next();
				if (key.startsWith("utm_") || (key.contains("session"))) {
					i.remove();
				}
			}
			querString = "?" + canonicalize(params);
		} else {
			querString = "";
		}

		return url.getHost() + (port != -1 && port != 80 ? +port : "") + path
				+ querString;
	}

	private static Object percentEncodeRfc3986(final String string) {
		try {
			return URLEncoder.encode(string, "UTF-8").replace("+", "%20")
					.replace("*", "%2A").replace("%7E", "~");
		} catch (UnsupportedEncodingException e) {
			return string;
		}
	}

	private static SortedMap<String, String> createParameterMap(
			final String queryString) {
		if (queryString == null || queryString.isEmpty()) {
			return null;
		}

		final String[] pairs = queryString.split("&");
		final Map<String, String> params = new HashMap<String, String>(
				pairs.length);

		for (final String pair : pairs) {
			if (pair.length() < 1) {
				continue;
			}

			String[] tokens = pair.split("=", 2);
			for (int j = 0; j < tokens.length; j++) {
				try {
					tokens[j] = URLDecoder.decode(tokens[j], "UTF-8");
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}

			switch (tokens.length) {
			case 1:
				if (pair.charAt(0) == '=') {
					params.put("", tokens[0]);
				} else {
					params.put(tokens[0], "");
				}
				break;

			case 2:
				params.put(tokens[0], tokens[1]);
				break;
			}
		}

		return new TreeMap<String, String>(params);
	}

	private static String canonicalize(
			final SortedMap<String, String> sortedParamMap) {
		if (sortedParamMap == null || sortedParamMap.isEmpty()) {
			return "";
		}

		final StringBuffer sb = new StringBuffer(350);
		final Iterator<Map.Entry<String, String>> iter = sortedParamMap
				.entrySet().iterator();

		while (iter.hasNext()) {
			final Map.Entry<String, String> pair = iter.next();
			sb.append(percentEncodeRfc3986(pair.getKey()));
			sb.append("=");
			sb.append(percentEncodeRfc3986(pair.getValue()));

			if (iter.hasNext()) {
				sb.append("&");
			}
		}

		return sb.toString();
	}

}
