package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.CopiadorParametroCalculo;
import br.com.eselo.component.Messages;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.AvaliacaoDAO;
import br.com.eselo.dao.CriterioDAO;
import br.com.eselo.dao.ParametroCalculoDAO;
import br.com.eselo.dao.RecomendacaoDAO;
import br.com.eselo.dao.TipoTesteDAO;
import br.com.eselo.model.Criterio;
import br.com.eselo.model.ParametroCalculo;
import br.com.eselo.model.Recomendacao;
import br.com.eselo.model.TipoTeste;

@Resource
public class CriteriosController {

	private final CriterioDAO dao;
	private final ParametroCalculoDAO paDAO;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public CriteriosController(CriterioDAO _dao, ParametroCalculoDAO _paDAO,
			Result _result, Validator _validator, UsuarioWeb _usuarioWeb) {
		this.dao = _dao;
		this.paDAO = _paDAO;
		this.result = _result;
		this.validator = _validator;
		this.usuarioWeb = _usuarioWeb;
	}

	@Get("/criterio/novo/{id}")
	@Restrito(descricao = "CRITERIO_ADICIONAR")
	public Criterio novo(Long id) {
		Recomendacao recomendacao = new RecomendacaoDAO(dao.getSession())
				.load(id);
		Criterio criterio = new Criterio();
		criterio.setRecomendacao(recomendacao);

		result.include("parametroCalculoId", criterio.getRecomendacao()
				.getParametroCalculo().getId());
		result.include("parametroCalculo", criterio.getRecomendacao()
				.getParametroCalculo());
		result.include("recomendacao", criterio.getRecomendacao());
		result.include("recomendacaoId", criterio.getRecomendacao().getId());

		populateSelectLists();

		return criterio;
	}

	@Get("/criterio/form/{id}")
	@Restrito(descricao = "CRITERIO_ATUALIZAR")
	public Criterio form(Long id) {
		Criterio criterio = dao.load(id);
		result.include("recomendacaoId", criterio.getRecomendacao().getId());
		result.include("parametroCalculo", criterio.getRecomendacao().getParametroCalculo());
		result.include("tipoTeste", criterio.getTipoTeste());
		populateSelectLists();

		return criterio;
	}

	@Post("/criterio")
	@Restrito(descricao = "CRITERIO_ADICIONAR")
	public void adicionar(Criterio criterio) {

		criterio.setRecomendacao(new RecomendacaoDAO(dao.getSession())
				.load(criterio.getRecomendacao().getId()));

		validarCriterio(criterio);

		result.include("recomendacaoId", criterio.getRecomendacao().getId());
		result.include("parametroCalculo", criterio.getRecomendacao().getParametroCalculo());
		result.include("tipoTeste", criterio.getTipoTeste());
		populateSelectLists();

		validator.onErrorUsePageOf(CriteriosController.class).novo(
				criterio.getRecomendacao().getId());

		
		ParametroCalculo pc = paDAO.load(criterio.getRecomendacao().getParametroCalculo().getId());
		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			
			dao.save(criterio);
			result.include("mensagem", Messages.getString("criterio.salvo"));
			result.redirectTo(RecomendacaoController.class).form(criterio.getRecomendacao().getId());
		}else{
			this.criarNovoParametroAvaliacao(criterio,ParametroCalculoController.MODO_ADD);
			result.include("mensagem", Messages.getString("criterio.salvo")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
			result.redirectTo(ParametroCalculoController.class).index(1);
		}
		
	}

	@Put("/criterio")
	@Restrito(descricao = "CRITERIO_ATUALIZAR")
	public void atualizar(Criterio criterio) {
		//criterio.setRecomendacao(new RecomendacaoDAO(dao.getSession()).load(criterio.getRecomendacao().getId()));

		@SuppressWarnings("unchecked")
		List<TipoTeste> tipoTestes = (List<TipoTeste>) dao.getAllTipoTeste();
		Criterio criterioCadastrado = dao.load(criterio.getId());


		result.include("recomendacaoId", criterio.getRecomendacao().getId());
		result.include("parametroCalculo", criterio.getRecomendacao().getParametroCalculo());
		result.include("tipoTeste", criterio.getTipoTeste());
		result.include("tipoTesteList", tipoTestes);

		validarCriterio(criterio);

		validator.onErrorUsePageOf(CriteriosController.class).form(criterioCadastrado.getId());

		if(validarAtualizacaoCriterio(criterio)){

			ParametroCalculo pc = paDAO.load(criterioCadastrado.getRecomendacao().getParametroCalculo().getId());
			if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
				if(criterioCadastrado.getTipoTeste().getId().intValue()==3 && criterio.getTipoTeste().getId().intValue()!=3){
					Criterio novoCriterio = new Criterio();
					
					novoCriterio.setId(criterio.getId());
					novoCriterio.setRecomendacao(criterioCadastrado.getRecomendacao());
					novoCriterio.setNumeroCriterio(criterio.getNumeroCriterio());
					novoCriterio.setDescricao(criterio.getDescricao());
					novoCriterio.setPeso(criterio.getPeso());
					novoCriterio.setTipoTeste(new TipoTesteDAO(dao.getSession()).load(criterio.getTipoTeste().getId()));
					novoCriterio.setRequisito(criterio.getRequisito());
					novoCriterio.setErro(criterio.isErro());
					novoCriterio.setVerificacaoAutomatica(criterio.isVerificacaoAutomatica());
					
					
					dao.remove(criterio.getId());
					dao.save(novoCriterio);
					
				}else{
					criterioCadastrado.setNumeroCriterio(criterio.getNumeroCriterio());
					criterioCadastrado.setDescricao(criterio.getDescricao());
					criterioCadastrado.setPeso(criterio.getPeso());
					criterioCadastrado.setTipoTeste(new TipoTesteDAO(dao.getSession()).load(criterio.getTipoTeste().getId()));
					criterioCadastrado.setRequisito(criterio.getRequisito());
					criterioCadastrado.setErro(criterio.isErro());
					criterioCadastrado.setVerificacaoAutomatica(criterio.isVerificacaoAutomatica());
					
					
					dao.update(criterioCadastrado);
				}
				
				result.include("mensagem", Messages.getString("criterio.atualizado"));
				result.redirectTo(RecomendacaoController.class).form(criterio.getRecomendacao().getId());
			}else{
				criterio.setRecomendacao(criterioCadastrado.getRecomendacao());
				criarNovoParametroAvaliacao(criterio,ParametroCalculoController.MODO_EDIT);
				result.include("mensagem", Messages.getString("criterio.salvo")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
				result.redirectTo(ParametroCalculoController.class).index(1);
			}
		}else{
			criterioCadastrado.setDescricao(criterio.getDescricao());
			dao.update(criterioCadastrado);
			result.include("mensagem", Messages.getString("validacao.sem_alteracao"));
			result.redirectTo(CriteriosController.class).form(criterio.getId());
		}
	}

	@Path("/criterio/delete/{id}")
	@Restrito(descricao = "CRITERIO_EXCLUIR")
	public void excluir(Long id) {
		Criterio criterio = dao.load(id);
		long idRecomendacao = criterio.getRecomendacao().getId();

		ParametroCalculo pc = paDAO.load(criterio.getRecomendacao().getParametroCalculo().getId());
		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			dao.remove(criterio.getId());
			result.include("mensagem", Messages.getString("criterio.excluído"));
			result.redirectTo(RecomendacaoController.class).form(idRecomendacao);
		}else{
			criarNovoParametroAvaliacao(criterio,ParametroCalculoController.MODO_DELETE);
			result.include("mensagem", Messages.getString("criterio.excluído")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());;
			result.redirectTo(ParametroCalculoController.class).index(1);
		}
		
	}

	private void criarNovoParametroAvaliacao(Criterio criterio, int tipo) {
		ParametroCalculo ultimoParametroAvaliacao = new CopiadorParametroCalculo().copiarParametroCalculo(paDAO.getLastParametrosCalculoAtivo());
		boolean ok = false;

		switch (tipo) {
		case ParametroCalculoController.MODO_ADD:
			for (Recomendacao recomendacao : ultimoParametroAvaliacao
					.getRecomendacoes()) {
				if (recomendacao.getNumeroRecomendacao().longValue() == criterio.getRecomendacao().getNumeroRecomendacao().longValue()) {
					recomendacao.addCriterio(criterio);
					ok = true;
					break;
				}
			}

			break;

		case ParametroCalculoController.MODO_EDIT:
			ultimoParametroAvaliacao = paDAO.getLastParametrosCalculoAtivo();
			for (Recomendacao recomendacao : ultimoParametroAvaliacao
					.getRecomendacoes()) {
				if (recomendacao.getNumeroRecomendacao().longValue() == criterio.getRecomendacao().getNumeroRecomendacao().longValue()) {
					for (Criterio c : recomendacao.getCriterios()) {
						if (criterio.getNumeroCriterio().longValue() == c.getNumeroCriterio().longValue()) {
							c.setDescricao(criterio.getDescricao());
							c.setFaixas(criterio.getFaixas());
							c.setId(criterio.getId());
							c.setNumeroCriterio(criterio.getNumeroCriterio());
							c.setPeso(criterio.getPeso());
							c.setRecomendacao(criterio.getRecomendacao());
							c.setTipoTeste(criterio.getTipoTeste());

							ok = true;
							break;
						}
					}
				}
			}

			break;

		case ParametroCalculoController.MODO_DELETE:
			for (Recomendacao recomendacao : ultimoParametroAvaliacao.getRecomendacoes()) {
				if (recomendacao.getNumeroRecomendacao().longValue() == criterio.getRecomendacao().getNumeroRecomendacao().longValue()) {
					for(int i=0; i< recomendacao.getCriterios().size(); i++){
						if(recomendacao.getCriterios().get(i).getNumeroCriterio().longValue() == criterio.getNumeroCriterio().longValue()){
							recomendacao.getCriterios().remove(i);
							ok = true;
							break;
						}
					}
				}
			}
			break;
		}

		if (ok) {
			new CopiadorParametroCalculo().criarNovoParametroCalculo(
					ultimoParametroAvaliacao, usuarioWeb, dao.getSession());
		}
	}

	private void validarCriterio(Criterio criterio) {

		if (criterio.getNumeroCriterio() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("criterio.validacao.numero"),
					"criteiro.numeroCriterio"));
		} else if (hasNumeroCriterioRecomendacaoAtual(criterio)) {
			validator.add(new ValidationMessage(
					Messages.getString("criterio.validacao.numero.unique"),
					"criteiro.numeroCriteiro"));
		} else if (criterio.getPeso() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("validacao.peso"),
					"criteiro.peso"));
		} else if (criterio.getPeso() <= 0) {
			validator
					.add(new ValidationMessage(
							Messages.getString("validacao.peso.zero"),
							"criteiro.peso"));
		} else if (criterio.getDescricao() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("criterio.validacao.descricao"),
					"criteiro.descricao"));
		}else if (criterio.getTipoTeste().getId() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("criterio.validacao.tipo_teste"),
					"criterio.tipoTeste"));
		} else {
			criterio.setTipoTeste(new TipoTesteDAO(dao.getSession())
					.load(criterio.getTipoTeste().getId()));
		}
	}

	private boolean validarAtualizacaoCriterio(Criterio criterio) {

		boolean alteracao = false;

		Criterio criterioCadastrado = dao.load(criterio.getId());

		if (criterio.getNumeroCriterio().compareTo(criterioCadastrado.getNumeroCriterio())!=0) {
			alteracao = true;
		} else if (criterio.getPeso().compareTo(criterioCadastrado.getPeso()) !=0) {
			alteracao = true;
		} else if (criterio.getDescricao().compareToIgnoreCase(criterioCadastrado.getDescricao()) != 0) {
			alteracao = true;
		} else if (criterio.getTipoTeste().getId().compareTo(criterioCadastrado.getTipoTeste().getId())!=0) {
			alteracao = true;
		}else if (criterio.getRequisito().compareToIgnoreCase(criterioCadastrado.getRequisito()) != 0) {
			alteracao = true;
		}else if (Boolean.compare(criterio.isErro(), criterioCadastrado.isErro()) != 0) {
			alteracao = true;
		}else if (Boolean.compare(criterio.isVerificacaoAutomatica(), criterioCadastrado.isVerificacaoAutomatica()) != 0) {
			alteracao = true;
		}

		return alteracao;
	}

	/**
	 * Recebe um critério antes de ser cadastrad0 e verifica se ja existe um
	 * criterio ativo com o mesmo numero.
	 *
	 * @param criterio
	 * @return boolean
	 */
	private boolean hasNumeroCriterioRecomendacaoAtual(Criterio criterio) {
		boolean retorno = false;

		if (criterio.getDescricao() != null
				&& criterio.getNumeroCriterio() != null
				&& criterio.getPeso() != null
				&& criterio.getRecomendacao() != null
				&& criterio.getTipoTeste().getId() != null) {
			Recomendacao recomendacao = criterio.getRecomendacao();

			List<Criterio> listaCadastrada = dao.criterioDaRecomendacao(recomendacao);

			for (Criterio criterioCadastrado : listaCadastrada) {
				if (criterioCadastrado.getNumeroCriterio().longValue() == criterio
						.getNumeroCriterio().longValue()) {
					if (criterio.getId() != null) {
						if (criterioCadastrado.getId().longValue() != criterio
								.getId().longValue()) {
							return true;
						}
					} else {
						return true;
					}
				}
			}

		}

		return retorno;
	}

	private void populateSelectLists() {
		result.include("tipoTesteList", dao.getAllTipoTeste());
	}

}
