package br.com.eselo.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.servlet.ServletContext;

import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.ioc.spring.VRaptorRequestHolder;
import br.com.caelum.vraptor.view.Results;
import br.com.eselo.component.Mailer;
import br.com.eselo.component.Util;
import br.com.eselo.controller.PainelAcessibilidadeController.CaminhoLink;
import br.com.eselo.dao.ErrosMaisComunsDAO;
import br.com.eselo.dao.MediaAvaliacaoDAO;
import br.com.eselo.model.Avaliacao;
import br.com.eselo.model.ErrosMaisComuns;
import br.com.eselo.model.GraficoMediaAvaliacao;

@Resource
public class ErrosMaisComunsController {

	private static final Logger LOGGER = LoggerFactory.getLogger(AvaliacaoController.class);
	private final ErrosMaisComunsDAO errosMaisComunsDAO;
	private ServletContext application;
	private Result result;
	private static final String PAINEL_AGREGADO = "eselo.config.avaliacao.painelAgregado";
	private final Properties properties = new Properties();
	private final MediaAvaliacaoDAO mediaAvaliacaoDAO;
	
	
	public ErrosMaisComunsController(ErrosMaisComunsDAO _errosMaisComunsDAO, ServletContext application, Result result, MediaAvaliacaoDAO _mediaAvaliacaoDAO) throws IOException {
		super();
		this.errosMaisComunsDAO = _errosMaisComunsDAO;
		this.application = application;
		this.result = result;
		this.mediaAvaliacaoDAO = _mediaAvaliacaoDAO;
		String fileName = "config.properties";
		InputStream stream = Mailer.class.getResourceAsStream("/" + fileName);
		Properties propertiesFile = new Properties();

		if (stream != null) {
			propertiesFile.load(stream);
			this.properties.putAll(propertiesFile);
		} else {
			LOGGER.warn("Could not find the file '{}' to load.", fileName);
		}
	}

	
	@Get("/gerar-painel-erros-mais-comuns")
	public void gerarPainelErrosMaisComuns() {
		
		
		gerarPainelErrosMaisComunsComMediaBrasil();
		List<ErrosMaisComuns> lsErrosMaisComuns = errosMaisComunsDAO.errosMaisComuns();
		File painelAgregado = new File(properties.getProperty(PAINEL_AGREGADO));
		
				
		boolean paginaGerada = false;
		
		try {
			//List<ErrosMaisComuns> lsErrosMaisComuns = errosMaisComunsDAO.errosMaisComuns();
			
			OutputStreamWriter bufferOut = new OutputStreamWriter(new FileOutputStream(painelAgregado + "/painelAgregado.txt"),"UTF-8");
		
		
			//Inicia criação da página
			
			bufferOut.write(" <div class=\"logoAvaliacao\"> ");
			bufferOut.write(" <div id=\"emag\"> ");
			bufferOut.write(" 		<label  id=\"centralizar\">eMAG v 3.1</label>	 ");
			bufferOut.write(" </div> ");
			
		
		
			
			//início Gráfico
			bufferOut.write(" <div id=\"grafico\">				 ");
			
			
			bufferOut
			.write("				<h2 class=\"oculto_visao\" >Gráfico com os 10 erros mais comuns.</h2><br> ");
	
			bufferOut.write("<ul class=\"oculto_visao\" >	");
			
			long total = 0;
			double percentual = 0;
			long totalDezErrosMaisComuns = 0;
			double qntErro;
			
			int i = 0;
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					DecimalFormat df = new DecimalFormat("0.##");
					
					i++;
					bufferOut.write("<li >erro: <span id=\"erro" + i +"\">" + "Erro: " + i + "</span><span>" + errosMaisComuns.getDescricao() + "</span> <span id=\"p_erro" + i + "\">" + df.format(percentual) + "</span></li>");				
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100;
			percentual = qntErro/total;	
			
			DecimalFormat df = new DecimalFormat("0.##");
			
			i++;
			bufferOut.write("<li >erro: <span id=\"erro" + i + "\">"  + "Erro: " + i + "</span><span>Outros</span>  <span id=\"p_erro" + i + "\">" + df.format(percentual) + "</span></li>");
		
			bufferOut.write("</ul>");
			bufferOut.write("<label id=\"qntAvaliacoes\" class=\"oculto\">" + i
					+ "</label>");
			bufferOut.write("		<div class=\"myChart\" >				");
			bufferOut.write("			<canvas id=\"myChartPainelAgregado\" class=\"myChartCanvas\"></canvas>				");
			bufferOut.write("		</div>");
			bufferOut.write("	</div>");
			bufferOut.write("	</div>");
			bufferOut.write("	</div>");
			
			//inicio Avaliação
			bufferOut.write(" <div class=\"avaliacao\"> ");
			
			
			bufferOut.write(" <h2 id=\"tituloTabela\">Os 10 erros mais comuns</h2>");
			//inicio tabela						
			bufferOut.write(" <table> ");
			bufferOut.write(" <thead> ");
			bufferOut.write(" <tr><th   colspan=\"3\"> Testes Automáticos</th></tr>");
			bufferOut.write(" <tr> ");
			bufferOut.write(" <th id=\"erro_numero\">Erro n°</th> ");
			bufferOut.write(" <th id=\"erro\">Erro</th> ");
			bufferOut.write(" <th id=\"incidencia\">Incidência</th> ");
			
			
			
			
			
			 total = 0;
			 percentual = 0;
			 totalDezErrosMaisComuns = 0;
			 qntErro = 0;
			 Integer erro_numero = 0;
			 
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100.00;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					erro_numero++;
					
					DecimalFormat df2 = new DecimalFormat("0.##");
					bufferOut.write(" <tr> ");
					bufferOut.write(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
					bufferOut.write(" <td headers=\"erro\" class=\"celula\"> " + errosMaisComuns.getDescricao() + " </td> ");
					bufferOut.write(" <td headers=\"incidencia\" class=\"celula\"> " + df2.format(percentual) + "% </td> ");
					bufferOut.write(" </tr> ");
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100.00;
			percentual = qntErro/total;	
			
			DecimalFormat df2 = new DecimalFormat("0.##");
			
			erro_numero++;
			
			bufferOut.write(" <tr> ");
			bufferOut.write(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
			bufferOut.write(" <td headers=\"erroMaisComum\" class=\"celula\"> <a href=\"/outros-erros-mais-comuns\"> Outros  </a></td> ");
			bufferOut.write(" <td headers=\"percentual\" class=\"celula\">" + df2.format(percentual) + "%</td> ");
			bufferOut.write(" <tr> ");

			bufferOut.write(" </tbody> ");
			bufferOut.write(" </table> <br>");
			bufferOut.write(" </div> ");
			
			//fim Avaliação
				
			
			bufferOut.close();
			paginaGerada = true;
			//Termina criação da página
			if(paginaGerada)
			{
				paginaGerada = gerarPainelOutrosErrosMaisComuns();
			}
			
			
		} catch (UnsupportedEncodingException e) {
			paginaGerada = false;
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			paginaGerada = false;
			e.printStackTrace();
		} catch (IOException e) {
			paginaGerada = false;
			e.printStackTrace();
		}finally
		{
			result.use(Results.xml()).from(paginaGerada).recursive().serialize();
		}
		
		
	}
	
	private double buscaMediaBrasil(List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao)
	{
		
		
		double mediaBrasil = 0;
		int qntAvaliacoes = 0;
		
		BigDecimal gigDecimalMedia;
		
		for (GraficoMediaAvaliacao graficoMediaAvaliacao : lsGraficoMediaAvaliacao) {
			
			qntAvaliacoes++;
			mediaBrasil +=  graficoMediaAvaliacao.getMediaGrafico().doubleValue();
		}
		
		List<CaminhoLink> lsCaminhoAvaliacao = new ArrayList<CaminhoLink>();
		CaminhoLink caminhoLink = new CaminhoLink();
		
		mediaBrasil = mediaBrasil/qntAvaliacoes;
		
		caminhoLink.setNome("Média Brasil - " + Util.formataCasasDecimais(mediaBrasil));
		caminhoLink.setLink("/media-avaliacao/orgao");
		caminhoLink.setClasse("mBrasil");
		lsCaminhoAvaliacao.add(caminhoLink);
		
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("lsCaminhoAvaliacao", lsCaminhoAvaliacao);
		
					
		return Util.formataCasasDecimais(mediaBrasil);
	}
	
	private void gerarPainelErrosMaisComunsComMediaBrasil()
	{
		
		List<ErrosMaisComuns> lsErrosMaisComuns = errosMaisComunsDAO.errosMaisComuns();
		File painelAgregado = new File(properties.getProperty(PAINEL_AGREGADO));
		
		
		
		Double mediaBrasil;
		
		List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao = mediaAvaliacaoDAO.gerarGraficoMediaAvaliacao("governo federal", null, 4);
		
		
	
		mediaBrasil = buscaMediaBrasil(lsGraficoMediaAvaliacao);
		
		
		
		String webaxscore = "";
		
		  if (mediaBrasil >= 95) {
		        webaxscore = "bom";      
		      } else if (mediaBrasil >= 85 & mediaBrasil < 95) {
		          webaxscore = "razoavel";
		      } else if (mediaBrasil >= 70 & mediaBrasil < 85) {
		          webaxscore = "ruim";
		      }else {
		        webaxscore = "pessimo";
		      }
	
		 //inicio Erros mais comuns
		  
		 
			
		  try {
			OutputStreamWriter bufferOut = new OutputStreamWriter(new FileOutputStream(painelAgregado + "/painelAgregadoComMediaBrasil.txt"),"UTF-8");
			
			
			bufferOut.write(" <div class=\"logoAvaliacao\"> ");
			bufferOut.write(" <div id=\"emag\"> ");		
			bufferOut.write(" 	<label  id=\"centralizar\">eMAG v 3.1</label> ");		
			bufferOut.write(" </div> ");
			
			bufferOut.write("		<div id=\"testesAutomaticos\">					\n");
			//bufferOut.write("				<label>Testes Automáticos<br> Nota: " + resumoAvaliacao.getValor() + "/100</label>	</div>	");
			
			bufferOut.write("			<div class=\"cell width-5 position-2 >\n");
			
			
			bufferOut.write("				<div id=\"block\">\n");
			bufferOut.write("					<div id=\"webaxscore\" class=\"scoreB_" + webaxscore + "\">Porcentagem");
			bufferOut.write("						<strong>Média</strong>");
			bufferOut.write("						<span>" + Util.formataCasasDecimais(mediaBrasil) +"%</span>\n");
			bufferOut.write("					</div>\n");
			bufferOut.write("				</div>\n");
			
			//bufferOut.write("				<br>");
			
			
			bufferOut.write("<div id=\"legenda\" >\n");
			bufferOut.write("<label>Legenda</label><br>");
			bufferOut.write("<ul>");
			bufferOut.write("	<li class=\"legenda\"><img id=\"bom\" alt=\"farol verde, média maior ou igual a 95%\" src=\"/static/img/verde.png\"></li>");
			bufferOut.write("	<li class=\"legenda\"><img id=\"razoavel\" alt=\"farol amarela, média maior ou igual a 85% e menor que 95%\" src=\"/static/img/amarelo.png\"></li>");
			bufferOut.write("	<li class=\"legenda\"><img id=\"ruim\" alt=\"farol laranja, média maior ou igual a 70% e menor que 85%\" src=\"/static/img/laranja.png\"></li>");
			bufferOut.write("	<li class=\"legenda\"><img id=\"pessimo\" alt=\"farol vermelho, média menor que 70%\" src=\"/static/img/vermelho.png\"></li>");
			bufferOut.write("</ul>");
			bufferOut.write("</div>");
			
			bufferOut.write("			</div>");
			
			bufferOut.write(" <div id=\"grafico2\"> ");
			bufferOut.write(" 	<h2 class=\"oculto_visao\">Gráfico com as notas das últimas avaliações ordenada da mais antiga para a mais atual, limitado até quatro avaliações.</h2> ");
			bufferOut.write(" 	<ul class=\"oculto_visao\"> ");
				
			int qntAvaliacoes = 0;
			
			String dataAvaliacao = "";
			
			
			
	for (GraficoMediaAvaliacao graficoMediaAvaliacao : lsGraficoMediaAvaliacao) {
		
		mediaBrasil += graficoMediaAvaliacao.getMediaGrafico().doubleValue();
		
		qntAvaliacoes++;
		
		//dataAvaliacao = graficoMediaAvaliacao.getIdAgendamento().substring(6, 8) + "/";
		dataAvaliacao = graficoMediaAvaliacao.getId_agendamentoGrafico().substring(4, 6) + "/";
		dataAvaliacao += graficoMediaAvaliacao.getId_agendamentoGrafico().substring(0, 4);
		
		bufferOut.write(" <li>Média "+ qntAvaliacoes + ": <span id=\"media"+ qntAvaliacoes +"\">"+graficoMediaAvaliacao.getMediaGrafico().doubleValue()+"</span><span>Data da avaliação: </span><span id=\"data" + qntAvaliacoes + "\">"+ dataAvaliacao +"</span></li>");
				
			}


			bufferOut.write("  	</ul> ");
			bufferOut.write(" 	<label id=\"qntAvaliacoes\" class=\"oculto\">"+ qntAvaliacoes +"</label> ");
			//bufferOut.write(" 	<label id=\"qntAvaliacoes\" >"+ legenda +"</label> ");
			
			bufferOut.write(" 	<div class=\"myChart\"> ");
			bufferOut.write(" 		<iframe class=\"chartjs-hidden-iframe\" ");
			bufferOut.write(" 			style=\"width: 100%; display: block; border: 0px; height: 0px; margin: 0px; position: absolute; left: 0px; right: 0px; top: 0px; bottom: 0px;\"></iframe> ");
			bufferOut.write(" 		<canvas id=\"myChartMediaAvaliacao\" style=\"width: 420px; height: 210px;\" ");
			bufferOut.write(" 			width=\"420\" height=\"210\"></canvas> ");
			bufferOut.write(" 	</div> ");
			bufferOut.write(" </div> ");
			bufferOut.write(" </div> ");
			
			
			long total = 0;
			double percentual = 0;
			long totalDezErrosMaisComuns = 0;
			double qntErro;
			
			int i = 0;
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					//DecimalFormat df = new DecimalFormat("0.##");
					
					i++;
					//bufferOut.write("<li >erro: <span id=\"erro" + i +"\">" + "Erro: " + i + "</span><span>" + errosMaisComuns.getDescricao() + "</span> <span id=\"p_erro" + i + "\">" + df.format(percentual) + "</span></li>");				
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100;
			percentual = qntErro/total;	
			
			//DecimalFormat df = new DecimalFormat("0.##");
			
		
			
			//inicio Avaliação
			bufferOut.write(" <div class=\"avaliacao\"> ");
			
			
					bufferOut.write(" <h2 id=\"tituloTabela\">Os 10 erros mais comuns</h2><br>");
					//inicio tabela						
					bufferOut.write(" <table> ");
						bufferOut.write(" <thead> ");
							bufferOut.write(" <tr><th   colspan=\"3\"> Testes Automáticos</th></tr>");
							bufferOut.write(" <tr> ");
								bufferOut.write(" <th id=\"erro_numero\">Erro n°</th> ");
								bufferOut.write(" <th id=\"erro\">Erro</th> ");
								bufferOut.write(" <th id=\"incidencia\">Incidência</th> ");
			
			
			
			
			
			 total = 0;
			 percentual = 0;
			 totalDezErrosMaisComuns = 0;
			 qntErro = 0;
			 Integer erro_numero = 0;
			 
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100.00;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					erro_numero++;
					
					//DecimalFormat df2 = new DecimalFormat("0.##");
							bufferOut.write(" <tr> ");
								bufferOut.write(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
								bufferOut.write(" <td headers=\"erro\" class=\"celula\"> " + errosMaisComuns.getDescricao() + " </td> ");
								bufferOut.write(" <td headers=\"incidencia\" class=\"celula\"> " + Util.formataCasasDecimais(percentual) + "% </td> ");
							bufferOut.write(" </tr> ");
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100.00;
			percentual = qntErro/total;	
			
			//DecimalFormat df2 = new DecimalFormat("0.##");
			
			erro_numero++;
			
						bufferOut.write(" <tr> ");
							bufferOut.write(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
							bufferOut.write(" <td headers=\"erroMaisComum\" class=\"celula\"> <a href=\"/outros-erros-mais-comuns\"> Outros  </a></td> ");
							bufferOut.write(" <td headers=\"percentual\" class=\"celula\">" + Util.formataCasasDecimais(percentual) + "%</td> ");
						bufferOut.write(" <tr> ");

					bufferOut.write(" </tbody> ");
			bufferOut.write(" </table> <br>");
			bufferOut.write(" </div> ");
			bufferOut.close();
			//fim Avaliação
		} catch (UnsupportedEncodingException e) {		

			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		  //fim Erros mais comuns
		
		 
	}
	
	@SuppressWarnings("finally")
	private boolean gerarPainelOutrosErrosMaisComuns()
	{
		boolean paginaGerada = false;
		
		File painelAgregado = new File(properties.getProperty(PAINEL_AGREGADO));
		
		try {
			List<ErrosMaisComuns> lsOutrosErrosMaisComuns = errosMaisComunsDAO.OutrosErrosMaisComuns();
			
			OutputStreamWriter bufferOut = new OutputStreamWriter(new FileOutputStream(painelAgregado + "/painelOutrosErrosMaisComuns.txt"),"UTF-8");
		
					
			
			//inicio Avaliação
			
			bufferOut
			.write("	<a href=\"/painel-acessibilidade\">Voltar ao painel de acessibilidade</a>\n");
			bufferOut.write(" <div class=\"avaliacao\"> ");
			
			bufferOut.write(" <h2 id=\"tituloTabela\">Outros erros mais comuns</h2>");
			//inicio tabela						
			bufferOut.write(" <table> ");
			bufferOut.write(" <thead> ");
			bufferOut.write(" <tr><th   colspan=\"3\"> Testes Automáticos</th></tr>");
			bufferOut.write(" <tr> ");
			bufferOut.write(" <th id=\"erro_numero\">Erro nº</th> ");
			bufferOut.write(" <th id=\"erro\">Erro</th> ");
			bufferOut.write(" <th id=\"incidencia\">Incidência</th> ");
			
			
			
			
			
			long total = 0;
			double percentual = 0;
			long totalDezErrosMaisComuns = 0;
			double qntErro = 0;
			int i = 0;
			int erro_numero = 0;
			
			for (ErrosMaisComuns errosMaisComuns : lsOutrosErrosMaisComuns) {
				i++;
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else if(i > 10)
				{
					
					qntErro = errosMaisComuns.getQtd_erros() * 100.00;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					DecimalFormat df2 = new DecimalFormat("0.##");
					
					erro_numero++;
					
					bufferOut.write(" <tr> ");
					bufferOut.write(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
					bufferOut.write(" <td headers=\"erro\" class=\"celula\"> " + errosMaisComuns.getDescricao() + " </td> ");
					bufferOut.write(" <td headers=\"incidencia\" class=\"celula\"> " + df2.format(percentual) + "% </td> ");
					bufferOut.write(" </tr> ");
				}
			}
			
			

			bufferOut.write(" </tbody> ");
			bufferOut.write(" </table> <br>");
		
						
			bufferOut.close();
			paginaGerada = true;
			//Termina criação da página
			
		} catch (UnsupportedEncodingException e) {
			paginaGerada = false;
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			paginaGerada = false;
			e.printStackTrace();
		} catch (IOException e) {
			paginaGerada = false;
			e.printStackTrace();
		}finally
		{
			return paginaGerada;
		}
	}
	
	public class CaminhoLink
	{
		private String nome;
		private String link;
		private String classe;
		private String media;
		
		public String getNome() {
			return nome;
		}
		public void setNome(String nome) {
			this.nome = nome;
		}
		public String getLink() {
			return link;
		}
		public void setLink(String link) {
			this.link = link;
		}
		public String getClasse() {
			return classe;
		}
		public void setClasse(String classe) {
			this.classe = classe;
		}
			
		
		
	}
	
}
