package br.com.eselo.controller;

import java.util.ArrayList;
import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.CopiadorParametroCalculo;
import br.com.eselo.component.Messages;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.AvaliacaoDAO;
import br.com.eselo.dao.CriterioDAO;
import br.com.eselo.dao.FaixaDAO;
import br.com.eselo.dao.ParametroCalculoDAO;
import br.com.eselo.dao.RecomendacaoDAO;
import br.com.eselo.model.Criterio;
import br.com.eselo.model.Faixa;
import br.com.eselo.model.ParametroCalculo;
import br.com.eselo.model.Recomendacao;

@Resource
public class FaixaController {

	private final FaixaDAO dao;
	private final CriterioDAO daoCriterio;
	private final ParametroCalculoDAO paDAO;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public FaixaController(FaixaDAO _dao,
			RecomendacaoDAO _daoRecomendacao, CriterioDAO _daoCriterio, ParametroCalculoDAO _daoParametro,
			Result _result, Validator _validator, UsuarioWeb _usuarioWeb) {
		this.dao = _dao;
		this.daoCriterio = _daoCriterio;
		this.paDAO = _daoParametro;
		this.result = _result;
		this.validator = _validator;
		this.usuarioWeb = _usuarioWeb;
	}

	public List<Faixa> lista(Long idCriterio) {
		return this.index(idCriterio, 1);
	}

	@Get("/faixas")
	@Restrito(descricao = "CONFORMIDADES_LISTA")
	public List<Faixa> index(Long idCriterio, int pagina) {
		CriterioDAO cDAO = new CriterioDAO(dao.getSession());
		Criterio criterioAtual = cDAO.load(idCriterio);

		// Paginação
		// Quantidade de registros por pagina
		int exibi = 7;

		// Informa a quantidade de registros
		int registros = dao.numLinhas(criterioAtual);

		// Quantidade de paginas na paginação
		int totalPag = registros / exibi;

		// Se houver resto na divisao acima acrescenta uma pagina na paginação
		if ((registros - (totalPag * exibi)) > 0)
			totalPag++;

		// Calculo do registro inicial e final
		int regIni = ((exibi * pagina) - exibi);
		int regFim = (regIni + exibi);

		if (regFim > registros) {
			regFim = registros;
		}

		// Fim paginação
		List<Faixa> resultado = dao.listAllAtivas(criterioAtual);
		List<Faixa> paginacao = new ArrayList<Faixa>();

		for (int i = regIni; i < regFim; i++) {
			paginacao.add(resultado.get(i));
		}

		result.include("paginas", totalPag);
		result.include("registros", registros);
		result.include("regInic", ++regIni);
		result.include("regFim", regFim);
		result.include("controle", "diretriz");
		result.include("parametroAvaliacao", criterioAtual.getRecomendacao().getParametroCalculo());

		return paginacao;
	}


	@Get("/faixas/gerarForm/{diretrizId}/{quantidade}")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void gerarForm(Long diretrizId, int quantidade) {
		result.include("quantidadeFaixas", quantidade);
		result.forwardTo(this).add(diretrizId);
	}

	@Get("/faixas/{criterioId}")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void add(Long criterioId) {
		Criterio criterio = daoCriterio.load(criterioId);
		result.include("recomendacaoId", criterio.getRecomendacao().getId());
		result.include("criterioId", criterioId);
		result.include("criterio", criterio);
		result.include("parametroCalculo", criterio.getRecomendacao().getParametroCalculo());
		populateSelectLists();
	}

	@Get("/faixas/form/{id}")
	@Restrito(descricao = "CONFORMIDADES_ATUALIZAR")
	public Faixa form(Long id) {
		Faixa faixa = dao.load(id);

		result.include("recomendacaoId", faixa.getCriterio().getRecomendacao().getId());
		result.include("criterioId", faixa.getCriterio().getId());
		result.include("parametroCalculo", faixa.getCriterio().getRecomendacao().getParametroCalculo());
		populateSelectLists();

		return faixa;
	}

	@Post("/faixas/add")
	@Restrito(descricao = "CONFORMIDADES_ADICIONAR")
	public void adicionar(List<Faixa> faixas) {

		Criterio criterio = daoCriterio.load(faixas.get(0).getCriterio().getId());

		for (Faixa faixa : faixas) {
			faixa.setCriterio(criterio);
			//criterioId = faixa.getCriterio().getId();
			validarFaixa(faixa);

			validator.onErrorUsePageOf(FaixaController.class).add(
					faixa.getCriterio().getId());
		}
		
		ParametroCalculo pc = paDAO.load(criterio.getRecomendacao().getParametroCalculo().getId());
		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			
			for (Faixa faixa : faixas) {
				dao.save(faixa);
			}
			result.include("mensagem", Messages.getString("faixas.mensagem.salva"));
			result.redirectTo(CriteriosController.class).form(criterio.getId());
		}else{
			this.criarNovoParametroAvaliacao(faixas,ParametroCalculoController.MODO_ADD);
			result.include("mensagem", Messages.getString("faixas.mensagem.salva")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
			result.redirectTo(ParametroCalculoController.class).index(1);
		}
	}

	@Put("/faixas")
	@Restrito(descricao = "CONFORMIDADES_ATUALIZAR")
	public void atualizar(Faixa faixa) {
		Faixa faixaCadastrada = dao.load(faixa.getId());

		Criterio criterio = daoCriterio.load(faixa.getCriterio().getId());

		result.include("recomendacaoId", criterio.getRecomendacao().getId());
		result.include("criterioId", criterio.getId());
		result.include("criterio", criterio);
		result.include("parametroCalculo", criterio.getRecomendacao().getParametroCalculo());

		validarFaixa(faixa);

		validator.onErrorRedirectTo(FaixaController.class).form(faixaCadastrada.getId());

		if(validarAtualizacaoFaixa(faixa)){
			
			ParametroCalculo pc = paDAO.load(criterio.getRecomendacao().getParametroCalculo().getId());
			if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
				faixaCadastrada.setCriterio(faixa.getCriterio());
				faixaCadastrada.setFaixaInicio(faixa.getFaixaInicio());
				faixaCadastrada.setFaixaFim(faixa.getFaixaFim());
				faixaCadastrada.setDescricao(faixa.getDescricao());
				faixaCadastrada.setPercentualConformidade(faixa.getPercentualConformidade());

				dao.update(faixaCadastrada);
				result.include("mensagem",	Messages.getString("faixas.mensagem.alterada"));
				result.redirectTo(CriteriosController.class).form(criterio.getId());
			}else{
				List<Faixa> faixas = new ArrayList<Faixa>();
				faixa.setCriterio(faixaCadastrada.getCriterio());
				faixas.add(faixa);
				
				criarNovoParametroAvaliacao(faixas, ParametroCalculoController.MODO_EDIT);
				result.include("mensagem",	Messages.getString("faixas.mensagem.alterada")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
				result.redirectTo(ParametroCalculoController.class).index(1);
			}
		}else{
			faixaCadastrada.setDescricao(faixa.getDescricao());
			dao.update(faixaCadastrada);
			result.include("mensagem", Messages.getString("validacao.sem_alteracao"));
			result.redirectTo(FaixaController.class).form(faixa.getId());	
		}
	}

	@Path("/faixas/delete/{id}")
	@Restrito(descricao = "CONFORMIDADES_EXCLUIR")
	public void excluir(Long id) {
		Faixa faixa = dao.load(id);
		long idCriterio = faixa.getCriterio().getId();
		
		List<Faixa> faixas = new ArrayList<Faixa>();
		faixas.add(faixa);
		
		ParametroCalculo pc = paDAO.load(faixa.getCriterio().getRecomendacao().getParametroCalculo().getId());

		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			dao.remove(faixa.getId());
			result.include("mensagem", Messages.getString("faixas.mensagem.excluida"));
			result.redirectTo(CriteriosController.class).form(idCriterio);
		}else{
			criarNovoParametroAvaliacao(faixas,ParametroCalculoController.MODO_DELETE);
			result.include("mensagem", Messages.getString("faixas.mensagem.excluida")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
			result.redirectTo(ParametroCalculoController.class).index(1);
		}
	}

	private void populateSelectLists() {
		result.include("faixaList", dao.getAllRecomendacao());
	}

	private void validarFaixa(Faixa faixa) {
		if (faixa.getCriterio().getId() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.faixa"),
					"faixa.diretriz.id"));
		} else if (faixa.getFaixa() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.faixa"),
					"faixa.faixa"));
		} else if (faixa.getDescricao() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.descricao"),
					"faixa.descricao"));
		} else if (faixa.getFaixaInicio() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.faixa_inicial"),
					"faixa.faixaInicio"));
		} else if (faixa.getFaixaFim() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.faixa_final"),
					"faixa.faixaFim"));
		} else if (faixa.getFaixaInicio() > faixa.getFaixaFim()) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.range_faixas"),
					"faixa.faixaInicio"));
		} else if (faixa.getPercentualConformidade() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("faixas.validacao.percentual_conformidade"),
					"faixa.percentualConformidade"));
		}
	}

	private boolean validarAtualizacaoFaixa(Faixa faixa) {

		boolean alteracao = false;

		Faixa faixaCadastrada = dao.load(faixa.getId());

		if (faixa.getFaixa().compareTo(faixaCadastrada.getFaixa())!=0) {
			alteracao = true;
		} else if (faixa.getDescricao().compareToIgnoreCase(faixaCadastrada.getDescricao()) != 0) {
			alteracao = true;
		} else if (faixa.getFaixaInicio().compareTo(faixaCadastrada.getFaixaInicio())!=0) {
			alteracao = true;
		} else if (faixa.getFaixaFim().compareTo(faixaCadastrada.getFaixaFim()) !=0) {
			alteracao = true;
		} else if (faixa.getPercentualConformidade().compareTo(faixaCadastrada.getPercentualConformidade()) !=0) {
			alteracao = true;
		}

		return alteracao;
	}

	private void criarNovoParametroAvaliacao(List<Faixa> faixas, int tipo) {
		ParametroCalculo ultimoParametroAvaliacao = new CopiadorParametroCalculo().copiarParametroCalculo(paDAO.getLastParametrosCalculoAtivo());
		boolean ok = false;
		Faixa faixa;

		switch (tipo) {
			case ParametroCalculoController.MODO_ADD:
				for(Faixa f:faixas){
					for(Recomendacao recomendacao: ultimoParametroAvaliacao.getRecomendacoes()){
						if(recomendacao.getNumeroRecomendacao().longValue()==f.getCriterio().getRecomendacao().getNumeroRecomendacao().longValue()){
							for(Criterio criterio: recomendacao.getCriterios()){
								if(criterio.getNumeroCriterio().longValue()==f.getCriterio().getNumeroCriterio().longValue()){
									criterio.addFaixa(f);
									ok=true;
									break;

								}
							}
						}
					}
				}

				break;

			case ParametroCalculoController.MODO_EDIT:
				//ultimoParametroAvaliacao = paDAO.getLastParametrosCalculo();
				faixa = faixas.get(0);
				for(Recomendacao recomendacao: ultimoParametroAvaliacao.getRecomendacoes()){
					if(recomendacao.getNumeroRecomendacao().longValue()==faixa.getCriterio().getRecomendacao().getNumeroRecomendacao().longValue()){
						for(Criterio criterio: recomendacao.getCriterios()){
							if(criterio.getNumeroCriterio().longValue()==faixa.getCriterio().getNumeroCriterio().longValue()){
								for(Faixa f: criterio.getFaixas()){
									if(f.getFaixa().longValue()==faixa.getFaixa().longValue()){
										f.setCriterio(faixa.getCriterio());
										f.setDescricao(faixa.getDescricao());
										f.setFaixa(faixa.getFaixa());
										f.setFaixaFim(faixa.getFaixaFim());
										f.setFaixaInicio(faixa.getFaixaInicio());
										f.setId(faixa.getId());
										f.setPercentualConformidade(faixa.getPercentualConformidade());
										ok=true;
									}
								}
							}
						}
					}
				}

				break;

			case ParametroCalculoController.MODO_DELETE:
				faixa = faixas.get(0);
				for(Recomendacao recomendacao: ultimoParametroAvaliacao.getRecomendacoes()){
					if(recomendacao.getNumeroRecomendacao().longValue()==faixa.getCriterio().getRecomendacao().getNumeroRecomendacao().longValue()){
						for(Criterio criterio: recomendacao.getCriterios()){
							if(criterio.getNumeroCriterio().longValue()==faixa.getCriterio().getNumeroCriterio().longValue()){
								for(int i=0; i< criterio.getFaixas().size(); i++){
									if(faixa.getFaixa().longValue() == criterio.getFaixas().get(i).getFaixa().longValue()){
										criterio.getFaixas().remove(i);
										ok=true;
										break;
									}

								}
							}
						}
					}
				}

				break;
		}

		if(ok){
		new CopiadorParametroCalculo().criarNovoParametroCalculo(ultimoParametroAvaliacao, usuarioWeb, dao.getSession());
		}
	}

}
