package br.com.eselo.controller;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.ioc.spring.VRaptorRequestHolder;
import br.com.eselo.component.Mailer;
import br.com.eselo.component.Util;
import br.com.eselo.dao.ErrosMaisComunsDAO;
import br.com.eselo.dao.MediaAvaliacaoDAO;
import br.com.eselo.dao.RelatoriosFadDAO;
import br.com.eselo.dao.UrlDAO;
import br.com.eselo.model.ErrosMaisComuns;
import br.com.eselo.model.GraficoMediaAvaliacao;
import br.com.eselo.model.MediaAvaliacao;
import br.com.eselo.model.RelatoriosFad;
import br.com.eselo.model.UltimaAvaliacao;

@Resource
public class PainelAcessibilidadeController {

	private static final Logger LOGGER = LoggerFactory
			.getLogger(AvaliacaoController.class);
	 private Result result;
	 private Validator validator;
	 private final UrlDAO daoUrl;
	 private final ErrosMaisComunsDAO errosMaisComunsDAO;
	 private final MediaAvaliacaoDAO mediaAvaliacaoDAO;
	 private final RelatoriosFadDAO relatoriosFadDAO;
	 private static final String PAINEL_AGREGADO = "eselo.config.avaliacao.painelAgregado";
	 private final Properties properties = new Properties();
	 private ServletContext application;
	 
	 private static final String HOST = "eselo.config.buscaRelatorioFAD.host";
	 private static final String USUARIO = "eselo.config.buscaRelatorioFAD.usuario";
	 private static final String SENHA = "eselo.config.buscaRelatorioFAD.senha";
	 private static final String DBNAME = "eselo.config.buscaRelatorioFAD.dbname";
	 
	public PainelAcessibilidadeController(ErrosMaisComunsDAO _errosMaisComunsDAO, Result result, Validator validator, UrlDAO _dao, MediaAvaliacaoDAO _mediaAvaliacaoDAO, ServletContext application, RelatoriosFadDAO _relatoriosFadDAO) throws IOException{
		
		this.errosMaisComunsDAO = _errosMaisComunsDAO;
		this.result = result;
	    this.validator = validator;
	    this.daoUrl = _dao;
	    this.mediaAvaliacaoDAO = _mediaAvaliacaoDAO;
	    this.application = application;
	    this.relatoriosFadDAO = _relatoriosFadDAO;
	    
	    String fileName = "config.properties";
		InputStream stream = Mailer.class.getResourceAsStream("/" + fileName);
		Properties propertiesFile = new Properties();

		if (stream != null) {
			propertiesFile.load(stream);
			this.properties.putAll(propertiesFile);
		} else {
			LOGGER.warn("Could not find the file '{}' to load.", fileName);
		}
	}
	
	@Get("/erros-mais-comuns")
	public void errosMaisComuns() {
		        
		
		File diretorioHtml = new File(properties.getProperty(PAINEL_AGREGADO) + "/painelAgregado.txt");
		
		StringBuilder html = new StringBuilder();
		try {
			BufferedReader myBuffer = new BufferedReader(new InputStreamReader(new FileInputStream(diretorioHtml), "UTF-8"));
			String linha;
			
			
			while ( ( linha = myBuffer.readLine() ) != null) {
				html.append(linha);
			}
			myBuffer.close();
			
			 result.include("html", html);
			 result.include("grafico_js", "<script src=\"/static/js/chartPainelAgregado.js\"></script>");
			 
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
	}
	
	@Get("/outros-erros-mais-comuns")
	public void outrosErrosMaisComuns() {
		
		File diretorioHtml = new File(properties.getProperty(PAINEL_AGREGADO) + "/painelOutrosErrosMaisComuns.txt");
		
		StringBuilder html = new StringBuilder();
		try {
			BufferedReader myBuffer = new BufferedReader(new InputStreamReader(new FileInputStream(diretorioHtml), "UTF-8"));
			String linha;
			
			
			while ( ( linha = myBuffer.readLine() ) != null) {
				html.append(linha);
			}
			myBuffer.close();
			
			 result.include("html", html);
			 result.include("grafico_js", "<script src=\"/static/js/chartPainelAgregado.js\"></script>");
			 
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		
	}
	
	
	

	
	
	
	@Get
	@Path("/media-avaliacao/{tipo}/{id}/{nome}/{media}")
	public void mediaAvaliacao(String tipo, String id,String nome, Double media) {

		String novoTipo = "";
		String nomeTotal = "";
		String tipoGrafico = "";
		String legenda = "Médias ";
		
		
		List<MediaAvaliacao> lsMediaAvaliacao = mediaAvaliacaoDAO.mediaAvaliacao(tipo, id);
		
		//List<String> caminhoAvaliacao = new ArrayList<String>();
		
		List<CaminhoLink> lsCaminhoAvaliacao = (List<CaminhoLink>) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute("lsCaminhoAvaliacao");
		
				
		CaminhoLink caminhoLink = new CaminhoLink();
		
		if(lsCaminhoAvaliacao == null)
		{
			lsCaminhoAvaliacao = new ArrayList<CaminhoLink>();
			//caminhoLink.setNome("Órgão: " + nome);
			//caminhoLink.setLink("/eselo/orgao");
			//lsCaminhoAvaliacao.add(caminhoLink);
		}
				
		boolean existe = false;
		
		if(!tipo.equalsIgnoreCase("dominio"))
		{
			VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("exibirTabelaRelatorio", false);
		}
		switch (tipo) {
		
		case "orgao": novoTipo="dominio";
		tipo = "Órgãos";		
		caminhoLink = new CaminhoLink();
		caminhoLink.setNome("Órgão: " + nome + " - " + media);
		caminhoLink.setLink("/media-avaliacao/dominio/"+ id + "/" + nome + "/" +  media);
		
		
		
		/*for (CaminhoLink i : lsCaminhoAvaliacao) {
			if(i.getNome().equalsIgnoreCase(caminhoLink.getNome()))
					{
				existe = true;
					}
		}
		
		if(!existe)
		{
		lsCaminhoAvaliacao.add(caminhoLink);
		}*/
		
	for (int i = 0; i < lsCaminhoAvaliacao.size(); i++) {
			
			if (lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("orgao") || lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("dominio") || lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("sitio")) {
				lsCaminhoAvaliacao.remove(i);
				i--;
				for (CaminhoLink i2 : lsCaminhoAvaliacao) {
					if(i2.getNome().equalsIgnoreCase(caminhoLink.getNome()))
							{
						existe = true;
							}
				}
				
				
			    }
		}
	
		
		if(!existe)
		{
		lsCaminhoAvaliacao.add(caminhoLink);
		}
		
	
		
		break;
			
		case "dominio": novoTipo="sitio";		
		tipo = "Domínios";
		nomeTotal = "sítios";
		tipoGrafico = "orgao";
		legenda += "do Órgão: " + nome;
		caminhoLink = new CaminhoLink();
		caminhoLink.setNome("Órgão: " + nome + " - " + media);
		caminhoLink.setLink("/media-avaliacao/dominio/"+ id + "/" +  nome + "/" +  media);
		caminhoLink.setClasse("orgao");
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("exibirTabelaRelatorio", true);
		buscarRelatorios(id);
		existe = false;
		
		for (int i = 0; i < lsCaminhoAvaliacao.size(); i++) {
			
			if (lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("orgao") || lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("dominio") || lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("sitio")) {
				lsCaminhoAvaliacao.remove(i);
				i--;
				for (CaminhoLink i2 : lsCaminhoAvaliacao) {
					if(i2.getNome().equalsIgnoreCase(caminhoLink.getNome()))
							{
						existe = true;
							}
				}
				
				
			    }
		}
	
		
		if(!existe)
		{
		lsCaminhoAvaliacao.add(caminhoLink);
		}
	
		
		break;
		case "sitio": novoTipo="url";
		tipo = "Sítios";		
		nomeTotal = "Páginas";
		tipoGrafico = "dominio";
		legenda += "do Domínio: " + nome;
		caminhoLink = new CaminhoLink();
		caminhoLink.setNome("Domínio: " + nome + " - " + media);
		caminhoLink.setLink("/media-avaliacao/sitio/"+ id + "/" +  nome + "/" +  media);
		caminhoLink.setClasse("dominio");
		
		existe = false;
		
	for (int i = 0; i < lsCaminhoAvaliacao.size(); i++) {
			
			if (lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("sitio")) {
				lsCaminhoAvaliacao.remove(i);
				i--;
			    }
		}
	
		for (CaminhoLink i : lsCaminhoAvaliacao) {
			if(i.getNome().equalsIgnoreCase(caminhoLink.getNome()))
					{
				existe = true;
					}
		}
		
		if(!existe)
		{
		lsCaminhoAvaliacao.add(caminhoLink);
		}
		
		
	
		
		break;
		case "url": novoTipo="url";
		tipo = "Páginas";
		nomeTotal = "páginas";
		tipoGrafico = "sitio";
		legenda += "do Sítio: " + nome;
		caminhoLink = new CaminhoLink();
		caminhoLink.setNome("Sítio: " + nome + " - " + media);	
		caminhoLink.setLink("/media-avaliacao/url/"+ id + "/" +  nome + "/" +  media);
		caminhoLink.setClasse("sitio");
		
		existe = false;
		
	for (int i = 0; i < lsCaminhoAvaliacao.size(); i++) {
			
			if (lsCaminhoAvaliacao.get(i).getClasse().equalsIgnoreCase("url")) {
				lsCaminhoAvaliacao.remove(i);
				i--;
			    }
		}
	
		for (CaminhoLink i : lsCaminhoAvaliacao) {
			if(i.getNome().equalsIgnoreCase(caminhoLink.getNome()))
					{
				existe = true;
					}
		}
		
		if(!existe)
		{
		lsCaminhoAvaliacao.add(caminhoLink);
		}
		
		break;
		
		default:
			break;
		}
		
		
		//tipo = tipo.substring(0,1).toUpperCase().concat(tipo.substring(1));
		result.include("lsMediaAvaliacao", lsMediaAvaliacao);
		result.include("tipo", tipo);
		result.include("legenda", legenda);
		result.include("novoTipo", novoTipo);
		result.include("nomeTotal", nomeTotal);
		
		
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("lsCaminhoAvaliacao", lsCaminhoAvaliacao);
		gerarGrafico(tipoGrafico, id, legenda, false, media);
		
		
	}
	
	@Get
	@Path("/media-avaliacao/{tipo}")
	public void mediaAvaliacao(String tipo) {
		
		String novoTipo = "";
			
		UltimaAvaliacao dataUltimaAvaliacao = mediaAvaliacaoDAO.ultimaAvaliacao();
		List<MediaAvaliacao> lsMediaAvaliacao = mediaAvaliacaoDAO.mediaAvaliacao(tipo, null);
		
		
		result.include("lsMediaAvaliacao", lsMediaAvaliacao);
		result.include("tipo", "Órgãos");
		result.include("novoTipo", "dominio");
		result.include("nomeTotal", "domínios");
		result.include("legenda", "Média Brasil: gov.br");
		
		gerarGrafico("governo federal", null, "Média Brasil: gov.br", true, 0);
		
		
		
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("ultimaAvaliacao", Util.convertDataBrasil(dataUltimaAvaliacao.getData_ultima_avaliacao()));
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("exibirTabelaRelatorio", false);
		
	}
	
	
	private double buscaMediaBrasil(List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao)
	{
		//
		
		double mediaBrasil = 0;
		int qntAvaliacoes = 0;
		
		BigDecimal gigDecimalMedia;
		
		for (GraficoMediaAvaliacao graficoMediaAvaliacao : lsGraficoMediaAvaliacao) {
			
			qntAvaliacoes++;
			mediaBrasil +=  graficoMediaAvaliacao.getMediaGrafico().doubleValue();
		}
		
		List<CaminhoLink> lsCaminhoAvaliacao = new ArrayList<CaminhoLink>();
		CaminhoLink caminhoLink = new CaminhoLink();
		
		mediaBrasil = mediaBrasil/qntAvaliacoes;
		
		caminhoLink.setNome("Média Brasil - " + Util.formataCasasDecimais(mediaBrasil));
		caminhoLink.setLink("/media-avaliacao/orgao");
		caminhoLink.setClasse("mBrasil");
		lsCaminhoAvaliacao.add(caminhoLink);
		
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("lsCaminhoAvaliacao", lsCaminhoAvaliacao);
		
					
		return Util.formataCasasDecimais(mediaBrasil);
	}
	
	private void gerarGrafico(String tipo, String id, String legenda, boolean retornarMediaBrasil, double mediaBrasil)
	{		
		List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao = mediaAvaliacaoDAO.gerarGraficoMediaAvaliacao(tipo, id, 4);
		
				
		if(retornarMediaBrasil)
		{
		mediaBrasil = buscaMediaBrasil(lsGraficoMediaAvaliacao);
		VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("mediaBrasil", mediaBrasil);
		}
		
		
		String webaxscore = "";
		
		  if (mediaBrasil >= 95) {
		        webaxscore = "bom";      
		      } else if (mediaBrasil >= 85 & mediaBrasil < 95) {
		          webaxscore = "razoavel";
		      } else if (mediaBrasil >= 70 & mediaBrasil < 85) {
		          webaxscore = "ruim";
		      }else {
		        webaxscore = "pessimo";
		      }
		
		
		StringBuffer html = new StringBuffer();
		
		html.append(" <div class=\"logoAvaliacao\"> ");
		html.append(" <div id=\"emag\"> ");		
		html.append(" 	<label id=\"centralizar\">eMAG v 3.1</label> ");		
		html.append(" </div> ");
		
		html.append("		<div id=\"testesAutomaticos\">					\n");
		//bufferOut.write("				<label>Testes Automáticos<br> Nota: " + resumoAvaliacao.getValor() + "/100</label>	</div>	");
		
		html.append("			<div class=\"cell width-5 position-2 >\n");
		
		
		html.append("				<div id=\"block\">\n");
		html.append("					<div id=\"webaxscore\" class=\"scoreB_" + webaxscore + "\">Porcentagem");
		html.append("						<strong>Média</strong>");
		html.append("						<span>" + Util.formataCasasDecimais(mediaBrasil) +"%</span>\n");
		html.append("					</div>\n");
		html.append("				</div>\n");
		
		//bufferOut.write("				<br>");
		
		
		html.append("<div id=\"legenda\" >\n");
		html.append("<label>Legenda</label><br>");
		html.append("<ul>");
		html.append("	<li class=\"legenda\"><img id=\"bom\" alt=\"farol verde, média maior ou igual a 95%\" src=\"/static/img/verde.png\"></li>");
		html.append("	<li class=\"legenda\"><img id=\"razoavel\" alt=\"farol amarela, média maior ou igual a 85% e menor que 95%\" src=\"/static/img/amarelo.png\"></li>");
		html.append("	<li class=\"legenda\"><img id=\"ruim\" alt=\"farol laranja, média maior ou igual a 70% e menor que 85%\" src=\"/static/img/laranja.png\"></li>");
		html.append("	<li class=\"legenda\"><img id=\"pessimo\" alt=\"farol vermelho, média menor que 70%\" src=\"/static/img/vermelho.png\"></li>");
		html.append("</ul>");
		html.append("</div>");
		
		html.append("			</div>");
		
		html.append(" <div id=\"grafico2\"> ");
		html.append(" 	<h2 class=\"oculto_visao\">Gráfico com as notas das últimas avaliações ordenada da mais antiga para a mais atual, limitado até quatro avaliações.</h2> ");
		html.append(" 	<ul class=\"oculto_visao\"> ");
			
		int qntAvaliacoes = 0;
		
		String dataAvaliacao = "";
		
		
		
for (GraficoMediaAvaliacao graficoMediaAvaliacao : lsGraficoMediaAvaliacao) {
	
	
	mediaBrasil += graficoMediaAvaliacao.getMediaGrafico().doubleValue();
	
	qntAvaliacoes++;
	
	//dataAvaliacao = graficoMediaAvaliacao.getIdAgendamento().substring(6, 8) + "/";
	dataAvaliacao = graficoMediaAvaliacao.getId_agendamentoGrafico().substring(4, 6) + "/";
	dataAvaliacao += graficoMediaAvaliacao.getId_agendamentoGrafico().substring(0, 4);
	
	html.append(" <li>Média "+ qntAvaliacoes + ": <span id=\"media"+ qntAvaliacoes +"\">"+graficoMediaAvaliacao.getMediaGrafico().doubleValue()+"</span><span>Data da avaliação: </span><span id=\"data" + qntAvaliacoes + "\">"+ dataAvaliacao +"</span></li>");
			
		}


		html.append("  	</ul> ");
		html.append(" 	<label id=\"qntAvaliacoes\" class=\"oculto\">"+ qntAvaliacoes +"</label> ");
		html.append(" 	<label id=\"qntAvaliacoes\" >"+ legenda +"</label> ");
		
		html.append(" 	<div class=\"myChart\"> ");
		html.append(" 		<iframe class=\"chartjs-hidden-iframe\" ");
		html.append(" 			style=\"width: 100%; display: block; border: 0px; height: 0px; margin: 0px; position: absolute; left: 0px; right: 0px; top: 0px; bottom: 0px;\"></iframe> ");
		html.append(" 		<canvas id=\"myChartMediaAvaliacao\" style=\"width: 420px; height: 210px;\" ");
		html.append(" 			width=\"420\" height=\"210\"></canvas> ");
		html.append(" 	</div> ");
		html.append(" </div> ");
		html.append(" </div> ");
		
		result.include("grafico_js", "<script type=\"text/javascript\" src=\"/static/js/chartMediaAvaliacao.js\"></script>");
		result.include("html", html.toString());
		
		
	}
	

	
	//@Get
	//@Path("/painel-acessibilidade/{id}")
	//public void painelAcessibilidade(int id) {
	public void buscarRelatorios(String id) {
		
		
        Map<String,byte[]> relatorio = new HashMap<String,byte[]>();
        Map<String,byte[]> planoTrabalho = new HashMap<String,byte[]>();
        
        List<RelatoriosFad> lsRelatoriosFad = relatoriosFadDAO.retornarRelatoriosFad(id, properties.getProperty(HOST),  properties.getProperty(DBNAME), properties.getProperty(USUARIO), properties.getProperty(SENHA));

        for (RelatoriosFad relatoriosFad : lsRelatoriosFad) {
        	relatorio.put(relatoriosFad.getId().toString(), relatoriosFad.getArquivo_fad());
        	planoTrabalho.put(relatoriosFad.getId().toString(), relatoriosFad.getArquivo_plano());
        }
        
          
       VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("lsRelatoriosFad", lsRelatoriosFad);
       VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("relatorio", relatorio);
       VRaptorRequestHolder.currentRequest().getServletContext().setAttribute("planoTrabalho", planoTrabalho);
       
     //serve para escolher qual tabela carregar(orgão ou os domínios com seus relatórios para download
		result.include("carregamento", "dominios");
       
	}
	
	@Get
	@Path("/painel-acessibilidade/{id}/{relatorio_plTrabalho}")
	public void painelAcessibilidade(String id, String relatorio_plTrabalho, HttpServletResponse response) {
		Map<String,byte[]> relatorio;
		Map<String,byte[]> planoTrabalho;
		
		 String keyToSearch = id;
		 byte[] dados = null;
		 String nome = "";
		if(relatorio_plTrabalho.equalsIgnoreCase("fad"))
		{
			relatorio = (Map<String, byte[]>) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute("relatorio");
			
			  if ( relatorio.containsKey( keyToSearch ) ) {

				  try {
						dados = relatorio.get(keyToSearch);
						nome = "Relatório Circunstanciado.pdf";
						 
						response.setContentType("inline/download");
						String arq = "attachment;filename=" + nome;
						response.setHeader("Content-Disposition", arq);
						ServletOutputStream os;
						
						os = response.getOutputStream();
						os.write(dados);
						os.flush();
						os.close();
						 
					} catch (IOException e) {
						e.printStackTrace();
					}
					          
	       }else{
	              System.err.println("Chave não existe");
	       }
			
		}else if(relatorio_plTrabalho.equalsIgnoreCase("plano"))
		{
			planoTrabalho = (Map<String, byte[]>) VRaptorRequestHolder.currentRequest().getServletContext().getAttribute("planoTrabalho");
			
			 if ( planoTrabalho.containsKey( keyToSearch ) ) {				 
			
				try {
					dados = planoTrabalho.get(keyToSearch);
					nome = "Plano de trabalho.pdf";
					 
					response.setContentType("inline/download");
					String arq = "attachment;filename=" + nome;
					response.setHeader("Content-Disposition", arq);
					ServletOutputStream os;
					
					os = response.getOutputStream();
					os.write(dados);
					os.flush();
					os.close();
					 
				} catch (IOException e) {
					e.printStackTrace();
				}				
			            
			       }else{
			              System.err.println("Chave não existe");
			       }
			
		}
       
        
		result.include("grafico_js", "<script type=\"text/javascript\" src=\"/static/js/chartMediaAvaliacao.js\"></script>");
	}
	
	@Get("/painel-acessibilidade")
	public void painelAcessibilidade() {


		//List<ErrosMaisComuns> lsErrosMaisComuns = errosMaisComunsDAO.errosMaisComuns();
		//File painelAgregado = new File(properties.getProperty(PAINEL_AGREGADO));
		StringBuffer html = new StringBuffer();
		
		Double mediaBrasil;
		
		//List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao = mediaAvaliacaoDAO.gerarGraficoMediaAvaliacao("governo federal", null, 4);
		
		
	
		//mediaBrasil = buscaMediaBrasil(lsGraficoMediaAvaliacao);
		
		
		
		/*String webaxscore = "";
		
		  if (mediaBrasil >= 95) {
		        webaxscore = "bom";      
		      } else if (mediaBrasil >= 85 & mediaBrasil < 95) {
		          webaxscore = "razoavel";
		      } else if (mediaBrasil >= 70 & mediaBrasil < 85) {
		          webaxscore = "ruim";
		      }else {
		        webaxscore = "pessimo";
		      }*/
	
		 //inicio Erros mais comuns
		  
		 
			
			//StringBuilder html = new StringBuilder();
			try {
				List<GraficoMediaAvaliacao> lsGraficoMediaAvaliacao = mediaAvaliacaoDAO.gerarGraficoMediaAvaliacao("governo federal", null, 4);
				mediaBrasil = buscaMediaBrasil(lsGraficoMediaAvaliacao);

				 File diretorioHtml = new File(properties.getProperty(PAINEL_AGREGADO) + "/painelAgregadoComMediaBrasil.txt");
				
				BufferedReader myBuffer = new BufferedReader(new InputStreamReader(new FileInputStream(diretorioHtml), "UTF-8"));
				String linha;
				
				
				while ( ( linha = myBuffer.readLine() ) != null) {
					html.append(linha);
				}

				myBuffer.close();
				
				 result.include("html", html);
				 
			} catch (FileNotFoundException e1) {
				e1.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}

		  //fim Erros mais comuns
		
		 /* html.append(" <div class=\"logoAvaliacao\"> ");
			html.append(" <div id=\"emag\"> ");		
			html.append(" 	<label  id=\"centralizar\">eMAG v 3.1</label> ");		
			html.append(" </div> ");
			
			html.append("		<div id=\"testesAutomaticos\">					\n");
			//bufferOut.write("				<label>Testes Automáticos<br> Nota: " + resumoAvaliacao.getValor() + "/100</label>	</div>	");
			
			html.append("			<div class=\"cell width-5 position-2 >\n");
			
			
			html.append("				<div id=\"block\">\n");
			html.append("					<div id=\"webaxscore\" class=\"scoreB_" + webaxscore + "\">Porcentagem");
			html.append("						<strong>Média</strong>");
			html.append("						<span>" + Util.formataCasasDecimais(mediaBrasil) +"%</span>\n");
			html.append("					</div>\n");
			html.append("				</div>\n");
			
			//bufferOut.write("				<br>");
			
			
			html.append("<div id=\"legenda\" >\n");
			html.append("<label>Legenda</label><br>");
			html.append("<ul>");
			html.append("	<li class=\"legenda\"><img id=\"bom\" alt=\"farol verde, média maior ou igual a 95%\" src=\"/static/img/verde.png\"></li>");
			html.append("	<li class=\"legenda\"><img id=\"razoavel\" alt=\"farol amarela, média maior ou igual a 85% e menor que 95%\" src=\"/static/img/amarelo.png\"></li>");
			html.append("	<li class=\"legenda\"><img id=\"ruim\" alt=\"farol laranja, média maior ou igual a 70% e menor que 85%\" src=\"/static/img/laranja.png\"></li>");
			html.append("	<li class=\"legenda\"><img id=\"pessimo\" alt=\"farol vermelho, média menor que 70%\" src=\"/static/img/vermelho.png\"></li>");
			html.append("</ul>");
			html.append("</div>");
			
			html.append("			</div>");
			
			html.append(" <div id=\"grafico2\"> ");
			html.append(" 	<h2 class=\"oculto_visao\">Gráfico com as notas das últimas avaliações ordenada da mais antiga para a mais atual, limitado até quatro avaliações.</h2> ");
			html.append(" 	<ul class=\"oculto_visao\"> ");
				
			int qntAvaliacoes = 0;
			
			String dataAvaliacao = "";
			
			
			
	for (GraficoMediaAvaliacao graficoMediaAvaliacao : lsGraficoMediaAvaliacao) {
		
		mediaBrasil += graficoMediaAvaliacao.getMediaGrafico().doubleValue();
		
		qntAvaliacoes++;
		
		//dataAvaliacao = graficoMediaAvaliacao.getIdAgendamento().substring(6, 8) + "/";
		dataAvaliacao = graficoMediaAvaliacao.getId_agendamentoGrafico().substring(4, 6) + "/";
		dataAvaliacao += graficoMediaAvaliacao.getId_agendamentoGrafico().substring(0, 4);
		
		html.append(" <li>Média "+ qntAvaliacoes + ": <span id=\"media"+ qntAvaliacoes +"\">"+graficoMediaAvaliacao.getMediaGrafico().doubleValue()+"</span><span>Data da avaliação: </span><span id=\"data" + qntAvaliacoes + "\">"+ dataAvaliacao +"</span></li>");
				
			}


			html.append("  	</ul> ");
			html.append(" 	<label id=\"qntAvaliacoes\" class=\"oculto\">"+ qntAvaliacoes +"</label> ");
			//html.append(" 	<label id=\"qntAvaliacoes\" >"+ legenda +"</label> ");
			
			html.append(" 	<div class=\"myChart\"> ");
			html.append(" 		<iframe class=\"chartjs-hidden-iframe\" ");
			html.append(" 			style=\"width: 100%; display: block; border: 0px; height: 0px; margin: 0px; position: absolute; left: 0px; right: 0px; top: 0px; bottom: 0px;\"></iframe> ");
			html.append(" 		<canvas id=\"myChartMediaAvaliacao\" style=\"width: 420px; height: 210px;\" ");
			html.append(" 			width=\"420\" height=\"210\"></canvas> ");
			html.append(" 	</div> ");
			html.append(" </div> ");
			html.append(" </div> ");
			
			
			long total = 0;
			double percentual = 0;
			long totalDezErrosMaisComuns = 0;
			double qntErro;
			
			int i = 0;
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					//DecimalFormat df = new DecimalFormat("0.##");
					
					i++;
					//html.append("<li >erro: <span id=\"erro" + i +"\">" + "Erro: " + i + "</span><span>" + errosMaisComuns.getDescricao() + "</span> <span id=\"p_erro" + i + "\">" + df.format(percentual) + "</span></li>");				
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100;
			percentual = qntErro/total;	
			
			//DecimalFormat df = new DecimalFormat("0.##");
			
		
			
			//inicio Avaliação
			html.append(" <div class=\"avaliacao\"> ");
			
			
					html.append(" <h2 id=\"tituloTabela\">Os 10 erros mais comuns</h2><br>");
					//inicio tabela						
					html.append(" <table> ");
						html.append(" <thead> ");
							html.append(" <tr><th   colspan=\"3\"> Testes Automáticos</th></tr>");
							html.append(" <tr> ");
								html.append(" <th id=\"erro_numero\">Erro n°</th> ");
								html.append(" <th id=\"erro\">Erro</th> ");
								html.append(" <th id=\"incidencia\">Incidência</th> ");
			
			
			
			
			
			 total = 0;
			 percentual = 0;
			 totalDezErrosMaisComuns = 0;
			 qntErro = 0;
			 Integer erro_numero = 0;
			 
			for (ErrosMaisComuns errosMaisComuns : lsErrosMaisComuns) {
				
				if(errosMaisComuns.getDescricao().equalsIgnoreCase("Total"))
				{
					total = errosMaisComuns.getQtd_erros();
				}
				else
				{
					qntErro = errosMaisComuns.getQtd_erros() * 100.00;
					percentual = qntErro/total;
					totalDezErrosMaisComuns = totalDezErrosMaisComuns + errosMaisComuns.getQtd_erros();
					
					erro_numero++;
					
					//DecimalFormat df2 = new DecimalFormat("0.##");
							html.append(" <tr> ");
								html.append(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
								html.append(" <td headers=\"erro\" class=\"celula\"> " + errosMaisComuns.getDescricao() + " </td> ");
								html.append(" <td headers=\"incidencia\" class=\"celula\"> " + Util.formataCasasDecimais(percentual) + "% </td> ");
							html.append(" </tr> ");
				}
			}
			
			qntErro = (total - totalDezErrosMaisComuns) * 100.00;
			percentual = qntErro/total;	
			
			//DecimalFormat df2 = new DecimalFormat("0.##");
			
			erro_numero++;
			
						html.append(" <tr> ");
							html.append(" <td headers=\"erro_numero\" class=\"celula\"> " + erro_numero + " </td> ");
							html.append(" <td headers=\"erroMaisComum\" class=\"celula\"> <a href=\"/outros-erros-mais-comuns\"> Outros  </a></td> ");
							html.append(" <td headers=\"percentual\" class=\"celula\">" + Util.formataCasasDecimais(percentual) + "%</td> ");
						html.append(" <tr> ");

					html.append(" </tbody> ");
			html.append(" </table> <br>");
			html.append(" </div> ");*/
			//fim Avaliação
			
			
			//inicio Erros mais comuns
				
			String novoTipo = "";
			
			UltimaAvaliacao dataUltimaAvaliacao = mediaAvaliacaoDAO.ultimaAvaliacao();
			List<MediaAvaliacao> lsMediaAvaliacao = mediaAvaliacaoDAO.mediaAvaliacao("orgao", null);
			
			
			//serve para escolher qual tabela carregar(orgão ou os domínios com seus relatórios para download
			result.include("carregamento", "orgao");
			
			result.include("html", html.toString());
			result.include("lsMediaAvaliacao", lsMediaAvaliacao);
			result.include("tipo", "Órgãos");
			result.include("novoTipo", "dominio");
			result.include("nomeTotal", "domínios");
			result.include("legenda", "Média Brasil: gov.br");
			result.include("grafico_js", "<script type=\"text/javascript\" src=\"/static/js/chartMediaAvaliacao.js\"></script>");
			//gerarGrafico("governo federal", null, "Média Brasil: gov.br", true, 0);
			
			
	
			
			//fim Erros mais comuns

	}
	
	
	public class CaminhoLink
	{
		private String nome;
		private String link;
		private String classe;
		private String media;
		
		public String getNome() {
			return nome;
		}
		public void setNome(String nome) {
			this.nome = nome;
		}
		public String getLink() {
			return link;
		}
		public void setLink(String link) {
			this.link = link;
		}
		public String getClasse() {
			return classe;
		}
		public void setClasse(String classe) {
			this.classe = classe;
		}
			
		
		
	}
	
	
		
	}



