package br.com.eselo.controller;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.ioc.spring.VRaptorRequestHolder;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.CopiadorParametroCalculo;
import br.com.eselo.component.Messages;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.AvaliacaoDAO;
import br.com.eselo.dao.ParametroCalculoDAO;
import br.com.eselo.dao.RecomendacaoDAO;
import br.com.eselo.dao.UsuarioDAO;
import br.com.eselo.integracao.CriterioAvaliado;
import br.com.eselo.model.Criterio;
import br.com.eselo.model.Faixa;
import br.com.eselo.model.ParametroCalculo;
import br.com.eselo.model.Recomendacao;
import br.com.eselo.model.Usuario;

@Resource
public class ParametroCalculoController {

	private final ParametroCalculoDAO dao;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	protected static final int MODO_ADD = 1;
	protected static final int MODO_EDIT = 2;
	protected static final int MODO_DELETE = 3;
	protected static final int MODO_IMPORT = 4;

	public ParametroCalculoController(ParametroCalculoDAO _dao,
			RecomendacaoDAO _daoRecomendacao, Result _result,
			Validator _validator, UsuarioWeb _usuarioWeb) {
		dao = _dao;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/parametrosCalculo/novo")
	@Restrito(descricao = "PARAMETRO_CALCULO_ADICIONAR")
	public void novo() {
		ParametroCalculo parametroCalculo = new ParametroCalculo();
		parametroCalculo.setUsuario(dao.getUsuario(usuarioWeb.getId()));
		parametroCalculo.setTsInicio(new java.util.Date());
		inserirParametroCalculo(parametroCalculo);

		result.redirectTo(ParametroCalculoController.class)
				.form(dao.getMaxId());
	}

	@Get("/parametrosCalculo/importar/")
	@Restrito(descricao = "PARAMETRO_CALCULO_ADICIONAR")
	public void importar(long idParametroCalculo) {

	String acao = VRaptorRequestHolder.currentRequest().getRequest().getParameter("acao");

	String mensagem="";

	if(acao.equals("Importar")){
		ParametroCalculo pc = dao.load(idParametroCalculo);
		this.criarNovoParametroCalculo(pc, MODO_IMPORT);
		mensagem = Messages.getString("parametro_calculo.criado")
				+ pc.getDescricao() + ".";

		result.redirectTo(ParametroCalculoController.class).form(dao.getMaxId());

	}else if(acao.equals("NÃ£o Importar")){
		criarNovoParametroCalculo(new ParametroCalculo(), MODO_ADD);
		mensagem = Messages.getString("parametro_calculo.criado.limpo");

		result.redirectTo(ParametroCalculoController.class).form(dao.getMaxId());

	}else if(acao.equals("Cancelar")){
		mensagem = Messages.getString("validacao.acao_cancelada");
		result.redirectTo(ParametroCalculoController.class).lista();
	}

		result.include("mensagem", mensagem);
	}

	@Post("/parametrosCalculo")
	@Restrito(descricao = "PARAMETRO_CALCULO_ADICIONAR")
	public void adicionar(ParametroCalculo parametroCalculo) {
		
		inserirParametroCalculo(parametroCalculo);

		result.include("mensagem", "Parâmetro de Cálculo salvo com sucesso.");
		result.redirectTo(ParametroCalculoController.class).index(1);
	}

	protected void inserirParametroCalculo(ParametroCalculo parametroCalculo) {
		if (parametroCalculo.getDescricao() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("parametro_calculo.validacao.descricao"),
					"parametroCalculo.descricao"));
		}

		dao.save(parametroCalculo);
	}

	@Put("/parametrosCalculo")
	@Restrito(descricao = "PARAMETRO_CALCULO_ATUALIZAR")
	public void atualizar(ParametroCalculo parametroCalculo) {

		@SuppressWarnings("unused")
		List<Usuario> usuarios = dao.getAllUsuarios();
		ParametroCalculo parametroCalculoCadastrado = dao.load(parametroCalculo.getId());

		parametroCalculo.setTsInicio(parametroCalculoCadastrado.getTsInicio());
		parametroCalculoCadastrado.setUsuario(new UsuarioDAO(dao.getSession()).load(usuarioWeb.getId()));
		parametroCalculo.setUsuario(parametroCalculoCadastrado.getUsuario());
		parametroCalculoCadastrado.setDescricao(parametroCalculo.getDescricao());

		result.include("mensagem", "Parâmetro de Cálculo atualizado com sucesso.");
		result.redirectTo(ParametroCalculoController.class).form(parametroCalculo.getId());
	}

	@Path("/parametrosCalculo/ativar/{id}")
	@Restrito(descricao = "PARAMETRO_CALCULO_ATIVAR")
	public void ativar(Long id) {

			ParametroCalculo pc = dao.load(id);
			
			if(validarParametroCalculoAtivacao(pc)){
				ativarParametroCalculo(id);
				result.redirectTo(ParametroCalculoController.class).lista();
			}else{
				result.redirectTo(ParametroCalculoController.class).errosAtivacao(id);
			}
		}

	@Path("/parametrosCalculo/delete/{id}")
	@Restrito(descricao = "PARAMETRO_CALCULO_EXCLUIR")
	public void excluir(Long id) {
		if (dao.listAll().size() > 1) {
			ParametroCalculo pc = dao.load(id);

			if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
				if ((pc.getTsFim() == null)) {

					if(pc.getTsInicio()!=null){
						pc = dao.getLastParametrosCalculoAtivavel(pc);
						pc.setTsFim(null);
						dao.remove(id);
						dao.update(pc);
					}else{
						dao.remove(id);
					}
					
					result.include("mensagem",
							Messages.getString("parametro_calculo.validacao.excluido"));
				} else {
					result.include("mensagem",
							Messages.getString("parametro_calculo.validacao.nao_pode_excluir"));
				}
			}else{

				result.include("mensagem",
						Messages.getString("parametro_calculo.validacao.nao_pode_excluir.em_uso"));
			}

		} else {
			result.include("mensagem",
					Messages.getString("parametro_calculo.validacao.nao_pode_excluir.unico"));
		}

		result.redirectTo(ParametroCalculoController.class).lista();
	}

	@Get("/parametrosCalculo")
	@Restrito(descricao = "PARAMETRO_CALCULO_LISTA")
	public List<ParametroCalculo> lista() {
		return this.index(1);
	}

	@Get("/parametrosCalculo/{pagina}")
	@Restrito(descricao = "PARAMETRO_CALCULO_LISTA")
	public List<ParametroCalculo> index(int pagina) {
		ParametroCalculo ultimoParametro = dao.load(dao.getMaxId());

		// Paginação
		// quantidade de registros por pagina
		int exibi = 7;
		// Informa a quantidade de registros
		int registros = dao.numLinhas();
		// Quantidade de paginas na paginação
		int totalPag = registros / exibi;
		// se houver resto na divisao acima acrescenta uma pagina na paginação
		if ((registros - (totalPag * exibi)) > 0)
			totalPag++;

		// Calculo do registro inicial e final
		int regIni = ((exibi * pagina) - exibi);
		int regFim = (regIni + exibi);

		if (regFim > registros)
			regFim = registros;

		// fim paginação

		List<ParametroCalculo> resultado = dao.listAll(regIni, regFim);
		List<ParametroCalculo> paginacao = new ArrayList<ParametroCalculo>();

		for (int i = regIni; i < regFim; i++) {
			paginacao.add(resultado.get(i));
		}

		result.include("paginas", totalPag);
		result.include("registros", registros);
		result.include("regInic", ++regIni);
		result.include("regFim", regFim);
		result.include("controle", "parametrosCalculo");
		result.include("ultimoParametro", ultimoParametro);

		return paginacao;
	}

	@Get("/parametrosCalculo/lista")
	@Restrito(descricao = "PARAMETRO_CALCULO_LISTA")
	public List<ParametroCalculo> lista(ParametroCalculo parametroCalculo) {
		result.include("parametroCalculo", parametroCalculo);
		return dao.listAll(parametroCalculo);
	}

	@Get("/parametrosCalculo/atualizar/{id}")
	@Restrito(descricao = "PARAMETRO_CALCULO_ATUALIZAR")
	public ParametroCalculo form(Long id) {
		ParametroCalculo parametroCalculo = this.dao.load(id);
		result.include("usuarioList", dao.getAllUsuarios());
		return parametroCalculo;
	}

	@Restrito(descricao = "PARAMETRO_CALCULO_ADICIONAR")
	public void form() {
		ParametroCalculo parametroCalculo = new ParametroCalculo();
		parametroCalculo.setUsuario(dao.getUsuario(usuarioWeb.getId()));
		//parametroCalculo.setTsInicio(new java.util.Date());
		result.include("parametroCalculo", parametroCalculo);
	}

	private void criarNovoParametroCalculo(ParametroCalculo parametroCalculo,
			int tipo) {
		boolean ok = false;

		switch (tipo) {
		case ParametroCalculoController.MODO_ADD:
			parametroCalculo.setId(0L);
			ok = true;
			break;

		case ParametroCalculoController.MODO_IMPORT:
			ok = true;
			break;
		}

		if (ok) {
			new CopiadorParametroCalculo().criarNovoParametroCalculo(
					parametroCalculo, usuarioWeb, dao.getSession());
		}

	}

	private boolean validarParametroCalculoAtivacao(ParametroCalculo parametroCalculo){
		boolean resultado = true;
		String mensagem="";
		String aviso= " Não foi possível ativar este parâmetro de cálculo!<br/> Antes da ativação será necessário resolver as pendências listadas abaixo.";
		
		if(parametroCalculo.getRecomendacoes().size()>0){
			for(Recomendacao r: parametroCalculo.getRecomendacoes()){
				if(r.getCriterios().size()>0){
					for(Criterio c: r.getCriterios()){
						if((c.getTipoTeste().getId().intValue()==CriterioAvaliado.TIPO_DESCRESCENTE) && (c.getFaixas().size() < 1)){
							mensagem = mensagem+" O criterio "+c.getNumeroCriterio()+" da recomendação "+r.getNumeroRecomendacao()+" é do tipo DECRESCENTE e não possui nenhuma faixa cadastrada.<br/>";
							resultado = false;
						}
					}
				}else{
					mensagem = mensagem+" A recomendação "+r.getNumeroRecomendacao()+" não possui nenhum critério cadastrado.<br/>";
					resultado = false;
				}
			}
		}else{
			mensagem = "Nenhuma recomendação foi cadastrada.";
			resultado = false;
		}
		
		if(resultado){
			result.include("mensagem", Messages.getString("parametro_calculo.validacao.ativado"));
		}else{
			result.include("mensagem",aviso);
			System.out.println(mensagem);
		}
		
		return resultado;
	}
	
	private void ativarParametroCalculo(Long id){
		ParametroCalculo parametroCalculoelegivel, parametroCalculoAtivo;
		parametroCalculoelegivel = dao.load(id);
		parametroCalculoAtivo = dao.getLastParametrosCalculoAtivo();
		
		parametroCalculoelegivel.setTsInicio(new Date());
		
		dao.update(parametroCalculoelegivel);
		
		if(parametroCalculoAtivo!=null){
			parametroCalculoAtivo.setTsFim(parametroCalculoelegivel.getTsInicio());
			dao.update(parametroCalculoAtivo);
			}
	}
	
	@Get("/parametrosCalculo/relatorio/{id}")
	@Restrito(descricao = "PARAMETRO_CALCULO_RELATORIO")
	public ParametroCalculo relatorio(Long id) {
		ParametroCalculo parametroCalculo = this.dao.load(id);
		result.include("usuarioList", dao.getAllUsuarios());
		result.include("recomendacaoList",
				new RecomendacaoDAO(dao.getSession())
						.listAllAtivas(parametroCalculo));

		return parametroCalculo;
	}
	
	@Get("/parametrosCalculo/errosAtivacao/{id}")
	@Restrito(descricao = "PARAMETRO_CALCULO_RELATORIO")
	public ParametroCalculo errosAtivacao(Long id) {
		ParametroCalculo parametroCalculo = this.dao.load(id);
		result.include("usuarioList", dao.getAllUsuarios());
		
		List<Recomendacao> recomendacaoList = new RecomendacaoDAO(dao.getSession()).listAllAtivas(parametroCalculo);
		List<Recomendacao> recomendacaoListErros = new ArrayList<Recomendacao>();
		
		for(Recomendacao r: recomendacaoList){
			if(r.getCriterios().size()<1){
				recomendacaoListErros.add(r);
			}else{
				List<Criterio> criterios = new ArrayList<Criterio>();
				for(Criterio c: r.getCriterios()){
					if(c.getTipoTeste().getId().intValue()==CriterioAvaliado.TIPO_DESCRESCENTE && c.getFaixas().size()<1){
						criterios.add(c);
					}
				}
				r.setCriterios(criterios);
				recomendacaoListErros.add(r);
			}
		}
		
		
		result.include("recomendacaoList", recomendacaoListErros);

		return parametroCalculo;
	}
	
	@Get("/parametroAtivo.xls")
	public File exportarAtivo(){
		Long id = dao.getLastParametrosCalculoAtivo().getId();
		return exportar(id);
	}
	
	@Get("/parametrosCalculo/exportar/{id}/*")
	@Restrito(descricao = "PARAMETRO_CALCULO_LISTA")
	public File exportar(Long id) {
		File relatorio = new File("workbook.ods");
		ParametroCalculo parametroCalculo = this.dao.load(id);
		List<Recomendacao> recomendacaoList = new RecomendacaoDAO(
				dao.getSession()).listAllAtivas(parametroCalculo);
		String msgAtivacao="";
		
		if(parametroCalculo.getTsInicio() != null){
			msgAtivacao = "Ativado em: "+parametroCalculo.getTsInicio()+".";	
		}
		if(parametroCalculo.getTsFim() != null){
			msgAtivacao = msgAtivacao +" Encerrado em: "+parametroCalculo.getTsFim();	
		}
		
		Workbook wb = new HSSFWorkbook();
		FileOutputStream fileOut;

		Map<String, CellStyle> styles = createStyles(wb);
		CreationHelper createHelper = wb.getCreationHelper();
		Sheet sheet = wb.createSheet("new sheet");
		sheet.setPrintGridlines(false);
		sheet.setDisplayGridlines(false);

		short rows = 0;
		Row row = sheet.createRow(rows);
		Cell cell = row.createCell(0);
		cell.setCellValue(createHelper.createRichTextString(parametroCalculo.getDescricao()));
		cell.setCellStyle(styles.get("title"));
		sheet.addMergedRegion(CellRangeAddress.valueOf("$A$1:$L$1"));
		rows++;
		
		row = sheet.createRow(rows);
		cell = row.createCell(0);
		cell.setCellValue(createHelper.createRichTextString(msgAtivacao));
		CellStyle style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_CENTER);
		cell.setCellStyle(style);
		
		sheet.addMergedRegion(CellRangeAddress.valueOf("$A$2:$L$2"));
		rows += 3;

		for (Recomendacao recomendacao : recomendacaoList) {
			row = sheet.createRow(rows);
			cell = row.createCell(0);
			cell.setCellValue(
					createHelper.createRichTextString("Recomendação "
							+ recomendacao.getNumeroRecomendacao()) 
							+ ": " + recomendacao.getDescricao() 
							+ " (Peso: "+recomendacao.getPeso()+")"
					);
			cell.setCellStyle(styles.get("subtitle"));
			rows++;
			sheet.addMergedRegion(CellRangeAddress.valueOf("$A$"+rows+":$L$"+rows));

			/*
			row = sheet.createRow(rows);
			cell = row.createCell(1);
			cell.setCellValue("Descrição");
			cell.setCellStyle(styles.get("label"));
			row.createCell(2).setCellValue(recomendacao.getDescricao());
			rows++;

			row = sheet.createRow(rows);
			cell = row.createCell(1);
			cell.setCellValue("Peso");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(2);
			cell.setCellValue(recomendacao.getPeso());
			cell.setCellStyle(styles.get("numeric"));
			rows += 2;
			*/

			row = sheet.createRow(rows);
			cell = row.createCell(1);
			cell.setCellValue("Número Critério");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(2);
			cell.setCellValue("Peso");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(3);
			cell.setCellValue("Descrição");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(4);
			cell.setCellValue("Requisito");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(5);
			cell.setCellValue("Tipo de Verificação");
			cell.setCellStyle(styles.get("label"));
			cell = row.createCell(6);
			cell.setCellValue("Tipo de Mensagem");
			cell.setCellStyle(styles.get("label"));
			rows++;

			for (Criterio criterio : recomendacao.getCriterios()) {
				row = sheet.createRow(rows);
				row.createCell(1).setCellValue(criterio.getNumeroCriterio());
				row.createCell(2).setCellValue(criterio.getPeso());
				row.createCell(3).setCellValue(criterio.getDescricao());
				
				row.createCell(4).setCellValue(criterio.getRequisito());
				
				String tipoVetificacao;
				if(criterio.isVerificacaoAutomatica()){
					tipoVetificacao = "Automática";
				}else if(!criterio.isVerificacaoAutomatica()){
					tipoVetificacao="Humana";
				}else{
					tipoVetificacao="";
				}
				
				String tipoErro;
				if(criterio.isErro()){
					tipoErro = "Erro";
				}else if(!criterio.isErro()){
					tipoErro="Aviso";
				}else{
					tipoErro="";
				}
				
				row.createCell(5).setCellValue(tipoVetificacao);
				row.createCell(6).setCellValue(tipoErro);
				
				if(criterio.getTipoTeste().getId().intValue()==3){
					rows++;
					row = sheet.createRow(rows);
					cell = row.createCell(2);
					cell.setCellValue("Faixa");
					cell.setCellStyle(styles.get("label"));
					cell = row.createCell(3);
					cell.setCellValue("Descrição");
					cell.setCellStyle(styles.get("label"));
					cell = row.createCell(4);
					cell.setCellValue("Início");
					cell.setCellStyle(styles.get("label"));
					cell = row.createCell(5);
					cell.setCellValue("Fim");
					cell.setCellStyle(styles.get("label"));
					cell = row.createCell(6);
					cell.setCellValue("%");
					cell.setCellStyle(styles.get("label"));
					rows++;

					for(Faixa faixa:criterio.getFaixas()){
						row = sheet.createRow(rows);
						row.createCell(2).setCellValue(faixa.getFaixa());
						row.createCell(3).setCellValue(faixa.getDescricao());
						row.createCell(4).setCellValue(faixa.getFaixaInicio());
						row.createCell(5).setCellValue(faixa.getFaixaFim());
						row.createCell(6).setCellValue(faixa.getPercentualConformidade());
						rows++;

					}
					
				}
				 
				
				rows++;
			}

			rows += 2;
		}

		try {
			fileOut = new FileOutputStream(relatorio);
			wb.write(fileOut);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return relatorio;
	}

	private static Map<String, CellStyle> createStyles(Workbook wb) {
		Map<String, CellStyle> styles = new HashMap<String, CellStyle>();

		CellStyle style;
		Font fontLargeBold = wb.createFont();
		fontLargeBold.setFontHeightInPoints((short) 12);
		fontLargeBold.setFontName("Arial");
		fontLargeBold.setBoldweight(Font.BOLDWEIGHT_BOLD);

		Font fontMediumBold = wb.createFont();
		fontMediumBold.setFontHeightInPoints((short) 11);
		fontMediumBold.setFontName("Arial");
		fontMediumBold.setBoldweight(Font.BOLDWEIGHT_BOLD);

		Font fontNormalBold = wb.createFont();
		fontNormalBold.setFontHeightInPoints((short) 10);
		fontNormalBold.setFontName("Arial");
		fontNormalBold.setBoldweight(Font.BOLDWEIGHT_BOLD);

		style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_CENTER);
		style.setFont(fontLargeBold);
		styles.put("title", style);

		style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_LEFT);
		style.setFont(fontMediumBold);
		style.setBorderBottom(CellStyle.BORDER_DOTTED);
        style.setBottomBorderColor(IndexedColors.GREY_40_PERCENT.getIndex());
		styles.put("subtitle", style);

		style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_LEFT);
		style.setFont(fontNormalBold);
		styles.put("label", style);

		style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_LEFT);
		style.setDataFormat(wb.createDataFormat().getFormat("0"));
		styles.put("numeric", style);

		return styles;
	}
}