package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.dao.PermissaoDAO;
import br.com.eselo.model.Permissao;

@Resource
public class PermissoesController {

	private final PermissaoDAO dao;
	private final Result result;
	private final Validator validator;

	public PermissoesController(PermissaoDAO _dao, Result _result,
			Validator _validator) {
		dao = _dao;
		result = _result;
		validator = _validator;
	}

	@Get("/permissoes")
	@Restrito(descricao = "PERMISSOES_LISTA")
	public List<Permissao> lista() {
		return dao.listAll();
	}

	@Restrito(descricao = "")
	public void form() {

	}

	@Get("/permissoes/{id}")
	@Restrito(descricao = "")
	public Permissao form(Long id) {
		Permissao permissao = dao.load(id);
		return permissao;
	}

	@Post("/permissoes")
	@Restrito(descricao = "")
	public void adicionar(Permissao permissao) {
		Long idPermissaoMesmoCpf = dao.getIdByDescricao(permissao.getPermissao());

		if (permissao.getPermissao() == null) {
			validator.add(new ValidationMessage("A descrição da permissão é obrigatória.",
					"permissao.permissao"));
		} else if (idPermissaoMesmoCpf != null) {
			validator.add(new ValidationMessage(
					"A permissão não pode ser igual a uma já existente.",
					"permissao.permissao"));
		}
		validator.onErrorUsePageOf(PermissoesController.class).form();

		dao.save(permissao);

		result.include("mensagem", "Permissão salva com sucesso.");
		result.redirectTo(PermissoesController.class).lista();
	}

	@Put("/permissoes")
	@Restrito(descricao = "")
	public void atualizar(Permissao permissao) {
		Permissao permissaoCadastrada = dao.load(permissao.getId());
		permissaoCadastrada.setPermissao(permissao.getPermissao());

		Long idPermissaoMesmaDescricao = dao.getIdByDescricao(permissaoCadastrada.getPermissao());

		if (permissaoCadastrada.getPermissao() == null) {
			validator.add(new ValidationMessage("A descrição da permissão é obrigatória.",
					"permissao.permissao"));
		} else if (idPermissaoMesmaDescricao != null
				&& !idPermissaoMesmaDescricao.equals(permissaoCadastrada.getId())) {
			validator.add(new ValidationMessage(
					"A permissão não pode ser igual a uma já existente.",
					"permissao.permissao"));
		}
		validator.onErrorUsePageOf(PermissoesController.class).form();

		dao.saveOrUpdate(permissaoCadastrada);

		result.include("mensagem", "Permissão alterada com sucesso.");
		result.redirectTo(PermissoesController.class).lista();
	}

	@Path("/permissoes/delete/{id}")
	@Restrito(descricao = "")
	public void excluir(Long id) {
		Permissao permissaoCadastrada = dao.load(id);

		if (!permissaoCadastrada.getGrupos().isEmpty()) {
			validator.add(new ValidationMessage("A permissão está sendo referenciada e não pode ser excluída.",
					"permissao.permissao"));
		}
		validator.onErrorForwardTo(PermissoesController.class).lista();

		dao.remove(id);

		result.include("mensagem", "Permissão excluída com sucesso.");
		result.redirectTo(PermissoesController.class).lista();
	}

	@Get("/permissoes/sincronizar")
	@Restrito(descricao = "PERMISSOES_SINCRONIZAR")
	public void sincronizar() {
		dao.sincronizar();

		result.include("mensagem", "Permissões sincronizadas com sucesso.");
		result.redirectTo(PermissoesController.class).lista();
	}

}
