package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.CopiadorParametroCalculo;
import br.com.eselo.component.Messages;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.AvaliacaoDAO;
import br.com.eselo.dao.CriterioDAO;
import br.com.eselo.dao.ParametroCalculoDAO;
import br.com.eselo.dao.RecomendacaoDAO;
import br.com.eselo.model.ParametroCalculo;
import br.com.eselo.model.Recomendacao;

@Resource
public class RecomendacaoController {

	private final RecomendacaoDAO dao;
	private final ParametroCalculoDAO paDAO;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public RecomendacaoController(RecomendacaoDAO _dao, Result _result,
			ParametroCalculoDAO _paDAO, Validator _validator,
			UsuarioWeb _usuarioWeb) {
		this.dao = _dao;
		this.result = _result;
		this.paDAO = _paDAO;
		this.validator = _validator;
		this.usuarioWeb = _usuarioWeb;
	}

	@Get("/recomendacao/novo/{id}")
	@Restrito(descricao = "RECOMENDACAO_ADICIONAR")
	public Recomendacao novo(Long id) {
		ParametroCalculo parametroCalculo = this.paDAO.load(id);
		Recomendacao recomendacao = new Recomendacao();
		recomendacao.setParametroCalculo(parametroCalculo);

		result.include("parametroCalculoId", recomendacao.getParametroCalculo()
				.getId());
		result.include("parametroCalculo", recomendacao.getParametroCalculo());

		return recomendacao;
	}

	@Get("/recomendacao/form/{id}")
	@Restrito(descricao = "RECOMENDACAO_ATUALIZAR")
	public Recomendacao form(Long id) {
		Recomendacao recomendacao = this.dao.load(id);
		result.include("parametroCalculoId", recomendacao.getParametroCalculo()
				.getId());
		result.include("parametroCalculo", recomendacao.getParametroCalculo());
		result.include("criterioList",
				new CriterioDAO(dao.getSession())
						.listAllAtivas(recomendacao));
		return recomendacao;
	}

	@Post("/recomendacao")
	@Restrito(descricao = "RECOMENDACAO_ADICIONAR")
	public void adicionar(Recomendacao recomendacao) {

		validarRecomendacao(recomendacao);

		validator.onErrorUsePageOf(RecomendacaoController.class).form(
				recomendacao.getParametroCalculo().getId());

		ParametroCalculo pc = paDAO.load(recomendacao.getParametroCalculo().getId());
		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			
			dao.save(recomendacao);
			result.include("mensagem", Messages.getString("recomendacao.salva"));
			result.redirectTo(ParametroCalculoController.class).form(pc.getId());
		}else{
			criarNovoParametroAvaliacao(recomendacao,ParametroCalculoController.MODO_ADD);
			result.include("mensagem", Messages.getString("recomendacao.salva")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
			result.redirectTo(ParametroCalculoController.class).index(1);
		}
		
	}

	@Put("/recomendacao")
	@Restrito(descricao = "RECOMENDACAO_ATUALIZAR")
	public void atualizar(Recomendacao recomendacao) {
		Recomendacao recomendacaoCadastrada = dao.load(recomendacao.getId());

		validarRecomendacao(recomendacao);

		validator.onErrorUsePageOf(RecomendacaoController.class).form(
				recomendacaoCadastrada.getId());

		if(validarAtualizacaoRecomendacao(recomendacao)){
			ParametroCalculo pc = paDAO.load(recomendacaoCadastrada.getParametroCalculo().getId());
			if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
				recomendacaoCadastrada.setNumeroRecomendacao(recomendacao.getNumeroRecomendacao());
				recomendacaoCadastrada.setDescricao(recomendacao.getDescricao());
				recomendacaoCadastrada.setPeso(recomendacao.getPeso());
				
				dao.update(recomendacaoCadastrada);
				result.include("mensagem", Messages.getString("recomendacao.atualizada"));
				result.redirectTo(ParametroCalculoController.class).form(pc.getId());
			}else{
				criarNovoParametroAvaliacao(recomendacao,ParametroCalculoController.MODO_EDIT);
				result.include("mensagem", Messages.getString("recomendacao.salva")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());
				result.redirectTo(ParametroCalculoController.class).index(1);
			}
		}else{
			recomendacaoCadastrada.setDescricao(recomendacao.getDescricao());
			dao.update(recomendacaoCadastrada);
			result.include("mensagem", Messages.getString("validacao.sem_alteracao"));
			result.redirectTo(RecomendacaoController.class).form(recomendacao.getId());
		}
	}

	@Path("/recomendacao/delete/{id}")
	@Restrito(descricao = "RECOMENDACAO_EXCLUIR")
	public void excluir(Long id) {
		Recomendacao recomendacao = dao.load(id);
		
		ParametroCalculo pc = paDAO.load(recomendacao.getParametroCalculo().getId());
		if(!(new AvaliacaoDAO(dao.getSession()).existeAvaliacao(pc))){
			result.include("mensagem", Messages.getString("recomendacao.exclusao"));
			dao.remove(recomendacao.getId());
			result.redirectTo(ParametroCalculoController.class).form(pc.getId());
		}else{
			criarNovoParametroAvaliacao(recomendacao,
			ParametroCalculoController.MODO_DELETE);
			result.include("mensagem", Messages.getString("recomendacao.exclusao")+" "+Messages.getString("parametro_calculo.criado")+" "+pc.getDescricao());

			result.redirectTo(ParametroCalculoController.class).index(1);
		}
		
	}

	private void criarNovoParametroAvaliacao(Recomendacao recomendacao, int tipo) {
		ParametroCalculo ultimoParametroAvaliacao = new CopiadorParametroCalculo().copiarParametroCalculo(paDAO.getLastParametrosCalculoAtivo());
		boolean ok = false;

		switch (tipo) {
		case ParametroCalculoController.MODO_ADD:
			ultimoParametroAvaliacao.addRecomendacao(recomendacao);
			ok = true;
			break;

		case ParametroCalculoController.MODO_EDIT:
			ultimoParametroAvaliacao = new CopiadorParametroCalculo().copiarParametroCalculo(paDAO.getLastParametrosCalculoAtivo());
			for (Recomendacao r : ultimoParametroAvaliacao.getRecomendacoes()) {
				if (r.getNumeroRecomendacao().longValue() == recomendacao.getNumeroRecomendacao().longValue()) {

					r.setNumeroRecomendacao(recomendacao.getNumeroRecomendacao());
					r.setDescricao(recomendacao.getDescricao());
					r.setPeso(recomendacao.getPeso());
					ok = true;
					break;
				}
			}

			break;

		case ParametroCalculoController.MODO_DELETE:
			ultimoParametroAvaliacao = paDAO.getLastParametrosCalculoAtivo();
			for (Recomendacao r : ultimoParametroAvaliacao.getRecomendacoes()) {
				if (r.getNumeroRecomendacao().longValue() == recomendacao.getNumeroRecomendacao().longValue()) {
					ultimoParametroAvaliacao.removeRecomendacao(recomendacao);
					ok = true;
					break;
				}
			}
			break;
		}

		if (ok) {
			new CopiadorParametroCalculo().criarNovoParametroCalculo(
					ultimoParametroAvaliacao, usuarioWeb, dao.getSession());
		}
	}

	private void validarRecomendacao(Recomendacao recomendacao) {

		if (recomendacao.getNumeroRecomendacao() == null) {
			validator.add(new ValidationMessage(Messages.getString("recomendacao.validacao.numero"),
					"recomendacao.numeroRecomendacao"));
		} else if (hasNumeroRecomendacaoNoParametroAtual(recomendacao)) {
			validator
					.add(new ValidationMessage(
							Messages.getString("recomendacao.validacao.numero.unique"),
							"recomendacao.numeroRecomendacao"));
		} else if (recomendacao.getDescricao() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("validacao.descricao"),
					"recomendacao.descricao"));
		} else if (recomendacao.getPeso() == null) {
			validator.add(new ValidationMessage(
					Messages.getString("validacao.peso"),
					"recomendacao.peso"));
		} else if (recomendacao.getPeso() <= 0) {
			validator.add(new ValidationMessage(
					Messages.getString("validacao.peso.zero"),
					"recomendacao.peso"));
		}

		result.include("parametroCalculoId", recomendacao.getParametroCalculo()
				.getId());
		result.include("parametroCalculo", recomendacao.getParametroCalculo());

	}

	private boolean validarAtualizacaoRecomendacao(Recomendacao recomendacao) {

		boolean alteracao = false;

		Recomendacao recomendacaoCadastrada = dao.load(recomendacao.getId());

		if (recomendacao.getNumeroRecomendacao().compareTo(recomendacaoCadastrada.getNumeroRecomendacao())!=0) {
			alteracao = true;
		} else if (recomendacao.getPeso().compareTo(recomendacaoCadastrada.getPeso()) !=0) {
			alteracao = true;
		} else if (recomendacao.getDescricao().compareToIgnoreCase(recomendacaoCadastrada.getDescricao()) != 0) {
			alteracao = true;
		}

		return alteracao;
	}

	private boolean hasNumeroRecomendacaoNoParametroAtual(
			Recomendacao recomendacao) {
		boolean retorno = false;

		if (recomendacao.getDescricao() != null
				&& recomendacao.getNumeroRecomendacao() != null
				&& recomendacao.getPeso() != null) {
			ParametroCalculo pc = paDAO.load(recomendacao.getParametroCalculo().getId());
			List<Recomendacao> listaCadastrada = dao.getByParametroCalculo(pc);
			for (Recomendacao recomendacaoCadastrada : listaCadastrada) {
				if (recomendacaoCadastrada.getNumeroRecomendacao().longValue() == recomendacao
						.getNumeroRecomendacao().longValue()) {
					if (recomendacao.getId() != null) {
						if (recomendacaoCadastrada.getId().longValue() != recomendacao
								.getId().longValue()) {
							return true;
						}
					} else {
						return true;
					}
				}
			}
		}
		return retorno;
	}

}
