package br.com.eselo.controller;

import java.util.List;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.Messages;
import br.com.eselo.dao.TipoTesteDAO;
import br.com.eselo.model.TipoTeste;

@Resource
public class TiposTesteController {

	private final TipoTesteDAO dao;
	private final Result result;
	private final Validator validator;

	public TiposTesteController(TipoTesteDAO _dao, Result _result,
			Validator _validator) {
		dao = _dao;
		result = _result;
		validator = _validator;
	}

	@Get("/tiposTeste")
	@Restrito(descricao = "TIPOS_TESTELISTA")
	public List<TipoTeste> lista() {
		return dao.listAll();
	}

	@Restrito(descricao = "TIPOS_TESTE_ADICIONAR")
	public void form() {
	}

	@Get("/tiposTeste/{id}")
	@Restrito(descricao = "TIPOS_TESTE_ATUALIZAR")
	public TipoTeste form(Long id) {
		TipoTeste tipoTeste = dao.load(id);

		/*
		if (!tipoTeste.getRecomendacoes().isEmpty()) {
			result.include("mensagem",
					Messages.getString("tipo_recomendacao.validacao.edit"));
			result.redirectTo(TiposTesteController.class).lista();
		}
		*/

		return tipoTeste;
	}

	@Put("/tiposTeste")
	@Restrito(descricao = "TIPOS_TESTE_ATUALIZAR")
	public void atualizar(TipoTeste tipoTeste) {
		TipoTeste tipoTesteCadastrada = dao.load(tipoTeste.getId());

		validarTipoConformidade(tipoTeste);

		validator.onErrorUsePageOf(TiposTesteController.class).form(
				tipoTeste.getId());

		tipoTesteCadastrada.setTipo(tipoTeste.getTipo());
		tipoTesteCadastrada.setDescricao(tipoTeste.getDescricao());

		dao.saveOrUpdate(tipoTesteCadastrada);

		result.include("mensagem",
				Messages.getString("tipo_recomendacao.mensagem.alterada"));
		result.redirectTo(TiposTesteController.class).lista();
	}

	private void validarTipoConformidade(TipoTeste tipoTeste) {
		if (tipoTeste.getTipo() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_recomendacao.validacao.tipo"),
					"tipoTeste.tipo"));
		} else if (tipoTeste.getDescricao() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_recomendacao.validacao.descricao"),
					"tipoTeste.descricao"));
		} else if (dao.existeTipo(tipoTeste)) {
			Long id = dao.getIdByDescricao(tipoTeste.getDescricao());

			if ((tipoTeste.getId() == null && id != null)
					|| (tipoTeste.getId() != null && tipoTeste.getId() == id)) {
				validator
						.add(new ValidationMessage(
								Messages.getString("tipo_recomendacao.validacao.descricao.unique"),
								"tipoTeste.descricao"));
			}
		}
		/*
		else if (tipoTeste.getMaximoConformidade() == null) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_recomendacao.validacao.maximo"),
					"tipoRecomendacao.maximoConformidade"));
		} else if (tipoTeste.getMaximoConformidade() <= 0) {
			validator.add(new ValidationMessage(Messages
					.getString("tipo_recomendacao.validacao.maximo.min"),
					"tipoRecomendacao.maximoConformidade"));
		}
		*/
	}

}
