package br.com.eselo.controller;

import java.io.IOException;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.UrlDAO;
import br.com.eselo.model.Url;

@Resource
public class UrlController {

	private final UrlDAO dao;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;

	public UrlController(UrlDAO _dao, Result _result, Validator _validator,
			UsuarioWeb _usuarioWeb) {
		dao = _dao;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
	}

	@Get("/url")
	@Restrito(descricao = "URL_LISTA")
	public List<Url> index() {
		return dao.listAll();

	}

	@Post("/url")
	@Restrito(descricao = "URL_LISTA")
	public List<Url> lista(String nUrl) {

		Long idUsuario = usuarioWeb.getUsuario().getId();

		if (nUrl == null)
			return this.dao.listAll(idUsuario);
		else
			return this.dao.listFiltro(nUrl, idUsuario);

	}

	@Post("/url/adiciona")
	@Restrito(descricao = "URL_ADICIONAR")
	public void adicionar(Url url) throws IOException, SAXException,
			ParserConfigurationException {
		if (url.getUrl() == null) {
			validator
					.add(new ValidationMessage("Url é obrigatória.", "url.url"));
		}
		validator.onErrorUsePageOf(AvaliacaoController.class).form();

		dao.save(url);

		result.include("mensagem", "Url salva com sucesso.");
		result.redirectTo(UrlController.class).index();
	}

}