package br.com.eselo.controller;

import java.security.SecureRandom;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.hibernate.Hibernate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import br.com.caelum.vraptor.Get;
import br.com.caelum.vraptor.Path;
import br.com.caelum.vraptor.Post;
import br.com.caelum.vraptor.Put;
import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.Validator;
import br.com.caelum.vraptor.validator.ValidationMessage;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.Mailer;
import br.com.eselo.component.TemplateMailer;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.dao.UsuarioDAO;
import br.com.eselo.model.Usuario;

@Resource
public class UsuariosController {

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LoggerFactory
			.getLogger(UsuariosController.class);

	private final UsuarioDAO dao;
	private final Result result;
	private final Validator validator;
	private final UsuarioWeb usuarioWeb;
	private final Mailer mailer;
	private final TemplateMailer templates;
	private final HttpServletRequest request;

	public UsuariosController(UsuarioDAO _dao, Result _result,
			Validator _validator, UsuarioWeb _usuarioWeb, Mailer _mailer,
			TemplateMailer _templates, HttpServletRequest _request) {
		dao = _dao;
		result = _result;
		validator = _validator;
		usuarioWeb = _usuarioWeb;
		mailer = _mailer;
		templates = _templates;
		request = _request;
	}

	@Get("/usuarios")
	@Restrito(descricao = "USUARIOS_LISTA")
	public List<Usuario> lista() {
		return dao.listAll();
	}

	@Get("/usuarios/lista")
	@Restrito(descricao = "USUARIOS_LISTA")
	public List<Usuario> lista(Usuario usuario) {
		result.include("usuario", usuario);
		return dao.listAll(usuario);
	}

	@Restrito(descricao = "USUARIOS_ADICIONAR")
	public void form() {
		result.include("grupoList", dao.getAllGrupos());
	}

	@Get("/usuarios/{id}")
	@Restrito(descricao = "USUARIOS_ATUALIZAR")
	public Usuario form(Long id) {
		Usuario usuario = dao.load(id);
		result.include("grupoList", dao.getAllGrupos());
		return usuario;
	}

	@Post("/usuarios")
	@Restrito(descricao = "USUARIOS_ADICIONAR")
	public void adicionar(Usuario usuario) {
		result.include("grupoList", dao.getAllGrupos());
		validate(usuario);

		dao.save(usuario);

		result.include("mensagem", "Usuário salvo com sucesso.");
		result.redirectTo(UsuariosController.class).lista();
	}



	@Put("/usuarios")
	@Restrito(descricao = "USUARIOS_ATUALIZAR")
	public void atualizar(Usuario usuario) {
		result.include("grupoList", dao.getAllGrupos());
		validate(usuario);

		Usuario usuarioCadastrado = dao.load(usuario.getId());
		usuarioCadastrado.setNome(usuario.getNome());
		usuarioCadastrado.setCpf(usuario.getCpf());
		usuarioCadastrado.setGrupo(usuario.getGrupo());
		usuarioCadastrado.setEmail(usuario.getEmail());

		if (usuario.getSenha() != null
				&& usuario.getSenha() != usuarioCadastrado.getSenha()) {
			usuarioCadastrado.setHashedSenha(usuario.getSenha());
		}

		dao.saveOrUpdate(usuarioCadastrado);

		result.include("mensagem", "Usuário alterado com sucesso.");
		result.redirectTo(UsuariosController.class).lista();
	}

	@Path("/usuarios/delete/{id}")
	@Restrito(descricao = "USUARIOS_EXCLUIR")
	public void excluir(Long id) {
		dao.remove(id);
		result.include("mensagem", "Usuário excluído com sucesso.");
		result.redirectTo(UsuariosController.class).lista();
	}

	@Get("/login")
	public void loginForm() {

	}

	@Post("/login")
	public void login(Usuario usuario) {
		Usuario carregado = dao.getByCpfAndSenha(usuario);
		if (carregado == null) {
			validator.add(new ValidationMessage("CPF e/ou senha inválidos.",
					"usuario.cpf"));
		}
		validator.onErrorUsePageOf(UsuariosController.class).loginForm();

		Hibernate.initialize(carregado.getGrupo());
		usuarioWeb.login(carregado);

		result.redirectTo(PainelController.class).index();
	}

	@Path("/logout")
	public void logout() {
		usuarioWeb.logout();
		result.redirectTo(UsuariosController.class).loginForm();
	}

	@Path("/usuarios/renovar_senha/{hash}")
	@Get
	public void renovarSenhaForm(String hash) {
		result.include("hash", hash);
	}

	@Path("/usuarios/renovar_senha")
	@Post
	public void renovarSenha(Usuario usuario) {
		Usuario carregado = dao.getByHash(usuario.getChaveAlteracaoSenha());
		if (carregado == null) {
			validator.add(new ValidationMessage("Código não encontrado.",
					"usuario.senha"));
		}

		if (usuario.getSenha() == null || usuario.getConfirmacaoSenha() == null) {
			validator.add(new ValidationMessage("Preencha a senha e a confirmação.",
					"usuario.senha"));
		}

		if (!usuario.getSenha().equals(usuario.getConfirmacaoSenha())) {
			validator.add(new ValidationMessage("A senha nova e confirmação não coincidem.",
					"usuario.senha"));
		}

		carregado.setHashedSenha(usuario.getSenha());
		carregado.setChaveAlteracaoSenha(null);
		dao.saveOrUpdate(carregado);

		validator.onErrorUsePageOf(UsuariosController.class)
				.renovarSenhaForm(usuario.getChaveAlteracaoSenha());

		result.include("mensagem",
				"Senha alterada com sucesso.");
		result.redirectTo(UsuariosController.class).loginForm();
	}

	@Path("/usuarios/recuperar_senha")
	@Get
	public void recuperarSenhaForm() {

	}

	@Path("/usuarios/recuperar_senha")
	@Post
	public void recuperarSenha(Usuario usuario){
		Usuario carregado = dao.getByCpf(usuario);
		if (carregado == null) {
			validator.add(new ValidationMessage("CPF não encontrado.",
					"usuario.cpf"));
		}else if (carregado.getEmail() == null) {
			validator.add(new ValidationMessage("Não foi cadastrado um e-mail para esse usuário.",
					"usuario.cpf"));
		}
		
		validator.onErrorUsePageOf(UsuariosController.class)
				.recuperarSenhaForm();

		String hash = RandomStringUtils.random(15, 0, 0, true, true, null,
				new SecureRandom());

		carregado.setChaveAlteracaoSenha(hash);
		dao.saveOrUpdate(carregado);

		Email email = this.templates
				.template("esqueciMinhaSenha")
				.with("nomeUsuario", carregado.getNome())
				.with("cpfUsuario", carregado.getCpfFormatado())
				.with("urlAlteracao",
						"http://"
								+ request.getHeader("Host")
								+ request.getRequestURI().replace(
										"recuperar_senha", "renovar_senha")
								+ "/" + hash)
				.to(carregado.getNome(), carregado.getEmail())
				.setSubject("e-Selo: Recuperação de Senha");
		try {
			mailer.send(email);
		} catch (EmailException e) {
			System.out.println("Erro ao tentar enviar email...");
			e.printStackTrace();
		}

		result.include("mensagem",
				"Email de recuperação de senha enviado com sucesso.");
		result.redirectTo(UsuariosController.class).loginForm();
	}

	private void validate(Usuario usuario) {
		 Long idUsuarioMesmoCpf = dao.getIdByCpf(usuario.getCpf(), usuario.getId());

		if (usuario.getNome() == null) {
			validator.add(new ValidationMessage("O nome é obrigatório.",
					"usuario.nome"));
		} else if (usuario.getCpf() == null) {
			validator.add(new ValidationMessage("O CPF é obrigatório.",
					"usuario.cpf"));
		} else if (usuario.getEmail() == null) {
			validator.add(new ValidationMessage("O email é obrigatório.",
					"usuario.email"));
		} else if (usuario.getGrupo().getId() == null) {
			validator.add(new ValidationMessage("O grupo é obrigatório.",
					"usuario.grupo.id"));
		} else if (usuario.getSenha() == null) {
			validator.add(new ValidationMessage("A senha é obrigatória.",
					"usuario.senha"));
		} else if (idUsuarioMesmoCpf != null) {
			validator.add(new ValidationMessage(
					"O CPF já está sendo utilizado por outro usuário.",
					"usuario.cpf"));
		}

		validator.onErrorUsePageOf(UsuariosController.class).form();
	}

}
