package br.com.eselo.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.Grupo;
import br.com.eselo.model.Permissao;
import br.com.eselo.model.Usuario;

@Component
public class GrupoDAO extends GenericDAO<Grupo> {

	public GrupoDAO(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Grupo> listAll() throws HibernateException {
		return this.getSession().createCriteria(this.getPersistentClass())
				.addOrder(Order.asc("nome")).list();
	}

	public Long getIdByNome(String nome) {
		Grupo grupo = (Grupo) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("nome", nome).ignoreCase())
				.uniqueResult();
		if (grupo != null) {
			return grupo.getId();
		}
		return null;
	}
	
	public Long getIdByNome(String nome, Long id) {
		Criteria criteria = getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("nome", nome).ignoreCase());
		
		if (id != null) {
			criteria.add(Restrictions.ne("id", id));
		}
				
		Grupo grupo = (Grupo) criteria.uniqueResult();
		
		if (grupo != null) {
			return grupo.getId();
		}
		return null;
	}
	
	public List<Permissao> getAllPermissoes() {
		PermissaoDAO permissaoDao = (PermissaoDAO) new PermissaoDAO(
				getSession());

		return permissaoDao.listAll();
	}

	public List<Permissao> getAllPermissoes(List<Permissao> list) {
		PermissaoDAO permissaoDao = (PermissaoDAO) new PermissaoDAO(
				getSession());
		
		List<Permissao> permissoesList = permissaoDao.listAll();

		for (Permissao permMarcada : list) {
			for (Permissao perm : permissoesList) {
				if (permMarcada.getId().equals(perm.getId())) {
					perm.setMarcado(true);
				}
			}
		}

		return permissoesList;
	}

	public boolean hasPermission(String descricaoPermissao, Usuario usuario) {
		Grupo grupo = this.load(usuario.getGrupo().getId());
		List<Permissao> permissoes = grupo.getPermissoes();
		boolean result = false;
		
		for (Permissao permissao : permissoes) {
			if (permissao.getPermissao().equalsIgnoreCase(descricaoPermissao)) {
				result = true;
				break;
			}
		}
		
		return result;
	}
	
	@SuppressWarnings("unchecked")
	public List<Grupo> listAll(Grupo grupo) {
		Criteria criteria = getSession().createCriteria(getPersistentClass());
		
		if (grupo.getNome() != null) {
			criteria.add(Restrictions.ilike("nome", "%" + grupo.getNome() + "%"));
		}
		
		criteria.addOrder(Order.asc("nome"));
		
		return criteria.list();
	}
	
}
