package br.com.eselo.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.ParametroCalculo;
import br.com.eselo.model.Usuario;

@Component
public class ParametroCalculoDAO extends GenericDAO<ParametroCalculo> {

	public ParametroCalculoDAO(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<ParametroCalculo> listAll() {
		return getSession().createCriteria(getPersistentClass())
				.addOrder(Order.desc("tsFim"))
				.addOrder(Order.desc("tsInicio")).list();
	}
	
	public List<Usuario> getAllUsuarios() {
		UsuarioDAO usuarioDao = (UsuarioDAO) new UsuarioDAO(
				getSession());

		return usuarioDao.listAll();
	}
	
	@SuppressWarnings("unchecked")
	public List<ParametroCalculo> listAll(ParametroCalculo parametroCalculo) {
		Criteria criteria = getSession().createCriteria(getPersistentClass());
		
		if (parametroCalculo.getDescricao() != null) {
			criteria.add(Restrictions.ilike("descricao", "%" + parametroCalculo.getDescricao() + "%"));
		}
		
		criteria.addOrder(Order.asc("descricao"));
		
		return criteria.list();
	}
	
	public Usuario getUsuario(Long id){
		UsuarioDAO usuarioDao = (UsuarioDAO) new UsuarioDAO(
				getSession());

		return usuarioDao.load(id);
		
	}
	
	public Long getMaxId(){
		return (Long) getSession().createCriteria(getPersistentClass())
				.setProjection(Projections.max("id"))
				.uniqueResult();
		
	}
	
	public ParametroCalculo getLastParametrosCalculoAtivo() {
		return (ParametroCalculo) getSession().createCriteria(getPersistentClass())
				.add(Restrictions.isNotNull("tsInicio"))
				.add(Restrictions.isNull("tsFim"))
				.uniqueResult();
	}
	
	public ParametroCalculo getLastParametrosCalculoAtivavel(ParametroCalculo parametroCalculo) {
		return (ParametroCalculo) getSession().createCriteria(getPersistentClass())
				.add(Restrictions.isNotNull("tsInicio"))
				.add(Restrictions.eq("tsFim",parametroCalculo.getTsInicio()))
				.uniqueResult();
	}
	
	public ParametroCalculo getParametrosCalculoMaxId() {
		return this.load(this.getMaxId());
	}

}
