package br.com.eselo.dao;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.reflections.Reflections;

import br.com.caelum.vraptor.Resource;
import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.model.Permissao;

@Component
public class PermissaoDAO extends GenericDAO<Permissao> {

	public PermissaoDAO(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Permissao> listAll() throws HibernateException {
		return this.getSession().createCriteria(this.getPersistentClass())
				.addOrder(Order.asc("permissao")).list();
	}

	public Long getIdByDescricao(String descricao) {
		Permissao permissao = (Permissao) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("permissao", descricao)).uniqueResult();
		if (permissao != null) {
			return permissao.getId();
		}
		return null;
	}
	
	public void sincronizar() {
		Reflections reflections = new Reflections("br.com.eselo.controller");
		
		Set<Class<?>> controllers = reflections.getTypesAnnotatedWith(Resource.class);
		
		for (Iterator<Class<?>> iterator = controllers.iterator(); iterator.hasNext();) {
			Class<?> class1 = (Class<?>) iterator.next();
			
			Method[] methods = class1.getDeclaredMethods();
			
			for (int i = 0; i < methods.length; i++) {
				Restrito annotation = methods[i].getAnnotation(Restrito.class);
				if (annotation != null) {
					String descricao = annotation.descricao();
					if (!descricao.isEmpty() && this.getIdByDescricao(descricao) == null) {
						Permissao permissao = new Permissao();
						permissao.setPermissao(descricao);
						this.save(permissao);
					}
				}
			}
		}
	}

}
