package br.com.eselo.dao;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import br.com.caelum.vraptor.ioc.Component;
import br.com.eselo.model.Grupo;
import br.com.eselo.model.Usuario;

@Component
public class UsuarioDAO extends GenericDAO<Usuario> {

	public UsuarioDAO(Session _session) {
		super(_session);
	}

	@SuppressWarnings("unchecked")
	public List<Usuario> listAll() {
		return getSession().createCriteria(getPersistentClass())
				.addOrder(Order.asc("nome")).list();
	}
	
	@SuppressWarnings("unchecked")
	public List<Usuario> listAll(Usuario usuario) {
		Criteria criteria = getSession().createCriteria(getPersistentClass());
		
		if (usuario.getNome() != null) {
			criteria.add(Restrictions.ilike("nome", "%" + usuario.getNome() + "%"));
		}
		
		if (usuario.getCpf() != null) {
			criteria.add(Restrictions.eq("cpf", usuario.getCpf().replaceAll("[.-]", "")));
		}
		
		criteria.addOrder(Order.asc("nome"));
		
	    
		return criteria.list();
	}

	public boolean existeUsuario(Usuario usuario) {
		Usuario encontrado = (Usuario) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("cpf", usuario.getCpf())).uniqueResult();
		

	    
		return encontrado != null;
	}

	public Usuario getByCpfAndSenha(Usuario usuario) {
		return (Usuario) getSession().createCriteria(getPersistentClass())
				.add(Restrictions.eq("cpf", usuario.getCpf()))
				.add(Restrictions.eq("senha", usuario.getSenha()))
				.uniqueResult();
	}

	public Long getIdByCpf(String cpf, Long id) {
		Criteria criteria = getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("cpf", cpf));
		
		if (id != null) {
			criteria.add(Restrictions.ne("id", id));
		}
		
		Usuario usuario = (Usuario) criteria.uniqueResult();
		
		if (usuario != null) {
			return usuario.getId();
		}
		

		return null;
	}

	public List<Grupo> getAllGrupos() {
		GrupoDAO grupoDao = (GrupoDAO) new GrupoDAO(getSession());


		session.flush(); 
	    session.close();
	    
		return grupoDao.listAll();
	}
	
	public Usuario getById(Long id) {
		Usuario _usuario = (Usuario) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("id", id)).uniqueResult();
		if (_usuario != null) {
			return _usuario;
		}
		
	    
		return null;
	}

	public Usuario getByCpf(Usuario usuario) {
		Usuario _usuario = (Usuario) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("cpf", usuario.getCpf())).uniqueResult();
		if (_usuario != null) {
			return _usuario;
		}
		return null;
	}

	public Usuario getByHash(String chaveAlteracaoSenha) {
		Usuario _usuario = (Usuario) getSession()
				.createCriteria(getPersistentClass())
				.add(Restrictions.eq("chaveAlteracaoSenha", chaveAlteracaoSenha)).uniqueResult();
		if (_usuario != null) {
			return _usuario;
		}

	    
		return null;
	}
}
