package br.com.eselo.integracao;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.basic.DateConverter;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;

public class RelatorioAvaliacao {
	
	private String url;
	private Date date;
	private List<RecomendacaoAvaliada> recomendacoes;
	private Long qtdeLinhas;
	
	public RelatorioAvaliacao() {
		this.recomendacoes = new ArrayList<RecomendacaoAvaliada>();
	}
	
	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}
	
	public List<RecomendacaoAvaliada> getRecomendacoes() {
		return recomendacoes;
	}

	public void setRecomendacoes(List<RecomendacaoAvaliada> recomendacoes) {
		this.recomendacoes = recomendacoes;
	}
	
	public void addRecomendacao(RecomendacaoAvaliada recomendacaoAvaliada) {
		this.recomendacoes.add(recomendacaoAvaliada);
	}

	public Long getQtdeLinhas() {
		return qtdeLinhas;
	}

	public void setQtdeLinhas(Long qtdeLinhas) {
		this.qtdeLinhas = qtdeLinhas;
	}

	@Override
	public String toString() {
		XStream xstream = new XStream(new DomDriver("UTF-8", new XmlFriendlyReplacer("__", "_")));  
		
		xstream.registerConverter(new DateConverter( "EEE MMM dd HH:mm:ss zzz yyyy", new String[0] ));  

		
		xstream.alias("relatorio_de_avaliacao", RelatorioAvaliacao.class);
		xstream.alias("recomendacao", RecomendacaoAvaliada.class);

		
		xstream.aliasField("data", RelatorioAvaliacao.class, "date");
		xstream.addImplicitCollection(RelatorioAvaliacao.class, "recomendacoes");


		xstream.aliasField("id_recomendacao", RecomendacaoAvaliada.class, "idRecomendacao");
		xstream.aliasField("total_erros", RecomendacaoAvaliada.class, "totalErros");
		xstream.aliasField("total_avisos", RecomendacaoAvaliada.class, "totalAvisos");
		xstream.aliasField("qtde_linhas", RelatorioAvaliacao.class, "qtdeLinhas");

		
		xstream.alias("criterio", CriterioAvaliado.class);
		xstream.addImplicitCollection(RecomendacaoAvaliada.class, "criterios");
		xstream.aliasField("id_criterio", CriterioAvaliado.class, "idCriterio");
		xstream.aliasField("itens_criterio", CriterioAvaliado.class, "qtdeItens");
		xstream.aliasField("erros_criterio", CriterioAvaliado.class, "qtdeErros");		
				
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"+xstream.toXML(this);
		
	}

	public static String TESTEXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"+
			"<relatorio_de_avaliacao>"+
			"  <url>www.teste.com.br</url>"+
			"  <data>Sun Aug 03 19:31:55 GMT-03:00 2014</data>"+
			"  <recomendacao>"+
			"    <id_recomendacao>3</id_recomendacao>"+
			"    <criterio>"+
			"      <id_criterio>7</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>5</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>4</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>3</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>2</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>1</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <total_erros>3</total_erros>"+
			"  </recomendacao>"+
			"  <recomendacao>"+
			"    <id_recomendacao>20</id_recomendacao>"+
			"    <criterio>"+
			"      <id_criterio>2</id_criterio>"+
			"      <erros_criterio>17</erros_criterio>"+
			"      <itens_criterio>40</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>1</id_criterio>"+
			"      <erros_criterio>13</erros_criterio>"+
			"      <itens_criterio>40</itens_criterio>"+
			"    </criterio>"+
			"    <total_erros>30</total_erros>"+
			"  </recomendacao>"+
			"  <recomendacao>"+
			"    <id_recomendacao>1</id_recomendacao>"+
			"    <criterio>"+
			"      <id_criterio>6</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>1</id_criterio>"+
			"      <erros_criterio>15</erros_criterio>"+
			"      <itens_criterio>1000</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>4</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>2</id_criterio>"+
			"      <erros_criterio>53</erros_criterio>"+
			"      <itens_criterio>1000</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>5</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>3</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <total_erros>72</total_erros>"+
			"  </recomendacao>"+
			"  <recomendacao>"+
			"    <id_recomendacao>2</id_recomendacao>"+
			"    <criterio>"+
			"      <id_criterio>3</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>2</id_criterio>"+
			"      <erros_criterio>0</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <criterio>"+
			"      <id_criterio>1</id_criterio>"+
			"      <erros_criterio>1</erros_criterio>"+
			"      <itens_criterio>0</itens_criterio>"+
			"    </criterio>"+
			"    <total_erros>1</total_erros>"+
			"  </recomendacao>"+
			"  <qtde_linhas>1000</qtde_linhas>"+
			"</relatorio_de_avaliacao>";

}
