package br.com.eselo.intercepts;

import br.com.caelum.vraptor.InterceptionException;
import br.com.caelum.vraptor.Intercepts;
import br.com.caelum.vraptor.Result;
import br.com.caelum.vraptor.core.InterceptorStack;
import br.com.caelum.vraptor.interceptor.Interceptor;
import br.com.caelum.vraptor.resource.ResourceMethod;
import br.com.caelum.vraptor.view.Results;
import br.com.eselo.annotations.Restrito;
import br.com.eselo.component.UsuarioWeb;
import br.com.eselo.controller.UsuariosController;
import br.com.eselo.dao.GrupoDAO;

@Intercepts
public class AutorizacaoInterceptor implements Interceptor {

	private final UsuarioWeb usuario;
	private final Result result;
	private final GrupoDAO dao;

	public AutorizacaoInterceptor(UsuarioWeb _usuario, Result _result, GrupoDAO dao) {
		this.usuario = _usuario;
		this.result = _result;
		this.dao = dao;
	}

	@Override
	public void intercept(InterceptorStack stack, ResourceMethod method,
			Object resourceInstance) throws InterceptionException {
		Restrito annotation = null;
		
		if (method.getMethod().isAnnotationPresent(Restrito.class)) {
			annotation = method.getMethod().getAnnotation(Restrito.class);
		} else {
			annotation = method.getMethod().getDeclaringClass().getAnnotation(Restrito.class);
		}
		
		if (usuario.isLogado()) {
			boolean hasPermission = this.dao.hasPermission(annotation.descricao(), usuario.getUsuario());
			
			if (!hasPermission) {
				result.use(Results.status()).forbidden("Acesso negado.");
			}
			stack.next(method, resourceInstance);
		} else if (annotation != null) {
			result.redirectTo(UsuariosController.class).loginForm();
		}
//		stack.next(method, resourceInstance);
	}

	public boolean accepts(ResourceMethod method) {
		return  method
                .getMethod() //metodo anotado
                .isAnnotationPresent(Restrito.class)
            || method
                .getResource() //ou recurso anotado
                .getType()
                .isAnnotationPresent(Restrito.class);
	}

}
