package br.com.eselo.model;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;

import br.com.eselo.integracao.CriterioAvaliado;

@Entity
@Table(name = "sel_criterio")
public class Criterio {
	
	public Criterio() {
		this.faixas = new ArrayList<Faixa>();
		this.erro = true;
		this.verificacaoAutomatica=true;
	}
	
	@Id
	@SequenceGenerator(name = "sel_criterio_id_criterio_seq", sequenceName = "sel_criterio_id_criterio_seq")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "sel_criterio_id_criterio_seq")
	@Column(name = "id_criterio", columnDefinition = "serial", unique = true, nullable = false)
	private Long id;
	
	@Column(name = "vl_peso",nullable = false)
	private Integer peso;

	@Column(name = "ds_descricao", nullable = false)
	private String descricao;
	
	@Column(name = "nu_criterio", nullable = false)
	private Long numeroCriterio;
	

	//@OneToMany(mappedBy = "criterio", targetEntity = br.com.eselo.model.ResultadoCriterio.class, fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	//private List<ResultadoCriterio> resultadosCriterio;
	
	
	public Long getNumeroCriterio() {
		return numeroCriterio;
	}
	
	public void setNumeroCriterio(Long numeroCriterio) {
		this.numeroCriterio = numeroCriterio;
	}



	@ManyToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "id_tipo_teste", columnDefinition = "integer", nullable = false, insertable = true, updatable = true)
	@ForeignKey(name = "fk_sel_criterio_sel_tipo_teste")
	private TipoTeste tipoTeste;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "id_recomendacao", columnDefinition = "integer", nullable = false, insertable = true, updatable = true)
	@ForeignKey(name = "fk_sel_criterio_sel_recomendacao")
	private Recomendacao recomendacao;
	
	@OneToMany(mappedBy = "criterio", targetEntity = br.com.eselo.model.Faixa.class, fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@OrderBy("faixaInicio desc, faixaFim desc")
	private List<Faixa> faixas;

	@Column(name = "lo_verificacao_automatica", nullable = false)
	private boolean verificacaoAutomatica;

	@Column(name = "lo_erro", nullable = false)
	private boolean erro;
	
	@Column(name = "ds_requisito", nullable = true)
	private String requisito;
	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Integer getPeso() {
		return peso;
	}

	public void setPeso(Integer peso) {
		this.peso = peso;
	}

	public String getDescricao() {
		return descricao;
	}

	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}

	public TipoTeste getTipoTeste() {
		return tipoTeste;
	}

	public void setTipoTeste(TipoTeste tipoTeste) {
		this.tipoTeste = tipoTeste;
	}

	public Recomendacao getRecomendacao() {
		return recomendacao;
	}

	public void setRecomendacao(Recomendacao recomendacao) {
		this.recomendacao = recomendacao;
	}

	public List<Faixa> getFaixas() {
		return faixas;
	}

	public void setFaixas(List<Faixa> faixas) {
		this.faixas = faixas;
	}

	public void addFaixa(Faixa faixa){
		this.faixas.add(faixa);
	}
	
	public void removeFaixa(Faixa faixa){
		this.faixas.remove(faixa);
	}
	
	public boolean isVerificacaoAutomatica() {
		return verificacaoAutomatica;
	}

	public void setVerificacaoAutomatica(boolean verificacaoAutomatica) {
		this.verificacaoAutomatica = verificacaoAutomatica;
	}

	public boolean isErro() {
		return erro;
	}

	public void setErro(boolean erro) {
		this.erro = erro;
	}

	public String getRequisito() {
		return requisito;
	}

	public void setRequisito(String requisito) {
		this.requisito = requisito;
	}

	public double getResultadoTeste(CriterioAvaliado criterioAvaliado){
		double percentualErros = criterioAvaliado.getPercentualErros();
		double resultado = 0.0;

		if(this.tipoTeste.getId().intValue() == CriterioAvaliado.TIPO_PROPORCIONAL){
			resultado = (1 - percentualErros) * this.getPeso();
		}else if(this.tipoTeste.getId().intValue() == CriterioAvaliado.TIPO_DESCRESCENTE){
			for(Faixa faixa: this.getFaixas()){
				if((criterioAvaliado.getPercentualErros() >=faixa.getFaixaInicio()) && (criterioAvaliado.getPercentualErros() <=faixa.getFaixaFim())){
					resultado = (faixa.getPercentualConformidade() * this.getPeso());
					break;
				}
			}
		}else if(this.tipoTeste.getId().intValue() == CriterioAvaliado.TIPO_VERDADEIRO){
			if(criterioAvaliado.getQtdeErros()>0){
			  resultado = 0.0;
			}else{
			  resultado = 1.0 * this.getPeso();				
			}
		}else if(this.tipoTeste.getId().intValue() == CriterioAvaliado.TIPO_FALSO){
			if(criterioAvaliado.getQtdeErros()<1){
				resultado = 1.0 * this.getPeso();
			}else{
				resultado = 0.0;
			}
		}
		return resultado;
	}
}
