package br.com.eselo.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.hibernate.annotations.ForeignKey;

import br.com.eselo.component.Util;

@Entity
@Table(name = "sel_parametro_calculo")
public class ParametroCalculo {

	public ParametroCalculo() {
		this.recomendacoes = new ArrayList<Recomendacao>();
	}

	@Id
	@SequenceGenerator(name = "sel_parametro_calculo_id_parametro_calculo_seq", sequenceName = "sel_parametro_calculo_id_parametro_calculo_seq")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "sel_parametro_calculo_id_parametro_calculo_seq")
	@Column(name = "id_parametro_calculo", columnDefinition = "serial", unique = true, nullable = false)
	private Long id;

	@Column(name = "ts_inicio", columnDefinition = "timestamp without time zone", nullable = true)
	@Temporal(TemporalType.TIMESTAMP)
	private Date tsInicio;

	@Column(name = "ts_fim", columnDefinition = "timestamp without time zone", nullable = true)
	@Temporal(TemporalType.TIMESTAMP)
	private Date tsFim;

	@Column(name = "ds_descricao", nullable = false)
	private String descricao;

	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "id_usuario", insertable = true, updatable = true)
	@ForeignKey(name = "fk_sel_parametro_calculo_sel_usuario")
	@Fetch(FetchMode.SELECT)
	private Usuario usuario;

	@OneToMany(mappedBy = "parametroCalculo", targetEntity = br.com.eselo.model.Recomendacao.class, fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@OrderBy("numeroRecomendacao asc")
	private List<Recomendacao> recomendacoes;

	@OneToMany(mappedBy = "parametroCalculo", targetEntity = br.com.eselo.model.Avaliacao.class, fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	private List<Avaliacao> avaliacoes;

	public List<Avaliacao> getAvaliacoes() {
		return avaliacoes;
	}

	public void setAvaliacoes(List<Avaliacao> avaliacoes) {
		this.avaliacoes = avaliacoes;
	}

	public void addAvaliacao(Avaliacao avaliacao) {
		this.avaliacoes.add(avaliacao);
	}

	public void removeAvaliacao(Avaliacao avaliacao) {
		this.avaliacoes.remove(avaliacao);
	}

	public List<Recomendacao> getRecomendacoes() {
		return recomendacoes;
	}

	public void setRecomendacoes(List<Recomendacao> recomendacoes) {
		this.recomendacoes = recomendacoes;
	}

	public void addRecomendacao(Recomendacao recomendacao) {
		this.recomendacoes.add(recomendacao);
	}

	public void removeRecomendacao(Recomendacao recomendacao) {
		this.recomendacoes.remove(recomendacao);
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Date getTsInicio() {
		return tsInicio;
	}

	public void setTsInicio(Date tsInicio) {
		this.tsInicio = tsInicio;
	}

	public Date getTsFim() {
		return tsFim;
	}

	public void setTsFim(Date tsFim) {
		this.tsFim = tsFim;
	}

	public Usuario getUsuario() {
		return usuario;
	}

	public void setUsuario(Usuario usuario) {
		this.usuario = usuario;
	}

	public String getDescricao() {
		return descricao;
	}

	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}

	public String getSlug() {
		return Util.toSlug(descricao);
	}

}
