package teste;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

public class WebChecker {

	private static final String HTTP_PROTOCOL_ALLOW_CIRCULAR_REDIRECTS = "http.protocol.allow-circular-redirects";
	private HttpClient client;
	private HttpMethod method;
	private String url;
	private int statusCode;
	private String content;
	
	private WebChecker(HttpClient client,String url) { 
		this.client = client; 
		this.url = url;
	}
	
	public static WebChecker from(String url) {
		
		HttpClient client = new HttpClient();
		client.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(3,false));
		client.getParams().setParameter(HTTP_PROTOCOL_ALLOW_CIRCULAR_REDIRECTS, true); 
		
		return new WebChecker(client,url);
	}
	
	public WebChecker withGetRequest() {
		this.method = new GetMethod(this.url);
		this.method.setRequestHeader("user-agent", "Mozilla/5.0");
		this.method.setFollowRedirects(true);
		return this;
	}
	
	public PostParams withPostRequest(){
		
        return new PostParams(this,new PostMethod(this.url));
        
	}
	
	public WebChecker execute() {
		try {
			
			this.statusCode = client.executeMethod(this.method);
			this.content = method.getResponseBodyAsString();
			
		} catch (HttpException e) {
		      System.err.println("Fatal protocol violation: " + e.getMessage());
		      e.printStackTrace();
		} catch (IOException e) {
		      System.err.println("Fatal transport error: " + e.getMessage());
		      e.printStackTrace();
		} finally {
		
		      method.releaseConnection();
		} 
		return this;
		
	}
	
	public String getContent() { return content; }
	public int getStatusCode() { return statusCode; }
	
	public String getParsedContent() {
		this.content = this.content.replaceAll("<", "&lt;");
		this.content = this.content.replaceAll(">", "&gt;");
		this.content = this.content.replaceAll(" ", "&nbsp");
		return content;
	}
	
	private void setMethod(HttpMethod method){
		this.method = method;
	}
	private void setRequestHeader(String key, String value){
		this.method.setRequestHeader(key,value);
	}
	private class PostParams{
		private  List<NameValuePair> data = new ArrayList<NameValuePair>();
		private WebChecker webChecker;
		private PostMethod post;
		
		private PostParams(WebChecker webChecker,PostMethod post){
			this.webChecker = webChecker;
			this.post = post;
		}
		
		public PostParams addParam(String key, String value) {
			this.data.add(new NameValuePair(key,value));
			return this;
		}
		
		public WebChecker execute(){
			this.post.setRequestBody(data.toArray(new NameValuePair[data.size()] ));
	        this.webChecker.setMethod(post);
	        this.webChecker.setRequestHeader("user-agent", "Mozilla/5.0");
			this.webChecker.execute();
			return webChecker;
		}
	}

	
	@SuppressWarnings("unused")
	public static void main(String ...arg){
		String local = "http://localhost:18080/e-Selo/calcular-nota";
		String remoto = "https://sistemas-treinamento.ifbaiano.edu.br/eselo/calcular-nota";
		
		WebChecker pagina = WebChecker.from(local).withPostRequest()
				
				//Relatorio de Avaliacao
				.addParam("relatorioAvaliacao.date", new Date().toString())
				.addParam("relatorioAvaliacao.url", "www.teste.com.br")
				.addParam("relatorioAvaliacao.qtdeLinhas", "1000")
				
					//Recomendacao
					.addParam("relatorioAvaliacao.recomendacoes[0].idRecomendacao", "1")
				
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[0].idCriterio","1")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[0].qtdeErros","15")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[0].qtdeItens","1000")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[1].idCriterio","2")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[1].qtdeErros","53")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[1].qtdeItens","1000")
		
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[2].idCriterio","3")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[2].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[2].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[3].idCriterio","4")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[3].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[3].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[4].idCriterio","5")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[4].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[4].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[5].idCriterio","6")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[5].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[0].criterios[5].qtdeItens","0")
				
					.addParam("relatorioAvaliacao.recomendacoes[0].totalErros","72")
					
					//Recomendacao
					.addParam("relatorioAvaliacao.recomendacoes[1].idRecomendacao", "2")
				
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[0].idCriterio","1")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[0].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[0].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[1].idCriterio","2")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[1].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[1].qtdeItens","0")
		
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[2].idCriterio","3")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[2].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[1].criterios[2].qtdeItens","0")
						
					.addParam("relatorioAvaliacao.recomendacoes[1].totalErros","1")
					
					
					
					
					
					//Recomendacao
					.addParam("relatorioAvaliacao.recomendacoes[2].idRecomendacao", "3")
				
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[0].idCriterio","1")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[0].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[0].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[1].idCriterio","2")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[1].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[1].qtdeItens","0")
		
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[2].idCriterio","3")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[2].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[2].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[3].idCriterio","4")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[3].qtdeErros","0")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[3].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[4].idCriterio","5")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[4].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[4].qtdeItens","0")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[5].idCriterio","7")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[5].qtdeErros","1")
						.addParam("relatorioAvaliacao.recomendacoes[2].criterios[5].qtdeItens","0")
				
					.addParam("relatorioAvaliacao.recomendacoes[2].totalErros","3")
					
					//Recomendacao
					.addParam("relatorioAvaliacao.recomendacoes[3].idRecomendacao", "20")
				
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[0].idCriterio","1")
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[0].qtdeErros","13")
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[0].qtdeItens","40")
						
						//Critério
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[1].idCriterio","2")
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[1].qtdeErros","17")
						.addParam("relatorioAvaliacao.recomendacoes[3].criterios[1].qtdeItens","40")
		
					.addParam("relatorioAvaliacao.recomendacoes[3].totalErros","30")

				.execute();

		System.out.println("Resultado esperado: 47,33%");
		System.out.println(pagina.getContent());

	}
	
}